@@ -18,28 +18,29 @@
 #include <drm/drm_panel.h>
 #include <linux/clk.h>
 #include <linux/component.h>
+#include <linux/irq.h>
 #include <linux/of.h>
-#include <linux/of_device.h>
 #include <linux/of_platform.h>
 #include <linux/of_graph.h>
-#include <linux/irq.h>
-#include <video/mipi_display.h>
 #include <linux/phy/phy.h>
 #include <linux/platform_device.h>
+#include <video/mipi_display.h>
 #include <video/videomode.h>
-#include <linux/regmap.h>
-#include <linux/mfd/syscon.h>
 
 #include "mtk_drm_ddp_comp.h"
+#include "mtk_drm_crtc.h"
 
 #define DSI_START		0x00
 
 #define DSI_INTEN		0x08
 
 #define DSI_INTSTA		0x0c
-#define LPRX_RD_RDY_INT_FLAG	BIT(0)
-#define CMD_DONE_INT_FLAG	BIT(1)
-#define DSI_BUSY		BIT(31)
+#define LPRX_RD_RDY_INT_FLAG		BIT(0)
+#define CMD_DONE_INT_FLAG		BIT(1)
+#define TE_RDY_INT_FLAG			BIT(2)
+#define VM_DONE_INT_FLAG		BIT(3)
+#define EXT_TE_RDY_INT_FLAG		BIT(4)
+#define DSI_BUSY			BIT(31)
 
 #define DSI_CON_CTRL		0x10
 #define DSI_RESET			BIT(0)
@@ -69,7 +70,7 @@
 #define DSI_PS_WC			0x3fff
 #define DSI_PS_SEL			(3 << 16)
 #define PACKED_PS_16BIT_RGB565		(0 << 16)
-#define LOOSELY_PS_18BIT_RGB666		BIT(16)
+#define LOOSELY_PS_18BIT_RGB666		(1 << 16)
 #define PACKED_PS_18BIT_RGB666		(2 << 16)
 #define PACKED_PS_24BIT_RGB888		(3 << 16)
 
@@ -82,7 +83,7 @@
 #define DSI_HFP_WC		0x58
 
 #define DSI_CMDQ_SIZE		0x60
-#define CMDQ_SIZE		0x3f
+#define CMDQ_SIZE			0x3f
 
 #define DSI_HSTX_CKL_WC		0x64
 
@@ -92,9 +93,7 @@
 #define DSI_RX_DATA3		0x80
 
 #define DSI_RACK		0x84
-#define RACK			BIT(0)
-
-#define DSI_MEM_CONTI		0x90
+#define RACK				BIT(0)
 
 #define DSI_PHY_LCCON		0x104
 #define LC_HS_TX_EN			BIT(0)
@@ -108,7 +107,7 @@
 
 #define DSI_PHY_TIMECON0	0x110
 #define LPX				(0xff << 0)
-#define HS_PRPR				(0xff << 8)
+#define HS_PREP				(0xff << 8)
 #define HS_ZERO				(0xff << 16)
 #define HS_TRAIL			(0xff << 24)
 
@@ -124,13 +123,28 @@
 #define CLK_TRAIL			(0xff << 24)
 
 #define DSI_PHY_TIMECON3	0x11c
-#define CLK_HS_PRPR			(0xff << 0)
+#define CLK_HS_PREP			(0xff << 0)
 #define CLK_HS_POST			(0xff << 8)
 #define CLK_HS_EXIT			(0xff << 16)
 
 #define DSI_VM_CMD_CON		0x130
-#define VM_CMD_EN		BIT(0)
-#define TS_VFP_EN		BIT(5)
+#define VM_CMD_EN			BIT(0)
+#define TS_VFP_EN			BIT(5)
+
+#define DSI_CMDQ0		0x180
+#define CONFIG				(0xff << 0)
+#define SHORT_PACKET			0
+#define LONG_PACKET			2
+#define BTA				BIT(2)
+#define DATA_ID				(0xff << 8)
+#define DATA_0				(0xff << 16)
+#define DATA_1				(0xff << 24)
+
+#define T_LPX		5
+#define T_HS_PREP	6
+#define T_HS_TRAIL	8
+#define T_HS_EXIT	7
+#define T_HS_ZERO	10
 
 #define NS_TO_CYCLE(n, c)    ((n) / (c) + (((n) % (c)) ? 1 : 0))
 
@@ -140,33 +154,18 @@
 	(type == MIPI_DSI_GENERIC_READ_REQUEST_2_PARAM) || \
 	(type == MIPI_DSI_DCS_READ))
 
-#define MTK_DSI_HOST_IS_WRITE(type) \
-	((type == MIPI_DSI_GENERIC_SHORT_WRITE_0_PARAM) || \
-	(type == MIPI_DSI_GENERIC_SHORT_WRITE_1_PARAM) || \
-	(type == MIPI_DSI_GENERIC_SHORT_WRITE_2_PARAM) || \
-	(type == MIPI_DSI_DCS_SHORT_WRITE) || \
-	(type == MIPI_DSI_DCS_SHORT_WRITE_PARAM) || \
-	(type == MIPI_DSI_GENERIC_LONG_WRITE) || \
-	(type == MIPI_DSI_DCS_LONG_WRITE))
-
 struct phy;
 
-struct mtk_dsi_driver_data {
-	const u32 reg_cmdq_ofs;
-};
-
 struct mtk_dsi {
 	struct mtk_ddp_comp ddp_comp;
 	struct device *dev;
+	struct mipi_dsi_host host;
 	struct drm_encoder encoder;
 	struct drm_connector conn;
-	struct device_node *panel_node, *device_node;
 	struct drm_panel *panel;
 	struct drm_bridge *bridge;
 	struct phy *phy;
 
-	struct mipi_dsi_host host;
-
 	void __iomem *regs;
 
 	struct clk *engine_clk;
@@ -181,51 +180,8 @@ struct mtk_dsi {
 	struct videomode vm;
 	int refcount;
 	bool enabled;
-	int irq_num, irq_data;
-	const struct mtk_dsi_driver_data *driver_data;
-};
-
-struct dsi_cmd_t0 {
-	u8 config;
-	u8 type;
-	u8 data0;
-	u8 data1;
-};
-
-struct dsi_cmd_t2 {
-	u8 config;
-	u8 type;
-	u16 wc16;
-	u8 *pdata;
-};
-
-struct dsi_rx_data {
-	u8 byte0;
-	u8 byte1;
-	u8 byte2;
-	u8 byte3;
-};
-
-struct dsi_tx_cmdq {
-	u8 byte0;
-	u8 byte1;
-	u8 byte2;
-	u8 byte3;
-};
-
-struct dsi_tx_cmdq_regs {
-	struct dsi_tx_cmdq data[128];
-};
-
-enum {
-	DSI_INT_SLEEPOUT_DONE_FLAG	= BIT(6),
-	DSI_INT_VM_CMD_DONE_FLAG	= BIT(5),
-	DSI_INT_EXT_TE_RDY_FLAG		= BIT(4),
-	DSI_INT_VM_DONE_FLAG		= BIT(3),
-	DSI_INT_TE_RDY_FLAG		= BIT(2),
-	DSI_INT_CMD_DONE_FLAG		= BIT(1),
-	DSI_INT_LPRX_RD_RDY_FLAG	= BIT(0),
-	DSI_INT_ALL_BITS		= (0x7f)
+	u32 irq_data;
+	wait_queue_head_t irq_wait_queue;
 };
 
 static inline struct mtk_dsi *encoder_to_dsi(struct drm_encoder *e)
@@ -250,30 +206,21 @@ static void mtk_dsi_mask(struct mtk_dsi *dsi, u32 offset, u32 mask, u32 data)
 	writel((temp & ~mask) | (data & mask), dsi->regs + offset);
 }
 
-static void mtk_dsi_set_cmdq(void __iomem *reg, u32 mask, u32 data)
-{
-	u32 temp = readl(reg);
-
-	writel((temp & ~mask) | (data & mask), reg);
-}
-
 static void mtk_dsi_phy_timconfig(struct mtk_dsi *dsi)
 {
 	u32 timcon0, timcon1, timcon2, timcon3;
 	u32 ui, cycle_time;
-	u32 lpx;
 
 	ui = 1000 / dsi->data_rate + 0x01;
 	cycle_time = 8000 / dsi->data_rate + 0x01;
-	lpx = 5;
 
-	timcon0 = (8 << 24) | (0xa << 16) | (0x6 << 8) | lpx;
-	timcon1 = (7 << 24) | (5 * lpx << 16) | ((3 * lpx) / 2) << 8 |
-		  (4 * lpx);
+	timcon0 = T_LPX | T_HS_PREP << 8 | T_HS_ZERO << 16 | T_HS_TRAIL << 24;
+	timcon1 = 4 * T_LPX | (3 * T_LPX / 2) << 8 | 5 * T_LPX << 16 |
+		  T_HS_EXIT << 24;
 	timcon2 = ((NS_TO_CYCLE(0x64, cycle_time) + 0xa) << 24) |
 		  (NS_TO_CYCLE(0x150, cycle_time) << 16);
-	timcon3 = (2 * lpx) << 16 | NS_TO_CYCLE(80 + 52 * ui, cycle_time) << 8 |
-		   NS_TO_CYCLE(0x40, cycle_time);
+	timcon3 = NS_TO_CYCLE(0x40, cycle_time) | (2 * T_LPX) << 16 |
+		  NS_TO_CYCLE(80 + 52 * ui, cycle_time) << 8;
 
 	writel(timcon0, dsi->regs + DSI_PHY_TIMECON0);
 	writel(timcon1, dsi->regs + DSI_PHY_TIMECON1);
@@ -297,59 +244,10 @@ static void mtk_dsi_reset_engine(struct mtk_dsi *dsi)
 	mtk_dsi_mask(dsi, DSI_CON_CTRL, DSI_RESET, 0);
 }
 
-static int mtk_dsi_poweron(struct mtk_dsi *dsi)
-{
-	struct device *dev = dsi->dev;
-	int ret;
-
-	if (++dsi->refcount != 1)
-		return 0;
-	/**
-	 * data_rate = (pixel_clock / 1000) * pixel_dipth * mipi_ratio;
-	 * pixel_clock unit is Khz, data_rata unit is MHz, so need divide 1000.
-	 * mipi_ratio is mipi clk coefficient for balance the pixel clk in mipi.
-	 * we set mipi_ratio is 1.05.
-	 */
-
-	dsi->data_rate = dsi->vm.pixelclock * 12 * 21;
-	dsi->data_rate /= (dsi->lanes * 1000 * 10);
-	dev_info(dev, "set mipitx's data rate: %dMHz\n", dsi->data_rate);
-
-	ret = clk_set_rate(dsi->hs_clk, dsi->data_rate * 1000000);
-	if (ret < 0) {
-		dev_err(dev, "Failed to set data rate: %d\n", ret);
-		goto err_refcount;
-	}
-
-	ret = phy_power_on(dsi->phy);
-
-	ret = clk_prepare_enable(dsi->engine_clk);
-	if (ret < 0) {
-		dev_err(dev, "Failed to enable engine clock: %d\n", ret);
-		goto err_phy_power_off;
-	}
-
-	ret = clk_prepare_enable(dsi->digital_clk);
-	if (ret < 0) {
-		dev_err(dev, "Failed to enable digital clock: %d\n", ret);
-		goto err_disable_engine_clk;
-	}
-
-	return ret;
-
-err_disable_engine_clk:
-	clk_disable_unprepare(dsi->engine_clk);
-err_phy_power_off:
-	phy_power_off(dsi->phy);
-err_refcount:
-	dsi->refcount--;
-	return ret;
-}
-
 static void mtk_dsi_clk_ulp_mode_enter(struct mtk_dsi *dsi)
 {
 	mtk_dsi_mask(dsi, DSI_PHY_LCCON, LC_HS_TX_EN, 0);
-	mtk_dsi_mask(dsi, DSI_PHY_LCCON, LC_ULPM_EN, LC_ULPM_EN);
+	mtk_dsi_mask(dsi, DSI_PHY_LCCON, LC_ULPM_EN, 0);
 }
 
 static void mtk_dsi_clk_ulp_mode_leave(struct mtk_dsi *dsi)
@@ -362,7 +260,7 @@ static void mtk_dsi_clk_ulp_mode_leave(struct mtk_dsi *dsi)
 static void mtk_dsi_lane0_ulp_mode_enter(struct mtk_dsi *dsi)
 {
 	mtk_dsi_mask(dsi, DSI_PHY_LD0CON, LD0_HS_TX_EN, 0);
-	mtk_dsi_mask(dsi, DSI_PHY_LD0CON, LD0_ULPM_EN, LD0_ULPM_EN);
+	mtk_dsi_mask(dsi, DSI_PHY_LD0CON, LD0_ULPM_EN, 0);
 }
 
 static void mtk_dsi_lane0_ulp_mode_leave(struct mtk_dsi *dsi)
@@ -393,11 +291,10 @@ static void mtk_dsi_set_mode(struct mtk_dsi *dsi)
 	u32 vid_mode = CMD_MODE;
 
 	if (dsi->mode_flags & MIPI_DSI_MODE_VIDEO) {
-		vid_mode = SYNC_PULSE_MODE;
-
-		if ((dsi->mode_flags & MIPI_DSI_MODE_VIDEO_BURST) &&
-		    !(dsi->mode_flags & MIPI_DSI_MODE_VIDEO_SYNC_PULSE))
+		if (dsi->mode_flags & MIPI_DSI_MODE_VIDEO_BURST)
 			vid_mode = BURST_MODE;
+		else if (dsi->mode_flags & MIPI_DSI_MODE_VIDEO_SYNC_PULSE)
+			vid_mode = SYNC_PULSE_MODE;
 		else
 			vid_mode = SYNC_EVENT_MODE;
 	}
@@ -407,8 +304,6 @@ static void mtk_dsi_set_mode(struct mtk_dsi *dsi)
 
 static void mtk_dsi_set_vm_cmd(struct mtk_dsi *dsi)
 {
-	writel(0x3c, dsi->regs + DSI_MEM_CONTI);
-
 	mtk_dsi_mask(dsi, DSI_VM_CMD_CON, VM_CMD_EN, VM_CMD_EN);
 	mtk_dsi_mask(dsi, DSI_VM_CMD_CON, TS_VFP_EN, TS_VFP_EN);
 }
@@ -544,16 +439,6 @@ static void mtk_dsi_config_vdo_timing(struct mtk_dsi *dsi)
 	mtk_dsi_ps_control(dsi);
 }
 
-static void mtk_dsi_set_interrupt_enable(struct mtk_dsi *dsi)
-{
-	u32 inten = DSI_INT_ALL_BITS;
-
-	if (dsi->mode_flags & MIPI_DSI_MODE_VIDEO)
-		inten &= ~(DSI_INT_TE_RDY_FLAG | DSI_INT_EXT_TE_RDY_FLAG);
-
-	writel(inten, dsi->regs + DSI_INTEN);
-}
-
 static void mtk_dsi_start(struct mtk_dsi *dsi)
 {
 	writel(0, dsi->regs + DSI_START);
@@ -567,124 +452,177 @@ static void mtk_dsi_stop(struct mtk_dsi *dsi)
 
 static void mtk_dsi_set_cmd_mode(struct mtk_dsi *dsi)
 {
-	u32 tmp_reg1;
-
-	tmp_reg1 = CMD_MODE;
-	writel(tmp_reg1, dsi->regs + DSI_MODE_CTRL);
+	writel(CMD_MODE, dsi->regs + DSI_MODE_CTRL);
 }
 
-void mtk_dsi_irq_wakeup(struct mtk_dsi *dsi, u32 irq_bit)
+static void mtk_dsi_set_interrupt_enable(struct mtk_dsi *dsi)
+{
+	u32 inten = LPRX_RD_RDY_INT_FLAG | CMD_DONE_INT_FLAG | VM_DONE_INT_FLAG;
+
+	writel(inten, dsi->regs + DSI_INTEN);
+}
+
+static void mtk_dsi_irq_data_set(struct mtk_dsi *dsi, u32 irq_bit)
 {
 	dsi->irq_data |= irq_bit;
 }
 
+static void mtk_dsi_irq_data_clear(struct mtk_dsi *dsi, u32 irq_bit)
+{
+	dsi->irq_data &= ~irq_bit;
+}
+
+static s32 mtk_dsi_wait_for_irq_done(struct mtk_dsi *dsi, u32 irq_flag,
+				     unsigned int timeout)
+{
+	s32 ret = 0;
+	unsigned long jiffies = msecs_to_jiffies(timeout);
+
+	ret = wait_event_interruptible_timeout(dsi->irq_wait_queue,
+					       dsi->irq_data & irq_flag,
+					       jiffies);
+	if (ret == 0) {
+		DRM_WARN("Wait DSI IRQ(0x%08x) Timeout\n", irq_flag);
+
+		mtk_dsi_enable(dsi);
+		mtk_dsi_reset_engine(dsi);
+	}
+
+	return ret;
+}
+
 static irqreturn_t mtk_dsi_irq(int irq, void *dev_id)
 {
 	struct mtk_dsi *dsi = dev_id;
-
 	u32 status, tmp;
+	u32 flag = LPRX_RD_RDY_INT_FLAG | CMD_DONE_INT_FLAG | VM_DONE_INT_FLAG;
 
-	status = readl(dsi->regs + DSI_INTSTA);
+	status = readl(dsi->regs + DSI_INTSTA) & flag;
 
-	if (status & DSI_INT_LPRX_RD_RDY_FLAG) {
-		/* write clear RD_RDY interrupt */
-		/* write clear RD_RDY interrupt must be before DSI_RACK */
-		/* because CMD_DONE will raise after DSI_RACK, */
-		/* so write clear RD_RDY after that will clear CMD_DONE too */
+	if (status) {
 		do {
-			/* send read ACK */
 			mtk_dsi_mask(dsi, DSI_RACK, RACK, RACK);
 			tmp = readl(dsi->regs + DSI_INTSTA);
 		} while (tmp & DSI_BUSY);
 
-		mtk_dsi_mask(dsi, DSI_INTSTA, DSI_INT_LPRX_RD_RDY_FLAG, 0);
-		mtk_dsi_irq_wakeup(dsi, DSI_INT_LPRX_RD_RDY_FLAG);
-	}
-
-	if (status & DSI_INT_CMD_DONE_FLAG) {
-		mtk_dsi_mask(dsi, DSI_INTSTA, DSI_INT_CMD_DONE_FLAG, 0);
-		mtk_dsi_irq_wakeup(dsi, DSI_INT_CMD_DONE_FLAG);
-	}
-
-	if (status & DSI_INT_TE_RDY_FLAG) {
-		mtk_dsi_mask(dsi, DSI_INTSTA, DSI_INT_TE_RDY_FLAG, 0);
-		mtk_dsi_irq_wakeup(dsi, DSI_INT_TE_RDY_FLAG);
-	}
-
-	if (status & DSI_INT_VM_DONE_FLAG) {
-		mtk_dsi_mask(dsi, DSI_INTSTA, DSI_INT_VM_DONE_FLAG, 0);
-		mtk_dsi_irq_wakeup(dsi, DSI_INT_VM_DONE_FLAG);
-	}
-
-	if (status & DSI_INT_EXT_TE_RDY_FLAG) {
-		mtk_dsi_mask(dsi, DSI_INTSTA, DSI_INT_EXT_TE_RDY_FLAG, 0);
-		mtk_dsi_irq_wakeup(dsi, DSI_INT_EXT_TE_RDY_FLAG);
-	}
-
-	if (status & DSI_INT_VM_CMD_DONE_FLAG) {
-		mtk_dsi_mask(dsi, DSI_INTSTA, DSI_INT_VM_CMD_DONE_FLAG, 0);
-		mtk_dsi_irq_wakeup(dsi, DSI_INT_VM_CMD_DONE_FLAG);
-	}
-
-	if (status & DSI_INT_SLEEPOUT_DONE_FLAG) {
-		mtk_dsi_mask(dsi, DSI_INTSTA, DSI_INT_SLEEPOUT_DONE_FLAG, 0);
-		mtk_dsi_irq_wakeup(dsi, DSI_INT_SLEEPOUT_DONE_FLAG);
+		mtk_dsi_mask(dsi, DSI_INTSTA, status, 0);
+		mtk_dsi_irq_data_set(dsi, status);
+		wake_up_interruptible(&dsi->irq_wait_queue);
 	}
 
 	return IRQ_HANDLED;
 }
 
-static void mtk_dsi_wait_for_idle(struct mtk_dsi *dsi)
+static s32 mtk_dsi_switch_to_cmd_mode(struct mtk_dsi *dsi, u8 irq_flag, u32 t)
 {
-	u32 timeout_ms = 500000; /* total 1s ~ 2s timeout */
-
-	while (timeout_ms--) {
-		if (!(readl(dsi->regs + DSI_INTSTA) & DSI_BUSY))
-			break;
-
-		usleep_range(2, 4);
-	}
-
-	if (timeout_ms == 0) {
-		dev_info(dsi->dev, "polling dsi wait not busy timeout!\n");
-
-		mtk_dsi_enable(dsi);
-		mtk_dsi_reset_engine(dsi);
-	}
-}
-
-static s32 mtk_dsi_wait_for_irq_timeout(struct mtk_dsi *dsi, u32 irq_bit,
-					u32 timeout_ms)
-{
-	while (timeout_ms--) {
-		if (dsi->irq_data & irq_bit) {
-			dsi->irq_data &= ~irq_bit;
-			return 0;
-		}
-
-		usleep_range(1000, 1100);
-	}
-
-	dsi->irq_data = 0;
-
-	return -1;
-}
-
-static void mtk_dsi_switch_to_cmd_mode(struct mtk_dsi *dsi)
-{
-	s32 ret = 0;
-
+	mtk_dsi_irq_data_clear(dsi, irq_flag);
 	mtk_dsi_set_cmd_mode(dsi);
 
-	ret = mtk_dsi_wait_for_irq_timeout(dsi, DSI_INT_VM_DONE_FLAG, 500);
-	if (ret != 0) {
-		dev_info(dsi->dev, "dsi wait engine idle timeout\n");
-
-		mtk_dsi_enable(dsi);
-		mtk_dsi_reset_engine(dsi);
+	if (!mtk_dsi_wait_for_irq_done(dsi, irq_flag, t)) {
+		DRM_ERROR("failed to switch cmd mode\n");
+		return -ETIME;
+	} else {
+		return 0;
 	}
 }
 
+static int mtk_dsi_poweron(struct mtk_dsi *dsi)
+{
+	struct device *dev = dsi->dev;
+	int ret;
+	u64 pixel_clock, total_bits;
+	u32 htotal, htotal_bits, bit_per_pixel, overhead_cycles, overhead_bits;
+
+	if (++dsi->refcount != 1)
+		return 0;
+
+	switch (dsi->format) {
+	case MIPI_DSI_FMT_RGB565:
+		bit_per_pixel = 16;
+		break;
+	case MIPI_DSI_FMT_RGB666_PACKED:
+		bit_per_pixel = 18;
+		break;
+	case MIPI_DSI_FMT_RGB666:
+	case MIPI_DSI_FMT_RGB888:
+	default:
+		bit_per_pixel = 24;
+		break;
+	}
+
+	/**
+	 * vm.pixelclock is in kHz, pixel_clock unit is Hz, so multiply by 1000
+	 * htotal_time = htotal * byte_per_pixel / num_lanes
+	 * overhead_time = lpx + hs_prepare + hs_zero + hs_trail + hs_exit
+	 * mipi_ratio = (htotal_time + overhead_time) / htotal_time
+	 * data_rate = pixel_clock * bit_per_pixel * mipi_ratio / num_lanes;
+	 */
+	pixel_clock = dsi->vm.pixelclock * 1000;
+	htotal = dsi->vm.hactive + dsi->vm.hback_porch + dsi->vm.hfront_porch +
+			dsi->vm.hsync_len;
+	htotal_bits = htotal * bit_per_pixel;
+
+	overhead_cycles = T_LPX + T_HS_PREP + T_HS_ZERO + T_HS_TRAIL +
+			T_HS_EXIT;
+	overhead_bits = overhead_cycles * dsi->lanes * 8;
+	total_bits = htotal_bits + overhead_bits;
+
+	dsi->data_rate = DIV_ROUND_UP_ULL(pixel_clock * total_bits,
+					  htotal * dsi->lanes);
+
+	ret = clk_set_rate(dsi->hs_clk, dsi->data_rate);
+	if (ret < 0) {
+		dev_err(dev, "Failed to set data rate: %d\n", ret);
+		goto err_refcount;
+	}
+
+	phy_power_on(dsi->phy);
+
+	ret = clk_prepare_enable(dsi->engine_clk);
+	if (ret < 0) {
+		dev_err(dev, "Failed to enable engine clock: %d\n", ret);
+		goto err_phy_power_off;
+	}
+
+	ret = clk_prepare_enable(dsi->digital_clk);
+	if (ret < 0) {
+		dev_err(dev, "Failed to enable digital clock: %d\n", ret);
+		goto err_disable_engine_clk;
+	}
+
+	mtk_dsi_enable(dsi);
+	mtk_dsi_reset_engine(dsi);
+	mtk_dsi_phy_timconfig(dsi);
+
+	mtk_dsi_rxtx_control(dsi);
+	mtk_dsi_ps_control_vact(dsi);
+	mtk_dsi_set_vm_cmd(dsi);
+	mtk_dsi_config_vdo_timing(dsi);
+	mtk_dsi_set_interrupt_enable(dsi);
+
+	mtk_dsi_clk_ulp_mode_leave(dsi);
+	mtk_dsi_lane0_ulp_mode_leave(dsi);
+	mtk_dsi_clk_hs_mode(dsi, 0);
+
+	if (dsi->panel) {
+		if (drm_panel_prepare(dsi->panel)) {
+			DRM_ERROR("failed to prepare the panel\n");
+			goto err_disable_digital_clk;
+		}
+	}
+
+	return 0;
+err_disable_digital_clk:
+	clk_disable_unprepare(dsi->digital_clk);
+err_disable_engine_clk:
+	clk_disable_unprepare(dsi->engine_clk);
+err_phy_power_off:
+	phy_power_off(dsi->phy);
+err_refcount:
+	dsi->refcount--;
+	return ret;
+}
+
 static void mtk_dsi_poweroff(struct mtk_dsi *dsi)
 {
 	if (WARN_ON(dsi->refcount == 0))
@@ -693,17 +631,12 @@ static void mtk_dsi_poweroff(struct mtk_dsi *dsi)
 	if (--dsi->refcount != 0)
 		return;
 
-	mtk_dsi_switch_to_cmd_mode(dsi);
-
-	if (dsi->panel) {
-		if (drm_panel_unprepare(dsi->panel)) {
-			DRM_ERROR("failed to prepare the panel\n");
-			return;
-		}
-
-		if (drm_panel_disable(dsi->panel)) {
-			DRM_ERROR("failed to disable the panel\n");
-			return;
+	if (!mtk_dsi_switch_to_cmd_mode(dsi, VM_DONE_INT_FLAG, 500)) {
+		if (dsi->panel) {
+			if (drm_panel_unprepare(dsi->panel)) {
+				DRM_ERROR("failed to unprepare the panel\n");
+				return;
+			}
 		}
 	}
 
@@ -732,37 +665,24 @@ static void mtk_output_dsi_enable(struct mtk_dsi *dsi)
 		return;
 	}
 
-	usleep_range(20000, 21000);
+	mtk_dsi_set_mode(dsi);
+	mtk_dsi_clk_hs_mode(dsi, 1);
 
-	mtk_dsi_rxtx_control(dsi);
-	mtk_dsi_phy_timconfig(dsi);
-	mtk_dsi_ps_control_vact(dsi);
-	mtk_dsi_set_vm_cmd(dsi);
-	mtk_dsi_config_vdo_timing(dsi);
-	mtk_dsi_set_interrupt_enable(dsi);
-
-	mtk_dsi_enable(dsi);
-	mtk_dsi_clk_ulp_mode_leave(dsi);
-	mtk_dsi_lane0_ulp_mode_leave(dsi);
-	mtk_dsi_clk_hs_mode(dsi, 0);
+	mtk_dsi_start(dsi);
 
 	if (dsi->panel) {
-		if (drm_panel_prepare(dsi->panel)) {
-			DRM_ERROR("failed to prepare the panel\n");
-			return;
-		}
-
 		if (drm_panel_enable(dsi->panel)) {
-			DRM_ERROR("failed to prepare the panel\n");
-			return;
+			DRM_ERROR("failed to enable the panel\n");
+			goto err_dsi_power_off;
 		}
 	}
 
-	mtk_dsi_set_mode(dsi);
-	mtk_dsi_clk_hs_mode(dsi, 1);
-	mtk_dsi_start(dsi);
-
 	dsi->enabled = true;
+
+	return;
+err_dsi_power_off:
+	mtk_dsi_stop(dsi);
+	mtk_dsi_poweroff(dsi);
 }
 
 static void mtk_output_dsi_disable(struct mtk_dsi *dsi)
@@ -770,8 +690,14 @@ static void mtk_output_dsi_disable(struct mtk_dsi *dsi)
 	if (!dsi->enabled)
 		return;
 
-	mtk_dsi_stop(dsi);
+	if (dsi->panel) {
+		if (drm_panel_disable(dsi->panel)) {
+			DRM_ERROR("failed to disable the panel\n");
+			return;
+		}
+	}
 
+	mtk_dsi_stop(dsi);
 	mtk_dsi_poweroff(dsi);
 
 	dsi->enabled = false;
@@ -828,12 +754,7 @@ static void mtk_dsi_encoder_enable(struct drm_encoder *encoder)
 static enum drm_connector_status mtk_dsi_connector_detect(
 	struct drm_connector *connector, bool force)
 {
-	struct mtk_dsi *dsi = connector_to_dsi(connector);
-
-	if (dsi->panel)
-		return connector_status_connected;
-
-	return connector_status_disconnected;
+	return connector_status_connected;
 }
 
 static int mtk_dsi_connector_get_modes(struct drm_connector *connector)
@@ -843,12 +764,15 @@ static int mtk_dsi_connector_get_modes(struct drm_connector *connector)
 	return drm_panel_get_modes(dsi->panel);
 }
 
-static struct drm_encoder *mtk_dsi_connector_best_encoder(
-		struct drm_connector *connector)
+static int mtk_dsi_atomic_check(struct drm_encoder *encoder,
+				struct drm_crtc_state *crtc_state,
+				struct drm_connector_state *conn_state)
 {
-	struct mtk_dsi *dsi = connector_to_dsi(connector);
+	struct mtk_drm_crtc *mtk_crtc = container_of(conn_state->crtc,
+						     struct mtk_drm_crtc, base);
+	mtk_crtc->bpc = conn_state->connector->display_info.bpc;
 
-	return &dsi->encoder;
+	return 0;
 }
 
 static const struct drm_encoder_helper_funcs mtk_dsi_encoder_helper_funcs = {
@@ -856,6 +780,7 @@ static const struct drm_encoder_helper_funcs mtk_dsi_encoder_helper_funcs = {
 	.mode_set = mtk_dsi_encoder_mode_set,
 	.disable = mtk_dsi_encoder_disable,
 	.enable = mtk_dsi_encoder_enable,
+	.atomic_check = mtk_dsi_atomic_check,
 };
 
 static const struct drm_connector_funcs mtk_dsi_connector_funcs = {
@@ -871,7 +796,6 @@ static const struct drm_connector_funcs mtk_dsi_connector_funcs = {
 static const struct drm_connector_helper_funcs
 	mtk_dsi_connector_helper_funcs = {
 	.get_modes = mtk_dsi_connector_get_modes,
-	.best_encoder = mtk_dsi_connector_best_encoder,
 };
 
 static int mtk_drm_attach_bridge(struct drm_bridge *bridge,
@@ -884,7 +808,6 @@ static int mtk_drm_attach_bridge(struct drm_bridge *bridge,
 
 	encoder->bridge = bridge;
 	bridge->encoder = encoder;
-
 	ret = drm_bridge_attach(encoder->dev, bridge);
 	if (ret) {
 		DRM_ERROR("Failed to attach bridge to drm\n");
@@ -938,7 +861,11 @@ static int mtk_dsi_create_conn_enc(struct drm_device *drm, struct mtk_dsi *dsi)
 	}
 	drm_encoder_helper_add(&dsi->encoder, &mtk_dsi_encoder_helper_funcs);
 
-	dsi->encoder.possible_crtcs = BIT(0) | BIT(1);
+	/*
+	 * Currently display data paths are statically assigned to a crtc each.
+	 * crtc 0 is OVL0 -> COLOR0 -> AAL -> OD -> RDMA0 -> UFOE -> DSI0
+	 */
+	dsi->encoder.possible_crtcs = 1;
 
 	/* If there's a bridge, attach to it and let it create the connector */
 	ret = mtk_drm_attach_bridge(dsi->bridge, &dsi->encoder);
@@ -953,7 +880,6 @@ static int mtk_dsi_create_conn_enc(struct drm_device *drm, struct mtk_dsi *dsi)
 
 err_encoder_cleanup:
 	drm_encoder_cleanup(&dsi->encoder);
-
 	return ret;
 }
 
@@ -961,10 +887,8 @@ static void mtk_dsi_destroy_conn_enc(struct mtk_dsi *dsi)
 {
 	drm_encoder_cleanup(&dsi->encoder);
 	/* Skip connector cleanup if creation was delegated to the bridge */
-	if (dsi->conn.dev) {
-		drm_connector_unregister(&dsi->conn);
+	if (dsi->conn.dev)
 		drm_connector_cleanup(&dsi->conn);
-	}
 }
 
 static void mtk_dsi_ddp_start(struct mtk_ddp_comp *comp)
@@ -994,7 +918,6 @@ static int mtk_dsi_host_attach(struct mipi_dsi_host *host,
 	dsi->lanes = device->lanes;
 	dsi->format = device->format;
 	dsi->mode_flags = device->mode_flags;
-	dsi->panel_node = device->dev.of_node;
 
 	if (dsi->conn.dev)
 		drm_helper_hpd_irq_event(dsi->conn.dev);
@@ -1007,250 +930,149 @@ static int mtk_dsi_host_detach(struct mipi_dsi_host *host,
 {
 	struct mtk_dsi *dsi = host_to_dsi(host);
 
-	dsi->panel_node = NULL;
-
 	if (dsi->conn.dev)
 		drm_helper_hpd_irq_event(dsi->conn.dev);
 
 	return 0;
 }
 
-static ssize_t mtk_dsi_host_read_cmd(struct mtk_dsi *dsi,
-				     const struct mipi_dsi_msg *msg)
+static void mtk_dsi_wait_for_idle(struct mtk_dsi *dsi)
 {
-	u8 max_try_count = 5;
-	u32 recv_data_cnt, tmp_val;
-	u32 recv_data0, recv_data1, recv_data2, recv_data3;
-	struct dsi_rx_data read_data0, read_data1, read_data2, read_data3;
-	struct dsi_cmd_t0 t0;
-	s32 ret;
+	u32 timeout_ms = 500000; /* total 1s ~ 2s timeout */
 
-	u8 *buffer = msg->rx_buf;
-	u8 buffer_size = msg->rx_len;
-	u8 packet_type;
+	while (timeout_ms--) {
+		if (!(readl(dsi->regs + DSI_INTSTA) & DSI_BUSY))
+			break;
 
-	if (readl(dsi->regs + DSI_MODE_CTRL) & 0x03) {
-		dev_info(dsi->dev, "dsi engine is not command mode\n");
-		return -1;
+		usleep_range(2, 4);
 	}
 
-	if (!buffer) {
-		dev_info(dsi->dev, "dsi receive buffer size may be NULL\n");
-		return -1;
+	if (timeout_ms == 0) {
+		DRM_WARN("polling dsi wait not busy timeout!\n");
+
+		mtk_dsi_enable(dsi);
+		mtk_dsi_reset_engine(dsi);
 	}
-
-	do {
-		if (max_try_count == 0) {
-			dev_info(dsi->dev, "dsi engine read counter has been maxinum\n");
-			return -1;
-		}
-
-		max_try_count--;
-		recv_data_cnt = 0;
-
-		mtk_dsi_wait_for_idle(dsi);
-
-		t0.config = 0x04;
-		t0.data0 = *((u8 *)(msg->tx_buf));
-
-		if (buffer_size < 0x3)
-			t0.type = MIPI_DSI_DCS_READ;
-		else
-			t0.type = MIPI_DSI_GENERIC_READ_REQUEST_1_PARAM;
-
-		t0.data1 = 0;
-
-		tmp_val = (t0.data1 << 24) | (t0.data0 << 16) | (t0.type << 8) |
-			t0.config;
-
-		writel(tmp_val, dsi->regs + dsi->driver_data->reg_cmdq_ofs);
-		mtk_dsi_mask(dsi, DSI_CMDQ_SIZE, CMDQ_SIZE, 1);
-
-		mtk_dsi_mask(dsi, DSI_RACK, RACK, RACK);
-		mtk_dsi_mask(dsi, DSI_INTSTA, LPRX_RD_RDY_INT_FLAG,
-			     ~LPRX_RD_RDY_INT_FLAG);
-		mtk_dsi_mask(dsi, DSI_INTSTA, CMD_DONE_INT_FLAG,
-			     ~CMD_DONE_INT_FLAG);
-		mtk_dsi_mask(dsi, DSI_INTEN, DSI_INT_LPRX_RD_RDY_FLAG,
-			     DSI_INT_LPRX_RD_RDY_FLAG);
-		mtk_dsi_mask(dsi, DSI_INTEN, DSI_INT_CMD_DONE_FLAG,
-			     DSI_INT_CMD_DONE_FLAG);
-
-		mtk_dsi_start(dsi);
-
-		dev_info(dsi->dev, "Start polling DSI read ready!!!\n");
-
-		 /* 2s timeout*/
-		ret = mtk_dsi_wait_for_irq_timeout(dsi,
-						   DSI_INT_LPRX_RD_RDY_FLAG,
-						   2000);
-		if (ret) {
-			dev_info(dsi->dev, "Polling DSI read ready timeout!!!\n");
-
-			mtk_dsi_enable(dsi);
-			mtk_dsi_reset_engine(dsi);
-
-			return ret;
-		}
-
-		dev_info(dsi->dev, "End polling DSI read ready!!!\n");
-
-		mtk_dsi_mask(dsi, DSI_RACK, RACK, RACK);
-		mtk_dsi_mask(dsi, DSI_INTSTA, LPRX_RD_RDY_INT_FLAG,
-			     ~LPRX_RD_RDY_INT_FLAG);
-
-		recv_data0 = readl(dsi->regs + DSI_RX_DATA0);
-		recv_data1 = readl(dsi->regs + DSI_RX_DATA1);
-		recv_data2 = readl(dsi->regs + DSI_RX_DATA2);
-		recv_data3 = readl(dsi->regs + DSI_RX_DATA3);
-
-		read_data0 = *((struct dsi_rx_data *)(&recv_data0));
-		read_data1 = *((struct dsi_rx_data *)(&recv_data1));
-		read_data2 = *((struct dsi_rx_data *)(&recv_data2));
-		read_data3 = *((struct dsi_rx_data *)(&recv_data3));
-
-		ret = readl(dsi->regs + DSI_CMDQ_SIZE);
-		dev_info(dsi->dev, "DSI_CMDQ_SIZE : 0x%x\n", ret & CMDQ_SIZE);
-
-		ret = readl(dsi->regs + dsi->driver_data->reg_cmdq_ofs);
-		dev_info(dsi->dev, "DSI_CMDQ_DATA0 : 0x%x\n",
-			 ret & 0xff);
-		dev_info(dsi->dev, "DSI_CMDQ_DATA1 : 0x%x\n",
-			 (ret >> 8) & 0xff);
-		dev_info(dsi->dev, "DSI_CMDQ_DATA2 : 0x%x\n",
-			 (ret >> 16) & 0xff);
-		dev_info(dsi->dev, "DSI_CMDQ_DATA3 : 0x%x\n",
-			 (ret >> 24) & 0xff);
-
-		dev_info(dsi->dev, "DSI_RX_DATA0: 0x%x\n", recv_data0);
-		dev_info(dsi->dev, "DSI_RX_DATA1: 0x%x\n", recv_data1);
-		dev_info(dsi->dev, "DSI_RX_DATA2: 0x%x\n", recv_data2);
-		dev_info(dsi->dev, "DSI_RX_DATA3: 0x%x\n", recv_data3);
-
-		dev_info(dsi->dev, "read_data0: %x,%x,%x,%x\n",
-			 read_data0.byte0, read_data0.byte1, read_data0.byte2,
-			 read_data0.byte3);
-		dev_info(dsi->dev, "read_data1: %x,%x,%x,%x\n",
-			 read_data1.byte0, read_data1.byte1, read_data1.byte2,
-			 read_data1.byte3);
-		dev_info(dsi->dev, "read_data2: %x,%x,%x,%x\n",
-			 read_data2.byte0, read_data2.byte1, read_data2.byte2,
-			 read_data2.byte3);
-		dev_info(dsi->dev, "read_data3: %x,%x,%x,%x\n",
-			 read_data3.byte0, read_data3.byte1, read_data3.byte2,
-			 read_data3.byte3);
-
-		packet_type = read_data0.byte0;
-		dev_info(dsi->dev, "DSI read packet_type is 0x%x\n",
-			 packet_type);
-
-		if (packet_type == 0x1a || packet_type == 0x1c) {
-			void *read_tmp = (void *)&recv_data1;
-
-			recv_data_cnt = read_data0.byte1 +
-					read_data0.byte2 * 16;
-			if (recv_data_cnt > 10)
-				recv_data_cnt = 10;
-
-			if (recv_data_cnt > buffer_size)
-				recv_data_cnt = buffer_size;
-
-			memcpy(buffer, read_tmp, recv_data_cnt);
-		} else {
-			/* short  packet */
-			recv_data_cnt = 2;
-			if (recv_data_cnt > buffer_size)
-				recv_data_cnt = buffer_size;
-
-			memcpy(buffer, &read_data0.byte1, 2);
-		}
-	} while (packet_type != 0x1c && packet_type != 0x21 &&
-		packet_type != 0x22 && packet_type != 0x1a);
-
-	dev_info(dsi->dev, "dsi get %d byte data from the panel address(0x%x)\n",
-		 recv_data_cnt, *((u8 *)(msg->tx_buf)));
-
-	return recv_data_cnt;
 }
 
-static ssize_t mtk_dsi_host_write_cmd(struct mtk_dsi *dsi,
-				      const struct mipi_dsi_msg *msg)
+static u32 mtk_dsi_recv_cnt(u8 type, u8 *read_data)
 {
-	u32 i;
-	u32 goto_addr, mask_para, set_para, reg_val;
-	void __iomem *cmdq_reg;
-	struct dsi_cmd_t0 t0;
-	struct dsi_cmd_t2 t2;
-	const char *tx_buf = msg->tx_buf;
-	struct dsi_tx_cmdq_regs *dsi_cmd_reg;
-
-	dsi_cmd_reg = (struct dsi_tx_cmdq_regs *)
-		      (dsi->regs + dsi->driver_data->reg_cmdq_ofs);
-
-	mtk_dsi_wait_for_idle(dsi);
-
-	if (msg->tx_len > 2) {
-		t2.config = 2;
-		t2.type = msg->type;
-		t2.wc16 = msg->tx_len;
-
-		reg_val = (t2.wc16 << 16) | (t2.type << 8) | t2.config;
-
-		writel(reg_val, (void __iomem *) &dsi_cmd_reg->data[0]);
-
-		goto_addr = (u32)(&dsi_cmd_reg->data[1].byte0);
-		mask_para = (0xff << ((goto_addr & 0x3) * 8));
-		set_para = (tx_buf[0] << ((goto_addr & 0x3) * 8));
-		cmdq_reg = (void __iomem *)(goto_addr & (~0x3));
-		mtk_dsi_set_cmdq(cmdq_reg, mask_para, set_para);
-
-		for (i = 1; i < msg->tx_len; i++) {
-			goto_addr = (u32)(&dsi_cmd_reg->data[1].byte1) + i - 1;
-			mask_para = (0xff << ((goto_addr & 0x3) * 8));
-			set_para = (tx_buf[i] << ((goto_addr & 0x3) * 8));
-			cmdq_reg = (void __iomem *)(goto_addr & (~0x3));
-			mtk_dsi_set_cmdq(cmdq_reg, mask_para, set_para);
-		}
-
-		mtk_dsi_mask(dsi, DSI_CMDQ_SIZE, CMDQ_SIZE,
-			     2 + (msg->tx_len - 1) / 4);
-	} else {
-		t0.config = 0;
-		t0.data0 = tx_buf[0];
-		if (msg->tx_len == 2) {
-			t0.type = MIPI_DSI_DCS_SHORT_WRITE_PARAM;
-			t0.data1 = tx_buf[1];
-		} else {
-			t0.type = MIPI_DSI_DCS_SHORT_WRITE;
-			t0.data1 = 0;
-		}
-
-		reg_val = (t0.data1 << 24) | (t0.data0 << 16) | (t0.type << 8) |
-			   t0.config;
-
-		writel(reg_val, (void __iomem *) &dsi_cmd_reg->data[0]);
-		mtk_dsi_mask(dsi, DSI_CMDQ_SIZE, CMDQ_SIZE, 1);
+	switch (type) {
+	case MIPI_DSI_RX_GENERIC_SHORT_READ_RESPONSE_1BYTE:
+	case MIPI_DSI_RX_DCS_SHORT_READ_RESPONSE_1BYTE:
+		return 1;
+	case MIPI_DSI_RX_GENERIC_SHORT_READ_RESPONSE_2BYTE:
+	case MIPI_DSI_RX_DCS_SHORT_READ_RESPONSE_2BYTE:
+		return 2;
+	case MIPI_DSI_RX_GENERIC_LONG_READ_RESPONSE:
+	case MIPI_DSI_RX_DCS_LONG_READ_RESPONSE:
+		return read_data[1] + read_data[2] * 16;
+	case MIPI_DSI_RX_ACKNOWLEDGE_AND_ERROR_REPORT:
+		DRM_INFO("type is 0x02, try again\n");
+		break;
+	default:
+		DRM_INFO("type(0x%x) cannot be non-recognite\n", type);
+		break;
 	}
 
-	mtk_dsi_start(dsi);
-	mtk_dsi_wait_for_idle(dsi);
-
 	return 0;
 }
 
+static void mtk_dsi_cmdq(struct mtk_dsi *dsi, const struct mipi_dsi_msg *msg)
+{
+	const char *tx_buf = msg->tx_buf;
+	u8 config, cmdq_size, cmdq_off, type = msg->type;
+	u32 reg_val, cmdq_mask, i;
+
+	if (MTK_DSI_HOST_IS_READ(type))
+		config = BTA;
+	else
+		config = (msg->tx_len > 2) ? LONG_PACKET : SHORT_PACKET;
+
+	if (msg->tx_len > 2) {
+		cmdq_size = 1 + (msg->tx_len + 3) / 4;
+		cmdq_off = 4;
+		cmdq_mask = CONFIG | DATA_ID | DATA_0 | DATA_1;
+		reg_val = (msg->tx_len << 16) | (type << 8) | config;
+	} else {
+		cmdq_size = 1;
+		cmdq_off = 2;
+		cmdq_mask = CONFIG | DATA_ID;
+		reg_val = (type << 8) | config;
+	}
+
+	for (i = 0; i < msg->tx_len; i++)
+		writeb(tx_buf[i], dsi->regs + DSI_CMDQ0 + cmdq_off + i);
+
+	mtk_dsi_mask(dsi, DSI_CMDQ0, cmdq_mask, reg_val);
+	mtk_dsi_mask(dsi, DSI_CMDQ_SIZE, CMDQ_SIZE, cmdq_size);
+}
+
+static ssize_t mtk_dsi_host_send_cmd(struct mtk_dsi *dsi,
+				     const struct mipi_dsi_msg *msg, u8 flag)
+{
+	mtk_dsi_wait_for_idle(dsi);
+	mtk_dsi_irq_data_clear(dsi, flag);
+	mtk_dsi_cmdq(dsi, msg);
+	mtk_dsi_start(dsi);
+
+	if (!mtk_dsi_wait_for_irq_done(dsi, flag, 2000))
+		return -ETIME;
+	else
+		return 0;
+}
+
 static ssize_t mtk_dsi_host_transfer(struct mipi_dsi_host *host,
 				     const struct mipi_dsi_msg *msg)
 {
 	struct mtk_dsi *dsi = host_to_dsi(host);
-	u8 type = msg->type;
-	ssize_t ret = 0;
+	u32 recv_cnt, i;
+	u8 read_data[16];
+	void *src_addr;
+	u8 irq_flag = CMD_DONE_INT_FLAG;
 
-	if (MTK_DSI_HOST_IS_READ(type))
-		ret = mtk_dsi_host_read_cmd(dsi, msg);
-	else if (MTK_DSI_HOST_IS_WRITE(type))
-		ret = mtk_dsi_host_write_cmd(dsi, msg);
+	if (readl(dsi->regs + DSI_MODE_CTRL) & MODE) {
+		DRM_ERROR("dsi engine is not command mode\n");
+		return -EINVAL;
+	}
 
-	return ret;
+	if (MTK_DSI_HOST_IS_READ(msg->type))
+		irq_flag |= LPRX_RD_RDY_INT_FLAG;
+
+	if (mtk_dsi_host_send_cmd(dsi, msg, irq_flag) < 0)
+		return -ETIME;
+
+	if (!MTK_DSI_HOST_IS_READ(msg->type))
+		return 0;
+
+	if (!msg->rx_buf) {
+		DRM_ERROR("dsi receive buffer size may be NULL\n");
+		return -EINVAL;
+	}
+
+	for (i = 0; i < 16; i++)
+		*(read_data + i) = readb(dsi->regs + DSI_RX_DATA0 + i);
+
+	recv_cnt = mtk_dsi_recv_cnt(read_data[0], read_data);
+
+	if (recv_cnt > 2)
+		src_addr = &read_data[4];
+	else
+		src_addr = &read_data[1];
+
+	if (recv_cnt > 10)
+		recv_cnt = 10;
+
+	if (recv_cnt > msg->rx_len)
+		recv_cnt = msg->rx_len;
+
+	if (recv_cnt)
+		memcpy(msg->rx_buf, src_addr, recv_cnt);
+
+	DRM_INFO("dsi get %d byte data from the panel address(0x%x)\n",
+		 recv_cnt, *((u8 *)(msg->tx_buf)));
+
+	return recv_cnt;
 }
 
 static const struct mipi_dsi_host_ops mtk_dsi_ops = {
@@ -1272,6 +1094,12 @@ static int mtk_dsi_bind(struct device *dev, struct device *master, void *data)
 		return ret;
 	}
 
+	ret = mipi_dsi_host_register(&dsi->host);
+	if (ret < 0) {
+		dev_err(dev, "failed to register DSI host: %d\n", ret);
+		goto err_ddp_comp_unregister;
+	}
+
 	ret = mtk_dsi_create_conn_enc(drm, dsi);
 	if (ret) {
 		DRM_ERROR("Encoder create failed with %d\n", ret);
@@ -1281,6 +1109,8 @@ static int mtk_dsi_bind(struct device *dev, struct device *master, void *data)
 	return 0;
 
 err_unregister:
+	mipi_dsi_host_unregister(&dsi->host);
+err_ddp_comp_unregister:
 	mtk_ddp_comp_unregister(drm, &dsi->ddp_comp);
 	return ret;
 }
@@ -1307,6 +1137,7 @@ static int mtk_dsi_probe(struct platform_device *pdev)
 	struct device *dev = &pdev->dev;
 	struct device_node *remote_node, *endpoint;
 	struct resource *regs;
+	int irq_num;
 	int comp_id;
 	int ret;
 
@@ -1314,13 +1145,25 @@ static int mtk_dsi_probe(struct platform_device *pdev)
 	if (!dsi)
 		return -ENOMEM;
 
-	dsi->mode_flags = MIPI_DSI_MODE_VIDEO;
-	dsi->format = MIPI_DSI_FMT_RGB888;
-	dsi->lanes = 4;
-	dsi->dev = dev;
+	dsi->host.ops = &mtk_dsi_ops;
+	dsi->host.dev = dev;
 
-	dsi->driver_data = (const struct mtk_dsi_driver_data *)
-			    of_device_get_match_data(&pdev->dev);
+	endpoint = of_graph_get_next_endpoint(dev->of_node, NULL);
+	if (endpoint) {
+		remote_node = of_graph_get_remote_port_parent(endpoint);
+		if (!remote_node) {
+			dev_err(dev, "No panel connected\n");
+			return -ENODEV;
+		}
+
+		dsi->bridge = of_drm_find_bridge(remote_node);
+		dsi->panel = of_drm_find_panel(remote_node);
+		of_node_put(remote_node);
+		if (!dsi->bridge && !dsi->panel) {
+			dev_info(dev, "Waiting for bridge or panel driver\n");
+			return -EPROBE_DEFER;
+		}
+	}
 
 	dsi->engine_clk = devm_clk_get(dev, "engine");
 	if (IS_ERR(dsi->engine_clk)) {
@@ -1336,6 +1179,13 @@ static int mtk_dsi_probe(struct platform_device *pdev)
 		return ret;
 	}
 
+	dsi->hs_clk = devm_clk_get(dev, "hs");
+	if (IS_ERR(dsi->hs_clk)) {
+		ret = PTR_ERR(dsi->hs_clk);
+		dev_err(dev, "Failed to get hs clock: %d\n", ret);
+		return ret;
+	}
+
 	regs = platform_get_resource(pdev, IORESOURCE_MEM, 0);
 	dsi->regs = devm_ioremap_resource(dev, regs);
 	if (IS_ERR(dsi->regs)) {
@@ -1347,47 +1197,10 @@ static int mtk_dsi_probe(struct platform_device *pdev)
 	dsi->phy = devm_phy_get(dev, "dphy");
 	if (IS_ERR(dsi->phy)) {
 		ret = PTR_ERR(dsi->phy);
-		dev_err(dev, "not found mipi-dphy: %d\n", ret);
+		dev_err(dev, "Failed to get MIPI-DPHY: %d\n", ret);
 		return ret;
 	}
 
-	dsi->hs_clk = devm_clk_get(dev, "hs");
-	if (IS_ERR(dsi->hs_clk)) {
-		dev_err(dev, "Failed to get hs clock\n");
-		return PTR_ERR(dsi->hs_clk);
-	}
-
-	endpoint = of_graph_get_next_endpoint(dev->of_node, NULL);
-	if (endpoint) {
-		remote_node = of_graph_get_remote_port_parent(endpoint);
-		if (!remote_node) {
-			dev_err(dev, "No panel connected\n");
-			return -ENODEV;
-		}
-
-		dsi->device_node = remote_node;
-	}
-
-	dsi->host.ops = &mtk_dsi_ops;
-	dsi->host.dev = dev;
-	ret = mipi_dsi_host_register(&dsi->host);
-	if (ret < 0) {
-		dev_err(dev, "failed to register DSI host: %d\n", ret);
-		return -EPROBE_DEFER;
-	}
-
-	dsi->bridge = of_drm_find_bridge(dsi->device_node);
-	dsi->panel = of_drm_find_panel(dsi->device_node);
-	of_node_put(remote_node);
-
-	dev_info(dev, "dsi connector detect panel = 0x%p, bridge = 0x%p\n",
-		 dsi->panel, dsi->bridge);
-
-	if (!dsi->bridge && !dsi->panel) {
-		dev_info(dev, "Waiting for bridge or panel driver\n");
-		return -EPROBE_DEFER;
-	}
-
 	comp_id = mtk_ddp_comp_get_id(dev->of_node, MTK_DSI);
 	if (comp_id < 0) {
 		dev_err(dev, "Failed to identify by alias: %d\n", comp_id);
@@ -1401,23 +1214,21 @@ static int mtk_dsi_probe(struct platform_device *pdev)
 		return ret;
 	}
 
-	dsi->irq_num = platform_get_irq(pdev, 0);
-	if (dsi->irq_num < 0) {
+	irq_num = platform_get_irq(pdev, 0);
+	if (irq_num < 0) {
 		dev_err(&pdev->dev, "failed to request dsi irq resource\n");
-		ret = dsi->irq_num;
 		return -EPROBE_DEFER;
 	}
 
-	irq_set_status_flags(dsi->irq_num, IRQ_TYPE_LEVEL_LOW);
-	ret = devm_request_irq(&pdev->dev, dsi->irq_num, mtk_dsi_irq,
+	irq_set_status_flags(irq_num, IRQ_TYPE_LEVEL_LOW);
+	ret = devm_request_irq(&pdev->dev, irq_num, mtk_dsi_irq,
 			       IRQF_TRIGGER_LOW, dev_name(&pdev->dev), dsi);
 	if (ret) {
 		dev_err(&pdev->dev, "failed to request mediatek dsi irq\n");
 		return -EPROBE_DEFER;
 	}
 
-	dsi->irq_data = 0;
-	dev_info(dev, "dsi irq num is 0x%x\n", dsi->irq_num);
+	init_waitqueue_head(&dsi->irq_wait_queue);
 
 	platform_set_drvdata(pdev, dsi);
 
@@ -1434,19 +1245,9 @@ static int mtk_dsi_remove(struct platform_device *pdev)
 	return 0;
 }
 
-static const struct mtk_dsi_driver_data mt8173_dsi_driver_data = {
-	.reg_cmdq_ofs = 0x200,
-};
-
-static const struct mtk_dsi_driver_data mt2701_dsi_driver_data = {
-	.reg_cmdq_ofs = 0x180,
-};
-
 static const struct of_device_id mtk_dsi_of_match[] = {
-	{ .compatible = "mediatek,mt2701-dsi",
-	  .data = &mt2701_dsi_driver_data },
-	{ .compatible = "mediatek,mt8173-dsi",
-	  .data = &mt8173_dsi_driver_data },
+	{ .compatible = "mediatek,mt2701-dsi" },
+	{ .compatible = "mediatek,mt8173-dsi" },
 	{ },
 };
 
