@@ -16,6 +16,7 @@
 #include <drm/drm_crtc.h>
 #include <drm/drm_crtc_helper.h>
 #include <drm/drm_edid.h>
+#include <linux/arm-smccc.h>
 #include <linux/clk.h>
 #include <linux/delay.h>
 #include <linux/hdmi.h>
@@ -35,16 +36,6 @@
 #include "mtk_hdmi.h"
 #include "mtk_hdmi_regs.h"
 
-#if (defined(CONFIG_MTK_IN_HOUSE_TEE_SUPPORT) || \
-	defined(CONFIG_TRUSTY))
-#include "trustzone/kree/system.h"
-#include "trustzone/tz_cross/trustzone.h"
-#include "trustzone/tz_cross/hdmi_ta.h"
-#define HDMI_TEE_ENABLE 1
-#else
-#define HDMI_TEE_ENABLE 0
-#endif
-
 #define NCTS_BYTES	7
 
 enum mtk_hdmi_clk_id {
@@ -240,6 +231,17 @@ static void mtk_hdmi_hw_vid_black(struct mtk_hdmi *hdmi, bool black)
 
 static void mtk_hdmi_hw_make_reg_writable(struct mtk_hdmi *hdmi, bool enable)
 {
+	struct arm_smccc_res res;
+
+	/*
+	 * MT8173 HDMI hardware has an output control bit to enable/disable HDMI
+	 * output. This bit can only be controlled in ARM supervisor mode.
+	 * The ARM trusted firmware provides an API for the HDMI driver to set
+	 * this control bit to enable HDMI output in supervisor mode.
+	 */
+	arm_smccc_smc(MTK_SIP_SET_AUTHORIZED_SECURE_REG, 0x14000904, 0x80000000,
+		      0, 0, 0, 0, 0, &res);
+
 	regmap_update_bits(hdmi->sys_regmap, hdmi->sys_offset + HDMI_SYS_CFG20,
 			   HDMI_PCLK_FREE_RUN, enable ? HDMI_PCLK_FREE_RUN : 0);
 	regmap_update_bits(hdmi->sys_regmap, hdmi->sys_offset + HDMI_SYS_CFG1C,
@@ -332,6 +334,8 @@ static void mtk_hdmi_hw_send_info_frame(struct mtk_hdmi *hdmi, u8 *buffer,
 		ctrl_frame_en = VS_EN;
 		ctrl_reg = GRL_ACP_ISRC_CTRL;
 		break;
+	default:
+		return;
 	}
 	mtk_hdmi_clear_bits(hdmi, ctrl_reg, ctrl_frame_en);
 	mtk_hdmi_write(hdmi, GRL_INFOFRM_TYPE, frame_type);
@@ -1059,7 +1063,7 @@ static int mtk_hdmi_setup_vendor_specific_infoframe(struct mtk_hdmi *hdmi,
 	}
 
 	err = hdmi_vendor_infoframe_pack(&frame, buffer, sizeof(buffer));
-	if (err) {
+	if (err < 0) {
 		dev_err(hdmi->dev, "Failed to pack vendor infoframe: %zd\n",
 			err);
 		return err;
@@ -1084,20 +1088,20 @@ static int mtk_hdmi_output_init(struct mtk_hdmi *hdmi)
 	return 0;
 }
 
-void mtk_hdmi_audio_enable(struct mtk_hdmi *hdmi)
+static void mtk_hdmi_audio_enable(struct mtk_hdmi *hdmi)
 {
 	mtk_hdmi_aud_enable_packet(hdmi, true);
 	hdmi->audio_enable = true;
 }
 
-void mtk_hdmi_audio_disable(struct mtk_hdmi *hdmi)
+static void mtk_hdmi_audio_disable(struct mtk_hdmi *hdmi)
 {
 	mtk_hdmi_aud_enable_packet(hdmi, false);
 	hdmi->audio_enable = false;
 }
 
-int mtk_hdmi_audio_set_param(struct mtk_hdmi *hdmi,
-			     struct hdmi_audio_param *param)
+static int mtk_hdmi_audio_set_param(struct mtk_hdmi *hdmi,
+				    struct hdmi_audio_param *param)
 {
 	if (!hdmi->audio_enable) {
 		dev_err(hdmi->dev, "hdmi audio is in disable state!\n");
@@ -1130,6 +1134,7 @@ static int mtk_hdmi_output_set_display_mode(struct mtk_hdmi *hdmi,
 
 	phy_power_on(hdmi->phy);
 	mtk_hdmi_aud_output_config(hdmi, mode);
+
 	mtk_hdmi_hw_vid_black(hdmi, false);
 	mtk_hdmi_hw_aud_unmute(hdmi);
 	mtk_hdmi_hw_send_av_unmute(hdmi);
@@ -1200,19 +1205,6 @@ static void hdmi_conn_destroy(struct drm_connector *conn)
 	drm_connector_cleanup(conn);
 }
 
-static int mtk_hdmi_get_phy_addr(struct mtk_hdmi *hdmi, u8 *edid, int len)
-{
-	int err;
-
-	err = mtk_cec_get_phy_addr(hdmi->cec_dev, edid, len);
-	if (err < 0) {
-		dev_err(hdmi->dev, "get cec physical address fail\n");
-		return err;
-	};
-
-	return 0;
-}
-
 static int mtk_hdmi_conn_get_modes(struct drm_connector *conn)
 {
 	struct mtk_hdmi *hdmi = hdmi_ctx_from_conn(conn);
@@ -1227,12 +1219,12 @@ static int mtk_hdmi_conn_get_modes(struct drm_connector *conn)
 		return -ENODEV;
 
 	hdmi->dvi_mode = !drm_detect_monitor_audio(edid);
+
 	drm_mode_connector_update_edid_property(conn, edid);
 
 	ret = drm_add_edid_modes(conn, edid);
 	drm_edid_to_eld(conn, edid);
 	kfree(edid);
-
 	return ret;
 }
 
@@ -1289,47 +1281,11 @@ static const struct drm_connector_helper_funcs
 static void mtk_hdmi_hpd_event(bool hpd, struct device *dev)
 {
 	struct mtk_hdmi *hdmi = dev_get_drvdata(dev);
-	struct edid *edid = NULL;
-	int ret;
-
-	if (hpd) {
-		edid = drm_get_edid(&hdmi->conn, hdmi->ddc_adpt);
-		if (edid != NULL) {
-			ret = mtk_hdmi_get_phy_addr(hdmi, (u8 *)edid,
-				EDID_LENGTH * (1 + (edid->extensions)));
-			if (ret < 0)
-				dev_err(hdmi->dev, "Get physical address fail\n");
-		}
-	}
 
 	if (hdmi && hdmi->bridge.encoder && hdmi->bridge.encoder->dev)
 		drm_helper_hpd_irq_event(hdmi->bridge.encoder->dev);
 }
 
-#if HDMI_TEE_ENABLE
-static int mtk_hdmi_tee_call(uint32_t cmd)
-{
-	TZ_RESULT tz_ret = TZ_RESULT_SUCCESS;
-	KREE_SESSION_HANDLE hdmi_session;
-
-	tz_ret = KREE_CreateSession(TZ_TA_HDMI_UUID, &hdmi_session);
-	if (tz_ret != TZ_RESULT_SUCCESS) {
-		pr_err("Create hdmi_session Error: %d\n", tz_ret);
-		return tz_ret;
-	}
-
-	tz_ret = KREE_TeeServiceCall(hdmi_session, cmd, 0, NULL);
-	if (tz_ret != TZ_RESULT_SUCCESS)
-		pr_err("KREE_TeeServiceCall error, ret = %x\n", tz_ret);
-
-	tz_ret = KREE_CloseSession(hdmi_session);
-	if (tz_ret != TZ_RESULT_SUCCESS)
-		pr_err("KREE_CloseSession error, ret = %x\n", tz_ret);
-
-	return tz_ret;
-}
-#endif
-
 /*
  * Bridge callbacks
  */
@@ -1349,7 +1305,7 @@ static int mtk_hdmi_bridge_attach(struct drm_bridge *bridge)
 	drm_connector_helper_add(&hdmi->conn, &mtk_hdmi_connector_helper_funcs);
 
 	hdmi->conn.polled = DRM_CONNECTOR_POLL_HPD;
-	hdmi->conn.interlace_allowed = false;
+	hdmi->conn.interlace_allowed = true;
 	hdmi->conn.doublescan_allowed = false;
 
 	ret = drm_mode_connector_attach_encoder(&hdmi->conn,
@@ -1437,14 +1393,12 @@ static void mtk_hdmi_bridge_pre_enable(struct drm_bridge *bridge)
 
 	mtk_hdmi_hw_make_reg_writable(hdmi, true);
 	mtk_hdmi_hw_1p4_version_enable(hdmi, true);
-#if HDMI_TEE_ENABLE
-	mtk_hdmi_tee_call(HDMI_TA_RESUME);
-#endif
 
 	hdmi->powered = true;
 }
 
-static void mtk_hdmi_send_infoframe(struct mtk_hdmi *hdmi, struct drm_display_mode *mode)
+static void mtk_hdmi_send_infoframe(struct mtk_hdmi *hdmi,
+				    struct drm_display_mode *mode)
 {
 	mtk_hdmi_setup_audio_infoframe(hdmi);
 	mtk_hdmi_setup_avi_infoframe(hdmi, mode);
@@ -1457,10 +1411,10 @@ static void mtk_hdmi_bridge_enable(struct drm_bridge *bridge)
 {
 	struct mtk_hdmi *hdmi = hdmi_ctx_from_bridge(bridge);
 
-	phy_power_on(hdmi->phy);
 	mtk_hdmi_output_set_display_mode(hdmi, &hdmi->mode);
 	clk_prepare_enable(hdmi->clk[MTK_HDMI_CLK_HDMI_PLL]);
 	clk_prepare_enable(hdmi->clk[MTK_HDMI_CLK_HDMI_PIXEL]);
+	phy_power_on(hdmi->phy);
 	mtk_hdmi_send_infoframe(hdmi, &hdmi->mode);
 
 	hdmi->enabled = true;
@@ -1489,9 +1443,7 @@ static int mtk_hdmi_dt_parse_pdata(struct mtk_hdmi *hdmi,
 
 	ret = mtk_hdmi_get_all_clk(hdmi, np);
 	if (ret) {
-		if (ret != -EPROBE_DEFER) {
-			dev_err(dev, "Failed to get clocks: %d\n", ret);
-		}
+		dev_err(dev, "Failed to get clocks: %d\n", ret);
 		return ret;
 	}
 
@@ -1537,12 +1489,8 @@ static int mtk_hdmi_dt_parse_pdata(struct mtk_hdmi *hdmi,
 
 	port = of_graph_get_port_by_id(np, 1);
 	if (!port) {
-		i2c_np = of_parse_phandle(np, "ddc-i2c-bus", 0);
-		if (!i2c_np) {
-			dev_err(dev, "Failed to find ddc-i2c-bus node\n");
-			return -EINVAL;
-		}
-		goto find_ddc_adpt;
+		dev_err(dev, "Missing output port node\n");
+		return -EINVAL;
 	}
 
 	ep = of_get_child_by_name(port, "endpoint");
@@ -1581,7 +1529,6 @@ static int mtk_hdmi_dt_parse_pdata(struct mtk_hdmi *hdmi,
 	}
 	of_node_put(remote);
 
-find_ddc_adpt:
 	hdmi->ddc_adpt = of_find_i2c_adapter_by_node(i2c_np);
 	if (!hdmi->ddc_adpt) {
 		dev_err(dev, "Failed to get ddc i2c adapter by node\n");
@@ -1596,16 +1543,15 @@ find_ddc_adpt:
  */
 
 static int mtk_hdmi_audio_hw_params(struct device *dev, void *data,
-		 struct hdmi_codec_daifmt *fmt,
-		 struct hdmi_codec_params *hparms)
+				    struct hdmi_codec_daifmt *daifmt,
+				    struct hdmi_codec_params *params)
 {
 	struct mtk_hdmi *hdmi = dev_get_drvdata(dev);
 	struct hdmi_audio_param hdmi_params;
-	unsigned int chan = hparms->cea.channels;
-	(void)data;
+	unsigned int chan = params->cea.channels;
 
-	dev_dbg(hdmi->dev, "%s: %u Hz, %d bit, %d channels, fmt=%d\n", __func__,
-		hparms->sample_rate, hparms->sample_width, chan, fmt->fmt);
+	dev_dbg(hdmi->dev, "%s: %u Hz, %d bit, %d channels\n", __func__,
+		params->sample_rate, params->sample_width, chan);
 
 	if (!hdmi->bridge.encoder)
 		return -ENODEV;
@@ -1628,7 +1574,7 @@ static int mtk_hdmi_audio_hw_params(struct device *dev, void *data,
 		return -EINVAL;
 	}
 
-	switch (hparms->sample_rate) {
+	switch (params->sample_rate) {
 	case 32000:
 	case 44100:
 	case 48000:
@@ -1639,11 +1585,11 @@ static int mtk_hdmi_audio_hw_params(struct device *dev, void *data,
 		break;
 	default:
 		dev_err(hdmi->dev, "rate[%d] not supported!\n",
-			hparms->sample_rate);
+			params->sample_rate);
 		return -EINVAL;
 	}
 
-	switch (fmt->fmt) {
+	switch (daifmt->fmt) {
 	case HDMI_I2S:
 		hdmi_params.aud_codec = HDMI_AUDIO_CODING_TYPE_PCM;
 		hdmi_params.aud_sampe_size = HDMI_AUDIO_SAMPLE_SIZE_16;
@@ -1651,19 +1597,13 @@ static int mtk_hdmi_audio_hw_params(struct device *dev, void *data,
 		hdmi_params.aud_i2s_fmt = HDMI_I2S_MODE_I2S_24BIT;
 		hdmi_params.aud_mclk = HDMI_AUD_MCLK_128FS;
 		break;
-	case HDMI_SPDIF:
-		hdmi_params.aud_codec = HDMI_AUDIO_CODING_TYPE_PCM;
-		hdmi_params.aud_sampe_size = HDMI_AUDIO_SAMPLE_SIZE_16;
-		hdmi_params.aud_input_type = HDMI_AUD_INPUT_SPDIF;
-		hdmi_params.aud_mclk = HDMI_AUD_MCLK_128FS;
-		break;
 	default:
 		dev_err(hdmi->dev, "%s: Invalid DAI format %d\n", __func__,
-			fmt->fmt);
+			daifmt->fmt);
 		return -EINVAL;
 	}
 
-	memcpy(&hdmi_params.codec_params, hparms,
+	memcpy(&hdmi_params.codec_params, params,
 	       sizeof(hdmi_params.codec_params));
 
 	mtk_hdmi_audio_set_param(hdmi, &hdmi_params);
@@ -1674,7 +1614,6 @@ static int mtk_hdmi_audio_hw_params(struct device *dev, void *data,
 static int mtk_hdmi_audio_startup(struct device *dev, void *data)
 {
 	struct mtk_hdmi *hdmi = dev_get_drvdata(dev);
-	(void)data;
 
 	dev_dbg(dev, "%s\n", __func__);
 
@@ -1686,17 +1625,16 @@ static int mtk_hdmi_audio_startup(struct device *dev, void *data)
 static void mtk_hdmi_audio_shutdown(struct device *dev, void *data)
 {
 	struct mtk_hdmi *hdmi = dev_get_drvdata(dev);
-	(void)data;
 
 	dev_dbg(dev, "%s\n", __func__);
 
 	mtk_hdmi_audio_disable(hdmi);
 }
 
-int mtk_hdmi_audio_digital_mute(struct device *dev, void *data, bool enable)
+static int
+mtk_hdmi_audio_digital_mute(struct device *dev, void *data, bool enable)
 {
 	struct mtk_hdmi *hdmi = dev_get_drvdata(dev);
-	(void)data;
 
 	dev_dbg(dev, "%s(%d)\n", __func__, enable);
 
@@ -1708,12 +1646,9 @@ int mtk_hdmi_audio_digital_mute(struct device *dev, void *data, bool enable)
 	return 0;
 }
 
-static int mtk_hdmi_audio_get_eld(struct device *dev, void *data,
-		   uint8_t *buf, size_t len)
-
+static int mtk_hdmi_audio_get_eld(struct device *dev, void *data, uint8_t *buf, size_t len)
 {
 	struct mtk_hdmi *hdmi = dev_get_drvdata(dev);
-	(void)data;
 
 	dev_dbg(dev, "%s\n", __func__);
 
@@ -1736,17 +1671,16 @@ static void mtk_hdmi_register_audio_driver(struct device *dev)
 		.ops = &mtk_hdmi_audio_codec_ops,
 		.max_i2s_channels = 2,
 		.i2s = 1,
-		.spdif = 1,
 	};
-	static struct platform_device *pdev;
+	struct platform_device *pdev;
 
-	if (pdev == NULL) {
-		pdev = platform_device_register_data(dev, HDMI_CODEC_DRV_NAME,
-			     PLATFORM_DEVID_NONE, &codec_data,
-			     sizeof(codec_data));
-		DRM_INFO("%s driver bound to HDMI\n", HDMI_CODEC_DRV_NAME);
-	}
-	return;
+	pdev = platform_device_register_data(dev, HDMI_CODEC_DRV_NAME,
+					     PLATFORM_DEVID_AUTO, &codec_data,
+					     sizeof(codec_data));
+	if (IS_ERR(pdev))
+		return;
+
+	DRM_INFO("%s driver bound to HDMI\n", HDMI_CODEC_DRV_NAME);
 }
 
 static int mtk_drm_hdmi_probe(struct platform_device *pdev)
@@ -1760,7 +1694,6 @@ static int mtk_drm_hdmi_probe(struct platform_device *pdev)
 		return -ENOMEM;
 
 	hdmi->dev = dev;
-	mtk_hdmi_register_audio_driver(dev);
 
 	ret = mtk_hdmi_dt_parse_pdata(hdmi, pdev);
 	if (ret)
@@ -1781,6 +1714,8 @@ static int mtk_drm_hdmi_probe(struct platform_device *pdev)
 		return ret;
 	}
 
+	mtk_hdmi_register_audio_driver(dev);
+
 	hdmi->bridge.funcs = &mtk_hdmi_bridge_funcs;
 	hdmi->bridge.of_node = pdev->dev.of_node;
 	ret = drm_bridge_add(&hdmi->bridge);
