@@ -154,6 +154,8 @@ struct mtk_i2c {
 	struct clk *clk_pmic;		/* PMIC clock for i2c from PMIC */
 	bool have_pmic;			/* can use i2c pins from PMIC */
 	bool use_push_pull;		/* IO config push-pull mode */
+	bool scp_share;			/* scp share bus */
+	bool scp_irq;			/* irq source flag */
 
 	u16 irq_stat;			/* interrupt status */
 	unsigned int speed_hz;		/* The speed in transfer */
@@ -303,7 +305,7 @@ static void mtk_i2c_init_hw(struct mtk_i2c *i2c)
 		writew(I2C_DCM_DISABLE, i2c->base + OFFSET_DCM_EN);
 
 	if (i2c->dev_comp->timing_adjust)
-		writew(i2c->clk_src_div - 1, i2c->base + OFFSET_CLOCK_DIV);
+		writew(I2C_DEFAULT_CLK_DIV - 1, i2c->base + OFFSET_CLOCK_DIV);
 
 	writew(i2c->timing_reg, i2c->base + OFFSET_TIMING);
 	writew(i2c->high_speed_reg, i2c->base + OFFSET_HS);
@@ -318,7 +320,6 @@ static void mtk_i2c_init_hw(struct mtk_i2c *i2c)
 	writew(I2C_DELAY_LEN, i2c->base + OFFSET_DELAY_LEN);
 
 	writel(I2C_DMA_HARD_RST, i2c->pdmabase + OFFSET_RST);
-	udelay(50);
 	writel(I2C_DMA_CLR_FLAG, i2c->pdmabase + OFFSET_RST);
 }
 
@@ -454,6 +455,8 @@ static int mtk_i2c_do_transfer(struct mtk_i2c *i2c, struct i2c_msg *msgs,
 	u16 control_reg;
 	u16 restart_flag = 0;
 	u32 reg_4g_mode;
+	u8 *dma_rd_buf = NULL;
+	u8 *dma_wr_buf = NULL;
 	dma_addr_t rpaddr = 0;
 	dma_addr_t wpaddr = 0;
 	int ret;
@@ -481,10 +484,7 @@ static int mtk_i2c_do_transfer(struct mtk_i2c *i2c, struct i2c_msg *msgs,
 	else
 		writew(I2C_FS_START_CON, i2c->base + OFFSET_EXT_CONF);
 
-	addr_reg = msgs->addr << 1;
-	if (i2c->op == I2C_MASTER_RD)
-		addr_reg |= 0x1;
-
+	addr_reg = i2c_8bit_addr_from_msg(msgs);
 	writew(addr_reg, i2c->base + OFFSET_SLAVE_ADDR);
 
 	/* Clear interrupt status */
@@ -516,11 +516,20 @@ static int mtk_i2c_do_transfer(struct mtk_i2c *i2c, struct i2c_msg *msgs,
 	if (i2c->op == I2C_MASTER_RD) {
 		writel(I2C_DMA_INT_FLAG_NONE, i2c->pdmabase + OFFSET_INT_FLAG);
 		writel(I2C_DMA_CON_RX, i2c->pdmabase + OFFSET_CON);
-		rpaddr = dma_map_single(i2c->dev, msgs->buf,
-					msgs->len, DMA_FROM_DEVICE);
-		if (dma_mapping_error(i2c->dev, rpaddr))
+
+		dma_rd_buf = i2c_get_dma_safe_msg_buf(msgs, 1);
+		if (!dma_rd_buf)
 			return -ENOMEM;
 
+		rpaddr = dma_map_single(i2c->dev, dma_rd_buf,
+					msgs->len, DMA_FROM_DEVICE);
+
+		if (dma_mapping_error(i2c->dev, rpaddr)) {
+			i2c_put_dma_safe_msg_buf(dma_rd_buf, msgs, false);
+
+			return -ENOMEM;
+		}
+
 		if (i2c->dev_comp->support_33bits) {
 			reg_4g_mode = mtk_i2c_set_4g_mode(rpaddr);
 			writel(reg_4g_mode, i2c->pdmabase + OFFSET_RX_4G_MODE);
@@ -531,11 +540,20 @@ static int mtk_i2c_do_transfer(struct mtk_i2c *i2c, struct i2c_msg *msgs,
 	} else if (i2c->op == I2C_MASTER_WR) {
 		writel(I2C_DMA_INT_FLAG_NONE, i2c->pdmabase + OFFSET_INT_FLAG);
 		writel(I2C_DMA_CON_TX, i2c->pdmabase + OFFSET_CON);
-		wpaddr = dma_map_single(i2c->dev, msgs->buf,
-					msgs->len, DMA_TO_DEVICE);
-		if (dma_mapping_error(i2c->dev, wpaddr))
+
+		dma_wr_buf = i2c_get_dma_safe_msg_buf(msgs, 1);
+		if (!dma_wr_buf)
 			return -ENOMEM;
 
+		wpaddr = dma_map_single(i2c->dev, dma_wr_buf,
+					msgs->len, DMA_TO_DEVICE);
+
+		if (dma_mapping_error(i2c->dev, wpaddr)) {
+			i2c_put_dma_safe_msg_buf(dma_wr_buf, msgs, false);
+
+			return -ENOMEM;
+		}
+
 		if (i2c->dev_comp->support_33bits) {
 			reg_4g_mode = mtk_i2c_set_4g_mode(wpaddr);
 			writel(reg_4g_mode, i2c->pdmabase + OFFSET_TX_4G_MODE);
@@ -546,16 +564,40 @@ static int mtk_i2c_do_transfer(struct mtk_i2c *i2c, struct i2c_msg *msgs,
 	} else {
 		writel(I2C_DMA_CLR_FLAG, i2c->pdmabase + OFFSET_INT_FLAG);
 		writel(I2C_DMA_CLR_FLAG, i2c->pdmabase + OFFSET_CON);
-		wpaddr = dma_map_single(i2c->dev, msgs->buf,
-					msgs->len, DMA_TO_DEVICE);
-		if (dma_mapping_error(i2c->dev, wpaddr))
+
+		dma_wr_buf = i2c_get_dma_safe_msg_buf(msgs, 1);
+		if (!dma_wr_buf)
 			return -ENOMEM;
-		rpaddr = dma_map_single(i2c->dev, (msgs + 1)->buf,
+
+		wpaddr = dma_map_single(i2c->dev, dma_wr_buf,
+				msgs->len, DMA_TO_DEVICE);
+
+		if (dma_mapping_error(i2c->dev, wpaddr)) {
+			i2c_put_dma_safe_msg_buf(dma_wr_buf, msgs, false);
+
+			return -ENOMEM;
+		}
+
+		dma_rd_buf = i2c_get_dma_safe_msg_buf((msgs + 1), 1);
+		if (!dma_rd_buf) {
+			dma_unmap_single(i2c->dev, wpaddr,
+					 msgs->len, DMA_TO_DEVICE);
+
+			i2c_put_dma_safe_msg_buf(dma_wr_buf, msgs, false);
+
+			return -ENOMEM;
+		}
+
+		rpaddr = dma_map_single(i2c->dev, dma_rd_buf,
 					(msgs + 1)->len,
 					DMA_FROM_DEVICE);
 		if (dma_mapping_error(i2c->dev, rpaddr)) {
 			dma_unmap_single(i2c->dev, wpaddr,
 					 msgs->len, DMA_TO_DEVICE);
+
+			i2c_put_dma_safe_msg_buf(dma_wr_buf, msgs, false);
+			i2c_put_dma_safe_msg_buf(dma_rd_buf, (msgs + 1), false);
+
 			return -ENOMEM;
 		}
 
@@ -594,14 +636,21 @@ static int mtk_i2c_do_transfer(struct mtk_i2c *i2c, struct i2c_msg *msgs,
 	if (i2c->op == I2C_MASTER_WR) {
 		dma_unmap_single(i2c->dev, wpaddr,
 				 msgs->len, DMA_TO_DEVICE);
+
+		i2c_put_dma_safe_msg_buf(dma_wr_buf, msgs, true);
 	} else if (i2c->op == I2C_MASTER_RD) {
 		dma_unmap_single(i2c->dev, rpaddr,
 				 msgs->len, DMA_FROM_DEVICE);
+
+		i2c_put_dma_safe_msg_buf(dma_rd_buf, msgs, true);
 	} else {
 		dma_unmap_single(i2c->dev, wpaddr, msgs->len,
 				 DMA_TO_DEVICE);
 		dma_unmap_single(i2c->dev, rpaddr, (msgs + 1)->len,
 				 DMA_FROM_DEVICE);
+
+		i2c_put_dma_safe_msg_buf(dma_wr_buf, msgs, true);
+		i2c_put_dma_safe_msg_buf(dma_rd_buf, (msgs + 1), true);
 	}
 
 	if (ret == 0) {
@@ -621,16 +670,25 @@ static int mtk_i2c_do_transfer(struct mtk_i2c *i2c, struct i2c_msg *msgs,
 	return 0;
 }
 
+
 static int mtk_i2c_transfer(struct i2c_adapter *adap,
 			    struct i2c_msg msgs[], int num)
 {
 	int ret;
 	int left_num = num;
+	int num_cnt;
 	struct mtk_i2c *i2c = i2c_get_adapdata(adap);
 
+	for (num_cnt = 0; num_cnt < num; num_cnt++) {
+		if (!msgs[num_cnt].len)
+			return -EINVAL;
+	}
+
 	ret = mtk_i2c_clock_enable(i2c);
 	if (ret)
-		return ret;
+		goto err_exit_sem;
+
+	mtk_i2c_init_hw(i2c);
 
 	i2c->auto_restart = i2c->dev_comp->auto_restart;
 
@@ -682,6 +740,101 @@ static int mtk_i2c_transfer(struct i2c_adapter *adap,
 
 err_exit:
 	mtk_i2c_clock_disable(i2c);
+err_exit_sem:
+	return ret;
+}
+
+static int mtk_i2c_smbus_transfer(struct i2c_adapter *adap, u16 addr,
+			unsigned short flags, char read_write,
+			u8 command, int size, union i2c_smbus_data *data)
+{
+	int ret = 0;
+	struct i2c_msg msgs[2];
+	union i2c_smbus_data local[2];
+	int num = 1;
+
+	if ( read_write)
+		num++;
+
+	msgs[0].addr  = addr;
+	msgs[0].flags = 0;
+	msgs[0].len   = 1;
+	msgs[0].buf   = local[0].block;
+	local[0].block[0] = command;
+	local[1].block[0] = command;
+
+	/* The data will get returned in this structure */
+	msgs[1].addr  = addr;
+	msgs[1].flags = I2C_M_RD/* | I2C_M_NOSTART*/;
+	msgs[1].len   = 1;
+	msgs[1].buf   = local[1].block;
+
+
+	switch (size) {
+	case I2C_SMBUS_BYTE:
+		/* Fall through */
+	case I2C_SMBUS_QUICK:
+		if ( !read_write) {
+			local[0].block[1] = 1;
+			msgs[0].len++;
+		}
+		break;
+	case I2C_SMBUS_BYTE_DATA:
+		if ( !read_write) {
+			local[0].block[1] = data->byte;
+			msgs[0].len++;
+		}
+		break;
+	case I2C_SMBUS_WORD_DATA:
+		if (!read_write) {
+			local[0].block[1] = data->word & 0xff;
+			local[0].block[2] = (data->word >> 8) & 0xff;
+			msgs[0].len += 2;
+		} else {
+			msgs[1].len = 2;
+		}
+		break;
+	case I2C_SMBUS_I2C_BLOCK_DATA:
+		if (!read_write) {
+			msgs[0].len += data->block[0];
+			memcpy(&local[0].block[1], &data->block[1], data->block[0]);
+		} else {
+			msgs[1].len = data->block[0];
+		}
+		break;
+	default:
+		return -EINVAL;
+	}
+
+	ret = mtk_i2c_transfer(adap, msgs, num);
+	if ( ret == num ) {
+		ret = 0;
+	}
+	if (!ret && read_write ) {
+		switch (size) {
+			case I2C_SMBUS_BYTE:
+				data->byte = msgs[1].buf[0];
+				break;
+			case I2C_SMBUS_QUICK:
+				break;
+			case I2C_SMBUS_BYTE_DATA:
+				data->byte = msgs[1].buf[0];
+				break;
+			case I2C_SMBUS_WORD_DATA:
+				data->word = msgs[1].buf[1] << 8;
+				data->word |= msgs[1].buf[0];
+				break;
+			case I2C_SMBUS_I2C_BLOCK_DATA:
+				data->block[0] = msgs[1].len;
+				memcpy(&data->block[1],
+					msgs[1].buf,
+					msgs[1].len);
+				break;
+			default:
+				return -EINVAL;
+		}
+	}
+
 	return ret;
 }
 
@@ -719,11 +872,15 @@ static irqreturn_t mtk_i2c_irq(int irqno, void *dev_id)
 
 static u32 mtk_i2c_functionality(struct i2c_adapter *adap)
 {
-	return I2C_FUNC_I2C | I2C_FUNC_SMBUS_EMUL;
+	return I2C_FUNC_I2C | I2C_FUNC_SMBUS_QUICK | I2C_FUNC_SMBUS_BYTE |
+	       I2C_FUNC_SMBUS_BYTE_DATA | I2C_FUNC_SMBUS_WORD_DATA |
+	       I2C_FUNC_SMBUS_BLOCK_DATA |
+	       I2C_FUNC_SMBUS_I2C_BLOCK;
 }
 
 static const struct i2c_algorithm mtk_i2c_algorithm = {
 	.master_xfer = mtk_i2c_transfer,
+	.smbus_xfer = mtk_i2c_smbus_transfer,
 	.functionality = mtk_i2c_functionality,
 };
 
@@ -735,15 +892,14 @@ static int mtk_i2c_parse_dt(struct device_node *np, struct mtk_i2c *i2c)
 	if (ret < 0)
 		i2c->speed_hz = I2C_DEFAULT_SPEED;
 
-	if (!i2c->dev_comp->timing_adjust) {
-		ret = of_property_read_u32(np, "clock-div", &i2c->clk_src_div);
-		if (ret < 0)
-			return ret;
+	ret = of_property_read_u32(np, "clock-div", &i2c->clk_src_div);
+	if (ret < 0)
+		return ret;
 
-		if (i2c->clk_src_div == 0)
-			return -EINVAL;
-	}
+	if (i2c->clk_src_div == 0)
+		return -EINVAL;
 
+	i2c->scp_share = of_property_read_bool(np, "mediatek,scp-share");
 	i2c->have_pmic = of_property_read_bool(np, "mediatek,have-pmic");
 	i2c->use_push_pull =
 		of_property_read_bool(np, "mediatek,use-push-pull");
@@ -799,7 +955,7 @@ static int mtk_i2c_probe(struct platform_device *pdev)
 		return -EINVAL;
 
 	if (i2c->dev_comp->timing_adjust)
-		i2c->clk_src_div = I2C_DEFAULT_CLK_DIV;
+		i2c->clk_src_div *= I2C_DEFAULT_CLK_DIV;
 
 	if (i2c->have_pmic && !i2c->dev_comp->pmic_i2c)
 		return -EINVAL;
@@ -843,6 +999,7 @@ static int mtk_i2c_probe(struct platform_device *pdev)
 	}
 
 	ret = mtk_i2c_clock_enable(i2c);
+
 	if (ret) {
 		dev_err(&pdev->dev, "clock enable failed!\n");
 		return ret;
@@ -860,10 +1017,8 @@ static int mtk_i2c_probe(struct platform_device *pdev)
 
 	i2c_set_adapdata(&i2c->adap, i2c);
 	ret = i2c_add_adapter(&i2c->adap);
-	if (ret) {
-		dev_err(&pdev->dev, "Failed to add i2c bus to i2c core\n");
+	if (ret)
 		return ret;
-	}
 
 	platform_set_drvdata(pdev, i2c);
 
@@ -879,37 +1034,11 @@ static int mtk_i2c_remove(struct platform_device *pdev)
 	return 0;
 }
 
-#ifdef CONFIG_PM_SLEEP
-static int mtk_i2c_resume(struct device *dev)
-{
-	int ret;
-	struct mtk_i2c *i2c = dev_get_drvdata(dev);
-
-	/* enable i2c clock to sync i2c register configuration */
-	ret = mtk_i2c_clock_enable(i2c);
-	if (ret) {
-		dev_err(dev, "clock enable failed!\n");
-		return ret;
-	}
-
-	mtk_i2c_init_hw(i2c);
-
-	mtk_i2c_clock_disable(i2c);
-
-	return 0;
-}
-#endif
-
-static const struct dev_pm_ops mtk_i2c_pm = {
-	SET_SYSTEM_SLEEP_PM_OPS(NULL, mtk_i2c_resume)
-};
-
 static struct platform_driver mtk_i2c_driver = {
 	.probe = mtk_i2c_probe,
 	.remove = mtk_i2c_remove,
 	.driver = {
 		.name = I2C_DRV_NAME,
-		.pm = &mtk_i2c_pm,
 		.of_match_table = of_match_ptr(mtk_i2c_of_match),
 	},
 };
