@@ -31,12 +31,15 @@
 #include <linux/slab.h>
 #include <linux/spinlock.h>
 #include <asm/barrier.h>
-#include <dt-bindings/memory/mt8173-larb-port.h>
+#ifndef CONFIG_ARM64
+#include <asm/dma-iommu.h>
+#endif
 #include <soc/mediatek/smi.h>
 
 #include "mtk_iommu.h"
 
 #define REG_MMU_PT_BASE_ADDR			0x000
+#define MMU_PT_ADDR_MASK			GENMASK(31, 7)
 
 #define REG_MMU_INVALIDATE			0x020
 #define F_ALL_INVLD				0x2
@@ -54,12 +57,16 @@
 
 #define REG_MMU_CTRL_REG			0x110
 #define F_MMU_PREFETCH_RT_REPLACE_MOD		BIT(4)
-/* The TF-protect-select is bit[5:4] in mt2712 while it's bit[6:5] in mt8173.*/
-#define F_MMU_TF_PROTECT_SEL(prot)		(((prot) & 0x3) << 5)
-#define F_MMU_TF_PROT_SEL(prot)			(((prot) & 0x3) << 4)
+#define F_MMU_TF_PROTECT_SEL_SHIFT(data) \
+	((data)->match_data->match_type != m4u_mt8173 ? 4 : 5)
+/* It's named by F_MMU_TF_PROT_SEL in mt2712. */
+#define F_MMU_TF_PROTECT_SEL(prot, data) \
+	(((prot) & 0x3) << F_MMU_TF_PROTECT_SEL_SHIFT(data))
 
 #define REG_MMU_IVRP_PADDR			0x114
-#define F_MMU_IVRP_PA_SET(pa, ext)		(((pa) >> 1) | ((!!(ext)) << 31))
+
+#define REG_MMU_VLD_PA_RNG			0x118
+#define F_MMU_VLD_PA_RNG(EA, SA)		(((EA) << 8) | (SA))
 
 #define REG_MMU_INT_CONTROL0			0x120
 #define F_L2_MULIT_HIT_EN			BIT(0)
@@ -95,6 +102,13 @@
 
 #define MTK_PROTECT_PA_ALIGN			128
 
+/*
+ * Get the local arbiter ID and the portid within the larb arbiter
+ * from mtk_m4u_id which is defined by MTK_M4U_ID.
+ */
+#define MTK_M4U_TO_LARB(id)		(((id) >> 5) & 0xf)
+#define MTK_M4U_TO_PORT(id)		((id) & 0x1f)
+
 struct mtk_iommu_domain {
 	spinlock_t			pgtlock; /* lock for page table */
 
@@ -105,7 +119,27 @@ struct mtk_iommu_domain {
 };
 
 static struct iommu_ops mtk_iommu_ops;
-static const struct of_device_id mtk_iommu_of_ids[];
+
+static LIST_HEAD(m4ulist);	/* List all the M4U HWs */
+
+#define for_each_m4u(data)	list_for_each_entry(data, &m4ulist, list)
+
+/*
+ * There may be 1 or 2 M4U HWs, But we always expect they are in the same domain
+ * for the performance.
+ *
+ * Here always return the mtk_iommu_data of the first probed M4U where the
+ * iommu domain information is recorded.
+ */
+static struct mtk_iommu_data *mtk_iommu_get_m4u_data(void)
+{
+	struct mtk_iommu_data *data;
+
+	for_each_m4u(data)
+		return data;
+
+	return NULL;
+}
 
 static struct mtk_iommu_domain *to_mtk_domain(struct iommu_domain *dom)
 {
@@ -116,9 +150,12 @@ static void mtk_iommu_tlb_flush_all(void *cookie)
 {
 	struct mtk_iommu_data *data = cookie;
 
-	writel_relaxed(F_INVLD_EN1 | F_INVLD_EN0, data->base + REG_MMU_INV_SEL);
-	writel_relaxed(F_ALL_INVLD, data->base + REG_MMU_INVALIDATE);
-	wmb(); /* Make sure the tlb flush all done */
+	for_each_m4u(data) {
+		writel_relaxed(F_INVLD_EN1 | F_INVLD_EN0,
+			       data->base + REG_MMU_INV_SEL);
+		writel_relaxed(F_ALL_INVLD, data->base + REG_MMU_INVALIDATE);
+		wmb(); /* Make sure the tlb flush all done */
+	}
 }
 
 static void mtk_iommu_tlb_add_flush_nosync(unsigned long iova, size_t size,
@@ -127,11 +164,16 @@ static void mtk_iommu_tlb_add_flush_nosync(unsigned long iova, size_t size,
 {
 	struct mtk_iommu_data *data = cookie;
 
-	writel_relaxed(F_INVLD_EN1 | F_INVLD_EN0, data->base + REG_MMU_INV_SEL);
+	for_each_m4u(data) {
+		writel_relaxed(F_INVLD_EN1 | F_INVLD_EN0,
+			       data->base + REG_MMU_INV_SEL);
 
-	writel_relaxed(iova, data->base + REG_MMU_INVLD_START_A);
-	writel_relaxed(iova + size - 1, data->base + REG_MMU_INVLD_END_A);
-	writel_relaxed(F_MMU_INV_RANGE, data->base + REG_MMU_INVALIDATE);
+		writel_relaxed(iova, data->base + REG_MMU_INVLD_START_A);
+		writel_relaxed(iova + size - 1,
+			       data->base + REG_MMU_INVLD_END_A);
+		writel_relaxed(F_MMU_INV_RANGE,
+			       data->base + REG_MMU_INVALIDATE);
+	}
 }
 
 static void mtk_iommu_tlb_sync(void *cookie)
@@ -140,15 +182,16 @@ static void mtk_iommu_tlb_sync(void *cookie)
 	int ret;
 	u32 tmp;
 
-	ret = readl_poll_timeout_atomic(data->base + REG_MMU_CPE_DONE, tmp,
-					tmp != 0, 10, 100000);
-	if (ret) {
-		dev_warn(data->dev,
-			 "Partial TLB flush timed out, falling back to full flush\n");
-		mtk_iommu_tlb_flush_all(cookie);
+	for_each_m4u(data) {
+		ret = readl_poll_timeout_atomic(data->base + REG_MMU_CPE_DONE,
+						tmp, tmp != 0, 10, 100000);
+		if (ret) {
+			dev_warn(data->dev, "Partial TLB flush timed out, falling back to full flush\n");
+			mtk_iommu_tlb_flush_all(cookie);
+		}
+		/* Clear the CPE status */
+		writel_relaxed(0, data->base + REG_MMU_CPE_DONE);
 	}
-	/* Clear the CPE status */
-	writel_relaxed(0, data->base + REG_MMU_CPE_DONE);
 }
 
 static const struct iommu_gather_ops mtk_iommu_gather_ops = {
@@ -170,7 +213,6 @@ static irqreturn_t mtk_iommu_isr(int irq, void *dev_id)
 	fault_iova = readl_relaxed(data->base + REG_MMU_FAULT_VA);
 	layer = fault_iova & F_MMU_FAULT_VA_LAYER_BIT;
 	write = fault_iova & F_MMU_FAULT_VA_WRITE_BIT;
-	fault_iova &= F_MMU_FAULT_VA_MSK;
 	fault_pa = readl_relaxed(data->base + REG_MMU_INVLD_PA);
 	regval = readl_relaxed(data->base + REG_MMU_INT_ID);
 	fault_larb = F_MMU0_INT_ID_LARB_ID(regval);
@@ -245,9 +287,9 @@ static int mtk_iommu_domain_finalise(struct mtk_iommu_data *data)
 	}
 
 	/* Update our support page sizes bitmap */
-	mtk_iommu_ops.pgsize_bitmap = dom->cfg.pgsize_bitmap;
+	dom->domain.pgsize_bitmap = dom->cfg.pgsize_bitmap;
 
-	writel(data->m4u_dom->cfg.arm_v7s_cfg.ttbr[0],
+	writel(data->m4u_dom->cfg.arm_v7s_cfg.ttbr[0] & MMU_PT_ADDR_MASK,
 	       data->base + REG_MMU_PT_BASE_ADDR);
 	return 0;
 }
@@ -256,7 +298,7 @@ static struct iommu_domain *mtk_iommu_domain_alloc(unsigned type)
 {
 	struct mtk_iommu_domain *dom;
 
-	if (type != IOMMU_DOMAIN_DMA)
+	if (type != IOMMU_DOMAIN_DMA && type != IOMMU_DOMAIN_UNMANAGED)
 		return NULL;
 
 	dom = kzalloc(sizeof(*dom), GFP_KERNEL);
@@ -286,13 +328,14 @@ static int mtk_iommu_attach_device(struct iommu_domain *domain,
 {
 	struct mtk_iommu_domain *dom = to_mtk_domain(domain);
 	struct mtk_iommu_client_priv *priv = dev->archdata.iommu;
-	struct mtk_iommu_data *data;
+	struct mtk_iommu_data *data, *curdata;
 	int ret;
 
 	if (!priv)
 		return -ENODEV;
 
-	data = dev_get_drvdata(priv->m4udev);
+	data = mtk_iommu_get_m4u_data();
+	curdata = dev_get_drvdata(priv->m4udev);
 	if (!data->m4u_dom) {
 		data->m4u_dom = dom;
 		ret = mtk_iommu_domain_finalise(data);
@@ -300,13 +343,20 @@ static int mtk_iommu_attach_device(struct iommu_domain *domain,
 			data->m4u_dom = NULL;
 			return ret;
 		}
-	} else if (data->m4u_dom != dom) {
-		/* All the client devices should be in the same m4u domain */
-		dev_err(dev, "try to attach into the error iommu domain\n");
-		return -EPERM;
 	}
 
-	mtk_iommu_config(data, dev, true);
+	/*
+	 * Always use the same pgtable for the performance.
+	 * Update the pgtable reg of M4U-HW1 with the existed pgtable.
+	 */
+	if (!curdata->m4u_dom) {
+		curdata->m4u_dom = data->m4u_dom;
+		writel((data->m4u_dom->cfg.arm_v7s_cfg.ttbr[0] &
+			MMU_PT_ADDR_MASK),
+		       curdata->base + REG_MMU_PT_BASE_ADDR);
+	}
+
+	mtk_iommu_config(curdata, dev, true);
 	return 0;
 }
 
@@ -314,13 +364,13 @@ static void mtk_iommu_detach_device(struct iommu_domain *domain,
 				    struct device *dev)
 {
 	struct mtk_iommu_client_priv *priv = dev->archdata.iommu;
-	struct mtk_iommu_data *data;
+	struct mtk_iommu_data *curdata;
 
 	if (!priv)
 		return;
 
-	data = dev_get_drvdata(priv->m4udev);
-	mtk_iommu_config(data, dev, false);
+	curdata = dev_get_drvdata(priv->m4udev);
+	mtk_iommu_config(curdata, dev, false);
 }
 
 static int mtk_iommu_map(struct iommu_domain *domain, unsigned long iova,
@@ -365,6 +415,7 @@ static phys_addr_t mtk_iommu_iova_to_phys(struct iommu_domain *domain,
 	return pa;
 }
 
+#ifdef CONFIG_ARM64
 static int mtk_iommu_add_device(struct device *dev)
 {
 	struct iommu_group *group;
@@ -379,6 +430,75 @@ static int mtk_iommu_add_device(struct device *dev)
 	iommu_group_put(group);
 	return 0;
 }
+#else
+static int mtk_iommu_init_arm_mapping(struct device *dev)
+{
+	struct mtk_iommu_data *data;
+	struct dma_iommu_mapping *mtk_mapping;
+	struct device *m4udev;
+
+	data =	mtk_iommu_get_m4u_data();
+	m4udev = data->dev;
+	mtk_mapping = m4udev->archdata.iommu;
+	if (!mtk_mapping) {
+		/* MTK iommu support 4GB iova address space. */
+		mtk_mapping = arm_iommu_create_mapping(&platform_bus_type,
+						0, 1ULL << 32);
+		if (IS_ERR(mtk_mapping))
+			return PTR_ERR(mtk_mapping);
+
+		m4udev->archdata.iommu = mtk_mapping;
+	}
+	return arm_iommu_attach_device(dev, mtk_mapping);
+}
+
+static int mtk_iommu_add_device(struct device *dev)
+{
+	struct of_phandle_args iommu_spec;
+	struct iommu_group *group;
+	int err;
+	int idx = 0;
+
+	while (!of_parse_phandle_with_args(dev->of_node, "iommus",
+					   "#iommu-cells", idx,
+					   &iommu_spec)) {
+		of_node_put(iommu_spec.np);
+		idx++;
+	}
+
+	/* Skip M4U device itself.*/
+	if (!idx) {
+		dev_dbg(dev, "%s, not a iommu client\n", __func__);
+		return -ENODEV;
+	}
+
+	if (!dev->archdata.iommu) /* Not a iommu client device */
+		return -ENODEV;
+
+	/*
+	 * This is a short-term bodge because the ARM DMA code doesn't
+	 * understand multi-device groups, but we have to call into it
+	 * successfully (and not just rely on a normal IOMMU API attach
+	 * here) in order to set the correct DMA API ops on @dev.
+	 */
+	group = iommu_group_alloc();
+	if (IS_ERR(group))
+		return PTR_ERR(group);
+
+	err = iommu_group_add_device(group, dev);
+	iommu_group_put(group);
+	if (err)
+		return err;
+
+	err = mtk_iommu_init_arm_mapping(dev);
+	if (err) {
+		dev_info(dev, "attach-iommu fail %d\n", err);
+		iommu_group_remove_device(dev);
+		return err;
+	}
+	return 0;
+}
+#endif
 
 static void mtk_iommu_remove_device(struct device *dev)
 {
@@ -408,7 +528,20 @@ static struct iommu_group *mtk_iommu_device_group(struct device *dev)
 		return ERR_PTR(-ENODEV);
 
 	/* All the client devices are in the same m4u iommu-group */
-	data = dev_get_drvdata(priv->m4udev);
+	data = mtk_iommu_get_m4u_data();
+
+	/*
+	 * We may run into a scenario that of_xlate have already set dev->archdata.iommu, but
+	 * iommu driver has not been finished probe, at this time data is NULL, we will trigger
+	 * bus scan all the device after all iommu's probe have been done.
+	 *
+	 * Just make the first enter is safe and will not crash the system. After all the iommu
+	 * driver has finished probe, bus_set_iommu will trigger all the not-been added device
+	 * add to iommu domain again.
+	 */
+	if (!data)
+		return ERR_PTR(-ENODEV);
+
 	if (!data->m4u_group) {
 		data->m4u_group = iommu_group_alloc();
 		if (IS_ERR(data->m4u_group))
@@ -431,7 +564,6 @@ static int mtk_iommu_of_xlate(struct device *dev, struct of_phandle_args *args)
 	if (!dev->archdata.iommu) {
 		/* Get the m4u device */
 		m4updev = of_find_device_by_node(args->np);
-		of_node_put(args->np);
 		if (WARN_ON(!m4updev))
 			return -EINVAL;
 
@@ -481,6 +613,7 @@ static struct iommu_ops mtk_iommu_ops = {
 
 static int mtk_iommu_hw_init(const struct mtk_iommu_data *data)
 {
+	enum mtk_iommu_match_type m4u_plat = data->match_data->match_type;
 	u32 regval;
 	int ret;
 
@@ -490,12 +623,9 @@ static int mtk_iommu_hw_init(const struct mtk_iommu_data *data)
 		return ret;
 	}
 
-	if (data->match_type == m4u_mt8173) {
-		regval = F_MMU_PREFETCH_RT_REPLACE_MOD |
-			F_MMU_TF_PROTECT_SEL(2);
-	} else {
-		regval = F_MMU_TF_PROT_SEL(2);
-	}
+	regval = F_MMU_TF_PROTECT_SEL(2, data);
+	if (m4u_plat == m4u_mt8173)
+		regval |= F_MMU_PREFETCH_RT_REPLACE_MOD;
 	writel_relaxed(regval, data->base + REG_MMU_CTRL_REG);
 
 	regval = F_L2_MULIT_HIT_EN |
@@ -515,11 +645,27 @@ static int mtk_iommu_hw_init(const struct mtk_iommu_data *data)
 		F_INT_PRETETCH_TRANSATION_FIFO_FAULT;
 	writel_relaxed(regval, data->base + REG_MMU_INT_MAIN_CONTROL);
 
-	writel_relaxed(F_MMU_IVRP_PA_SET(data->protect_base, data->enable_4GB),
-		       data->base + REG_MMU_IVRP_PADDR);
+	if (m4u_plat == m4u_mt8173 || m4u_plat == m4u_mt8518)
+		regval = (data->protect_base >> 1) | (data->enable_4GB << 31);
+	else
+		regval = lower_32_bits(data->protect_base) |
+			 upper_32_bits(data->protect_base);
+	writel_relaxed(regval, data->base + REG_MMU_IVRP_PADDR);
 
+	if (data->enable_4GB && m4u_plat == m4u_mt2712) {
+		/*
+		 * If 4GB mode is enabled, the validate PA range is from
+		 * 0x1_0000_0000 to 0x1_ffff_ffff. here record bit[32:30].
+		 */
+		regval = F_MMU_VLD_PA_RNG(7, 4);
+		writel_relaxed(regval, data->base + REG_MMU_VLD_PA_RNG);
+	}
 	writel_relaxed(0, data->base + REG_MMU_DCM_DIS);
-	writel_relaxed(0, data->base + REG_MMU_STANDARD_AXI_MODE);
+
+	/* It's MISC control register whose default value is ok except mt8173.*/
+	if (m4u_plat == m4u_mt8173 || m4u_plat == m4u_mt8518 ||
+	    m4u_plat == m4u_mt8168)
+		writel_relaxed(0, data->base + REG_MMU_STANDARD_AXI_MODE);
 
 	if (devm_request_irq(data->dev, data->irq, mtk_iommu_isr, 0,
 			     dev_name(data->dev), (void *)data)) {
@@ -539,21 +685,20 @@ static const struct component_master_ops mtk_iommu_com_ops = {
 
 static int mtk_iommu_probe(struct platform_device *pdev)
 {
-	const struct of_device_id        *of_id;
 	struct mtk_iommu_data   *data;
 	struct device           *dev = &pdev->dev;
 	struct resource         *res;
 	struct component_match  *match = NULL;
 	void                    *protect;
 	int                     i, larb_nr, ret;
+	static int iommu_cnt;
 
 	data = devm_kzalloc(dev, sizeof(*data), GFP_KERNEL);
 	if (!data)
 		return -ENOMEM;
 	data->dev = dev;
 
-	of_id = of_match_node(mtk_iommu_of_ids, dev->of_node);
-	data->match_type = (enum mtk_iommu_match_type)of_id->data;
+	data->match_data = of_device_get_match_data(dev);
 
 	/* Protect memory. HW will access here while translation fault.*/
 	protect = devm_kzalloc(dev, MTK_PROTECT_PA_ALIGN * 2, GFP_KERNEL);
@@ -562,7 +707,7 @@ static int mtk_iommu_probe(struct platform_device *pdev)
 	data->protect_base = ALIGN(virt_to_phys(protect), MTK_PROTECT_PA_ALIGN);
 
 	/* Whether the current dram is over 4GB */
-	data->enable_4GB = !!(max_pfn > (0xffffffffUL >> PAGE_SHIFT));
+	data->enable_4GB = !!(max_pfn > (BIT_ULL(32) >> PAGE_SHIFT));
 
 	res = platform_get_resource(pdev, IORESOURCE_MEM, 0);
 	data->base = devm_ioremap_resource(dev, res);
@@ -574,7 +719,9 @@ static int mtk_iommu_probe(struct platform_device *pdev)
 		return data->irq;
 
 	data->bclk = devm_clk_get(dev, "bclk");
-	if (IS_ERR(data->bclk))
+	if (PTR_ERR(data->bclk) == -ENOENT)
+		data->bclk = NULL;
+	else if (IS_ERR(data->bclk))
 		return PTR_ERR(data->bclk);
 
 	larb_nr = of_count_phandle_with_args(dev->of_node,
@@ -595,7 +742,7 @@ static int mtk_iommu_probe(struct platform_device *pdev)
 		if (!of_device_is_available(larbnode))
 			continue;
 
-		ret = of_property_read_u32(larbnode, "mediatek,larbidx", &idx);
+		ret = of_property_read_u32(larbnode, "mediatek,larb-id", &idx);
 		/* The index is consecutive if there is no this property */
 		if (ret)
 			idx = i;
@@ -620,7 +767,14 @@ static int mtk_iommu_probe(struct platform_device *pdev)
 	if (ret)
 		return ret;
 
-	if (!iommu_present(&platform_bus_type))
+	list_add_tail(&data->list, &m4ulist);
+
+	iommu_cnt++;
+	/*
+	 * trigger the bus to scan all the device to add them to iommu domain after all the iommu
+	 * have finished probe.
+	 */
+	if (!iommu_present(&platform_bus_type) && iommu_cnt == data->match_data->iommu_cnt)
 		bus_set_iommu(&platform_bus_type, &mtk_iommu_ops);
 
 	return component_master_add_with_match(dev, &mtk_iommu_com_ops, match);
@@ -652,6 +806,8 @@ static int __maybe_unused mtk_iommu_suspend(struct device *dev)
 	reg->ctrl_reg = readl_relaxed(base + REG_MMU_CTRL_REG);
 	reg->int_control0 = readl_relaxed(base + REG_MMU_INT_CONTROL0);
 	reg->int_main_control = readl_relaxed(base + REG_MMU_INT_MAIN_CONTROL);
+	reg->ivrp_paddr = readl_relaxed(base + REG_MMU_IVRP_PADDR);
+	clk_disable_unprepare(data->bclk);
 	return 0;
 }
 
@@ -660,27 +816,59 @@ static int __maybe_unused mtk_iommu_resume(struct device *dev)
 	struct mtk_iommu_data *data = dev_get_drvdata(dev);
 	struct mtk_iommu_suspend_reg *reg = &data->reg;
 	void __iomem *base = data->base;
+	int ret;
 
-	writel_relaxed(data->m4u_dom->cfg.arm_v7s_cfg.ttbr[0],
-		       base + REG_MMU_PT_BASE_ADDR);
+	ret = clk_prepare_enable(data->bclk);
+	if (ret) {
+		dev_err(data->dev, "Failed to enable clk(%d) in resume\n", ret);
+		return ret;
+	}
 	writel_relaxed(reg->standard_axi_mode,
 		       base + REG_MMU_STANDARD_AXI_MODE);
 	writel_relaxed(reg->dcm_dis, base + REG_MMU_DCM_DIS);
 	writel_relaxed(reg->ctrl_reg, base + REG_MMU_CTRL_REG);
 	writel_relaxed(reg->int_control0, base + REG_MMU_INT_CONTROL0);
 	writel_relaxed(reg->int_main_control, base + REG_MMU_INT_MAIN_CONTROL);
-	writel_relaxed(F_MMU_IVRP_PA_SET(data->protect_base, data->enable_4GB),
-		       base + REG_MMU_IVRP_PADDR);
+	writel_relaxed(reg->ivrp_paddr, base + REG_MMU_IVRP_PADDR);
+	if (data->m4u_dom)
+		writel((data->m4u_dom->cfg.arm_v7s_cfg.ttbr[0] &
+			MMU_PT_ADDR_MASK),
+		       base + REG_MMU_PT_BASE_ADDR);
 	return 0;
 }
 
-const struct dev_pm_ops mtk_iommu_pm_ops = {
-	SET_SYSTEM_SLEEP_PM_OPS(mtk_iommu_suspend, mtk_iommu_resume)
+static const struct dev_pm_ops mtk_iommu_pm_ops = {
+	SET_NOIRQ_SYSTEM_SLEEP_PM_OPS(mtk_iommu_suspend, mtk_iommu_resume)
+};
+
+#define MT8173_IOMMU_CNT	1
+#define MT2712_IOMMU_CNT	2
+
+const struct mtk_iommu_match_data mt8168_data = {
+	.match_type = m4u_mt8168,
+	.iommu_cnt = 1,
+};
+
+const struct mtk_iommu_match_data mt8173_data = {
+	.match_type = m4u_mt8173,
+	.iommu_cnt = MT8173_IOMMU_CNT,
+};
+
+const struct mtk_iommu_match_data mt2712_data = {
+	.match_type = m4u_mt2712,
+	.iommu_cnt = MT2712_IOMMU_CNT,
+};
+
+const struct mtk_iommu_match_data mt8518_data = {
+	.match_type = m4u_mt8518,
+	.iommu_cnt = 1,
 };
 
 static const struct of_device_id mtk_iommu_of_ids[] = {
-	{ .compatible = "mediatek,mt8173-m4u", .data = (void *)m4u_mt8173},
-	{ .compatible = "mediatek,mt2712-m4u", .data = (void *)m4u_mt2712},
+	{ .compatible = "mediatek,mt8173-m4u", .data = (void *)&mt8173_data},
+	{ .compatible = "mediatek,mt2712-m4u", .data = (void *)&mt2712_data},
+	{ .compatible = "mediatek,mt8518-m4u", .data = (void *)&mt8518_data},
+	{ .compatible = "mediatek,mt8168-m4u", .data = (void *)&mt8168_data},
 	{}
 };
 
@@ -696,27 +884,34 @@ static struct platform_driver mtk_iommu_driver = {
 
 static int mtk_iommu_init_fn(struct device_node *np)
 {
-	static bool init_done;
-	int ret;
 	struct platform_device *pdev;
+	int ret;
 
-	if (!init_done) {
-		pdev = of_platform_device_create(np, NULL,
-						 platform_bus_type.dev_root);
-		if (!pdev)
-			return -ENOMEM;
+	pdev = of_platform_device_create(np, NULL,
+					 platform_bus_type.dev_root);
+	if (!pdev)
+		return -ENOMEM;
 
-		ret = platform_driver_register(&mtk_iommu_driver);
-		if (ret) {
-			pr_err("%s: Failed to register driver\n", __func__);
-			return ret;
-		}
-		init_done = true;
+	ret = platform_driver_register(&mtk_iommu_driver);
+	if (ret) {
+		pr_err("%s: Failed to register driver\n", __func__);
+		return ret;
 	}
-
+	/*
+	 * We must set the ops here or we may not get the very master "np" of_xlate callback
+	 * not set at the time of_iommu_configure was called by of_dma_configure, when platform device
+	 * node was created by of_platform_device_create_pdata. And we get dev->archdata.iommu not
+	 * set, then we could not add the device into iommu group.
+	 *
+	 * If iommu probe was deferred, it will be delayed after late_initcall, that's when defer_probe
+	 * work was scheduled. Our of_xlate callback could not be excuted at that time and iommu clients
+	 * could not be added to the iommu domain.
+	 */
 	of_iommu_set_ops(np, &mtk_iommu_ops);
+
 	return 0;
 }
 
 IOMMU_OF_DECLARE(mt8173m4u, "mediatek,mt8173-m4u", mtk_iommu_init_fn);
 IOMMU_OF_DECLARE(mt2712m4u, "mediatek,mt2712-m4u", mtk_iommu_init_fn);
+IOMMU_OF_DECLARE(mt8518m4u, "mediatek,mt8518-m4u", mtk_iommu_init_fn);
