@@ -30,7 +30,7 @@
 #include <linux/pinctrl/consumer.h>
 #include <linux/regulator/consumer.h>
 #include <media/v4l2-chip-ident.h>
-#include <media/v4l2-int-device.h>
+#include "v4l2-int-device.h"
 #include "mxc_v4l2_capture.h"
 
 #define ADV7180_VOLTAGE_ANALOG               1800000
@@ -887,9 +887,10 @@ static struct v4l2_int_ioctl_desc adv7180_ioctl_desc[] = {
 	{vidioc_int_g_ctrl_num, (v4l2_int_ioctl_func*)ioctl_g_ctrl},
 	{vidioc_int_s_ctrl_num, (v4l2_int_ioctl_func*)ioctl_s_ctrl},
 	{vidioc_int_enum_framesizes_num,
-				(v4l2_int_ioctl_func *) ioctl_enum_framesizes},
+				(v4l2_int_ioctl_func *)ioctl_enum_framesizes},
 	{vidioc_int_enum_frameintervals_num,
-				(v4l2_int_ioctl_func *) ioctl_enum_frameintervals},
+				(v4l2_int_ioctl_func *)
+				ioctl_enum_frameintervals},
 	{vidioc_int_g_chip_ident_num,
 				(v4l2_int_ioctl_func *)ioctl_g_chip_ident},
 };
