@@ -127,9 +127,8 @@ static int vol_cdev_open(struct inode *inode, struct file *file)
 	return 0;
 }
 
-static int vol_cdev_release(struct inode *inode, struct file *file)
+static int vol_cdev_release_desc(struct ubi_volume_desc *desc)
 {
-	struct ubi_volume_desc *desc = file->private_data;
 	struct ubi_volume *vol = desc->vol;
 
 	dbg_gen("release device %d, volume %d, mode %d",
@@ -153,6 +152,11 @@ static int vol_cdev_release(struct inode *inode, struct file *file)
 	return 0;
 }
 
+static int vol_cdev_release(struct inode *inode, struct file *file)
+{
+	return vol_cdev_release_desc(file->private_data);
+}
+
 static loff_t vol_cdev_llseek(struct file *file, loff_t offset, int origin)
 {
 	struct ubi_volume_desc *desc = file->private_data;
@@ -174,9 +178,9 @@ static int vol_cdev_fsync(struct file *file, loff_t start, loff_t end,
 	struct ubi_device *ubi = desc->vol->ubi;
 	struct inode *inode = file_inode(file);
 	int err;
-	mutex_lock(&inode->i_mutex);
+	inode_lock(inode);
 	err = ubi_sync(ubi->ubi_num);
-	mutex_unlock(&inode->i_mutex);
+	inode_unlock(inode);
 	return err;
 }
 
@@ -416,7 +420,7 @@ static long vol_cdev_ioctl(struct file *file, unsigned int cmd,
 		}
 
 		rsvd_bytes = (long long)vol->reserved_pebs *
-					ubi->leb_size-vol->data_pad;
+					vol->usable_leb_size;
 		if (bytes < 0 || bytes > rsvd_bytes) {
 			err = -EINVAL;
 			break;
@@ -454,7 +458,7 @@ static long vol_cdev_ioctl(struct file *file, unsigned int cmd,
 
 		/* Validate the request */
 		err = -EINVAL;
-		if (req.lnum < 0 || req.lnum >= vol->reserved_pebs ||
+		if (!ubi_leb_valid(vol, req.lnum) ||
 		    req.bytes < 0 || req.bytes > vol->usable_leb_size)
 			break;
 
@@ -485,7 +489,7 @@ static long vol_cdev_ioctl(struct file *file, unsigned int cmd,
 			break;
 		}
 
-		if (lnum < 0 || lnum >= vol->reserved_pebs) {
+		if (!ubi_leb_valid(vol, lnum)) {
 			err = -EINVAL;
 			break;
 		}
@@ -980,7 +984,6 @@ static long ubi_cdev_ioctl(struct file *file, unsigned int cmd,
 		kfree(req);
 		break;
 	}
-
 #if defined(CONFIG_SONOS)
 	/* Get information about volume geometry and available pebs.  This enables sufficient
 	 * information for a user-space utility to implement volume resize safely.
@@ -1171,71 +1174,75 @@ const struct file_operations ubi_ctrl_cdev_operations = {
 	.llseek		= no_llseek,
 };
 
-#ifdef CONFIG_SONOS
-struct ubi_volume_desc * rootfs_open(int vol_id, int ubi_num)
+#ifdef CONFIG_SONOS_SECBOOT
+/* compare to vol_cdev_ioctl(UBI_IOCVOLUP) */
+struct ubi_volume_desc *rootfs_start_update(const char *ubi_volume_name,
+					    size_t data_len)
 {
 	struct ubi_volume_desc *desc;
-
-	desc = ubi_open_volume(ubi_num, vol_id, UBI_READWRITE);
-	if (IS_ERR(desc)) {
-		printk("rootfs open device %d, volume %d fail %ld\n",
-			ubi_num, vol_id, PTR_ERR(desc));
-		return NULL;
-	}
-	return desc;
-}
-
-int rootfs_update(struct ubi_volume_desc *desc, int64_t bytes)
-{
-	int err = 0;
-	struct ubi_volume *vol = desc->vol;
-	struct ubi_device *ubi = vol->ubi;
-	int64_t rsvd_bytes;
+	struct ubi_volume *vol;
+	struct ubi_device *ubi;
+	size_t rsvd_bytes;
+	int err;
 
 	if (!capable(CAP_SYS_RESOURCE)) {
-		err = -EPERM;
-		goto  err_skip;
+		return ERR_PTR(-EPERM);
 	}
 
-	if (desc->mode == UBI_READONLY) {
-		err = -EROFS;
-		goto err_skip;
+	desc = ubi_open_volume_nm(0, ubi_volume_name, UBI_EXCLUSIVE);
+	if (IS_ERR(desc)) {
+		pr_err("rootfs_start_update UBI open volume %s failed: %ld",
+		       ubi_volume_name, PTR_ERR(desc));
+		return desc;
+	}
+	vol = desc->vol;
+	ubi = vol->ubi;
+
+	rsvd_bytes = ((size_t)vol->reserved_pebs) *
+		     ((size_t)vol->usable_leb_size);
+	if (data_len > rsvd_bytes) {
+		ubi_err(ubi,
+			"update (%zu) too large for volume %s capacity (%zu)",
+			data_len, ubi_volume_name, rsvd_bytes);
+		ubi_close_volume(desc);
+		return ERR_PTR(-EINVAL);
 	}
 
-	rsvd_bytes = (long long)vol->reserved_pebs *
-			ubi->leb_size-vol->data_pad;
-	if (bytes < 0 || bytes > rsvd_bytes) {
-		err = -EINVAL;
-		goto err_skip;
+	err = ubi_start_update(ubi, vol, data_len);
+	if (err) {
+		ubi_close_volume(desc);
+		return ERR_PTR(err);
 	}
-
-	err = get_exclusive(desc);
-	if (err < 0)
-		goto err_skip;
-
-	err = ubi_start_update(ubi, vol, bytes);
-	if (bytes == 0)
+	if (data_len == 0) {
+		ubi_volume_notify(ubi, vol, UBI_VOLUME_UPDATED);
 		revoke_exclusive(desc, UBI_READWRITE);
-err_skip:
-	return err;
-}
+	}
 
-ssize_t rootfs_write(struct ubi_volume_desc *desc, const char *buf,
-			      size_t count, int flag)
+	return desc;
+}
+EXPORT_SYMBOL(rootfs_start_update);
+
+/* compare to vol_cdev_write */
+ssize_t rootfs_write(struct ubi_volume_desc *desc, const void *buf,
+		     size_t count)
 {
 	int err = 0;
 	struct ubi_volume *vol = desc->vol;
 	struct ubi_device *ubi = vol->ubi;
 
-	if ( !vol->updating ) {
-		printk("rootfs_write not ready\n");
-		return -1;
+	if (!vol->updating) {
+		ubi_err(ubi, "not ready");
+		return -EINVAL;
 	}
 
-	err = rootfs_ubi_more_update_data(ubi, vol, buf, count, flag);
+	if (count > (size_t)INT_MAX) {
+		ubi_err(ubi, "write too big (%zu bytes)", count);
+		return -EINVAL;
+	}
 
+	err = rootfs_ubi_more_update_data(ubi, vol, buf, (int)count);
 	if (err < 0) {
-		ubi_err(ubi, "cannot accept more %zd bytes of data, error %d",
+		ubi_err(ubi, "cannot accept more %zu bytes of data, error %d",
 			count, err);
 		return err;
 	}
@@ -1247,11 +1254,6 @@ ssize_t rootfs_write(struct ubi_volume_desc *desc, const char *buf,
 		 */
 		count = err;
 
-		if (vol->changing_leb) {
-			revoke_exclusive(desc, UBI_READWRITE);
-			return count;
-		}
-
 		err = ubi_check_volume(ubi, vol->vol_id);
 		if (err < 0)
 			return err;
@@ -1268,26 +1270,11 @@ ssize_t rootfs_write(struct ubi_volume_desc *desc, const char *buf,
 
 	return count;
 }
+EXPORT_SYMBOL(rootfs_write);
 
-int rootfs_release(struct ubi_volume_desc *desc)
+void rootfs_release(struct ubi_volume_desc *desc)
 {
-	struct ubi_volume *vol = desc->vol;
-
-	if (vol->updating) {
-		ubi_warn(vol->ubi, "update of volume %d not finished, volume is damaged",
-			 vol->vol_id);
-		ubi_assert(!vol->changing_leb);
-		vol->updating = 0;
-		vfree(vol->upd_buf);
-	} else if (vol->changing_leb) {
-		dbg_gen("only %lld of %lld bytes received for atomic LEB change for volume %d:%d, cancel",
-			vol->upd_received, vol->upd_bytes, vol->ubi->ubi_num,
-			vol->vol_id);
-		vol->changing_leb = 0;
-		vfree(vol->upd_buf);
-	}
-
-	ubi_close_volume(desc);
-	return 0;
+	(void)vol_cdev_release_desc(desc);
 }
+EXPORT_SYMBOL(rootfs_release);
 #endif
