@@ -77,24 +77,16 @@ int ath_nand_local_read(u_char *cal_part, loff_t from, size_t len,
 	struct gluebi_device *gluebi;
 	int lnum, offs, bytes_left;
 
-	if ( !strncmp(cal_part,"mtdblock",8 ) ) {
-		// device name - don't need to walk the list
-		sscanf(&cal_part[8],"%d",&i);
+	for ( i = 0; i < 10; i++ ) {
 		mtd = __mtd_next_device(i);
-		printk(KERN_INFO "reading %d bytes from %s - %s [%d]\n",len,cal_part,mtd->name,i);
-	}
-	else {
-		for ( i = 0; i < 10; i++ ) {
-			mtd = __mtd_next_device(i);
-			if ( mtd == NULL )
-				return -1;
-			if ( memcmp(mtd->name, cal_part, 3) == 0 )
-				break;
-		}
-		if ( i == 10 ) {
-			printk("ath_nand_local_read %s not found use art\n", cal_part);
+		if ( mtd == NULL )
 			return -1;
-		}
+		if ( memcmp(mtd->name, cal_part, 3) == 0 )
+			break;
+	}
+	if ( i == 10 ) {
+		printk("ath_nand_local_read %s not found use art\n", cal_part);
+		return -1;
 	}
 
 	gluebi = container_of(mtd, struct gluebi_device, mtd);
@@ -118,26 +110,11 @@ int ath_nand_local_read(u_char *cal_part, loff_t from, size_t len,
 		buf += to_read;
 	}
 
-	// Need to use this interface to validate rootfs digest before mounting,
-	// but want to leave things in their status quo ante state, so if this
-	// was opened for rootfs, close the volume.
 	ubi_close_volume(desc);
 
 	*retlen = len - bytes_left;
 	return err;
 }
-
-void get_rootfs_ubi_info(struct mtd_info * mtd, int *vol_id, int *ubi_num)
-{
-	struct gluebi_device *gluebi;
-	gluebi = container_of(mtd, struct gluebi_device, mtd);
-	if ( gluebi == NULL ) {
-		printk("No gluebi of %s\n", mtd->name);
-		return;
-	}
-	*vol_id = gluebi->vol_id;
-	*ubi_num = gluebi->ubi_num;
-}
 #endif
 
 /**
@@ -173,9 +150,6 @@ static int gluebi_get_device(struct mtd_info *mtd)
 	struct gluebi_device *gluebi;
 	int ubi_mode = UBI_READONLY;
 
-	if (!try_module_get(THIS_MODULE))
-		return -ENODEV;
-
 	if (mtd->flags & MTD_WRITEABLE)
 		ubi_mode = UBI_READWRITE;
 
@@ -203,7 +177,6 @@ static int gluebi_get_device(struct mtd_info *mtd)
 				       ubi_mode);
 	if (IS_ERR(gluebi->desc)) {
 		mutex_unlock(&devices_mutex);
-		module_put(THIS_MODULE);
 		return PTR_ERR(gluebi->desc);
 	}
 	gluebi->refcnt += 1;
@@ -227,7 +200,6 @@ static void gluebi_put_device(struct mtd_info *mtd)
 	gluebi->refcnt -= 1;
 	if (gluebi->refcnt == 0)
 		ubi_close_volume(gluebi->desc);
-	module_put(THIS_MODULE);
 	mutex_unlock(&devices_mutex);
 }
 
