@@ -1,23 +1,58 @@
 /****************************************************************************
 *
-*    Copyright (C) 2005 - 2014 by Vivante Corp.
+*    The MIT License (MIT)
 *
-*    This program is free software; you can redistribute it and/or modify
-*    it under the terms of the GNU General Public License as published by
-*    the Free Software Foundation; either version 2 of the license, or
-*    (at your option) any later version.
+*    Copyright (c) 2014 - 2018 Vivante Corporation
+*
+*    Permission is hereby granted, free of charge, to any person obtaining a
+*    copy of this software and associated documentation files (the "Software"),
+*    to deal in the Software without restriction, including without limitation
+*    the rights to use, copy, modify, merge, publish, distribute, sublicense,
+*    and/or sell copies of the Software, and to permit persons to whom the
+*    Software is furnished to do so, subject to the following conditions:
+*
+*    The above copyright notice and this permission notice shall be included in
+*    all copies or substantial portions of the Software.
+*
+*    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+*    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+*    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+*    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+*    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+*    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+*    DEALINGS IN THE SOFTWARE.
+*
+*****************************************************************************
+*
+*    The GPL License (GPL)
+*
+*    Copyright (C) 2014 - 2018 Vivante Corporation
+*
+*    This program is free software; you can redistribute it and/or
+*    modify it under the terms of the GNU General Public License
+*    as published by the Free Software Foundation; either version 2
+*    of the License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
-*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
-*    along with this program; if not write to the Free Software
-*    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+*    along with this program; if not, write to the Free Software Foundation,
+*    Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+*
+*****************************************************************************
+*
+*    Note: This software is released under dual MIT and GPL licenses. A
+*    recipient may use this file under the terms of either the MIT license or
+*    GPL License. If you wish to use only one license not the other, you can
+*    indicate your decision by deleting one of the above license notices in your
+*    version of this file.
 *
 *****************************************************************************/
 
+
 #ifndef __gc_hal_options_h_
 #define __gc_hal_options_h_
 
@@ -38,6 +73,19 @@
 #   define gcdPRINT_VERSION                     0
 #endif
 
+/*
+USE_KERNEL_VIRTUAL_BUFFERS
+
+This define enables the use of VM for gckCommand and fence buffers.
+*/
+#ifndef USE_KERNEL_VIRTUAL_BUFFERS
+#if defined(UNDER_CE)
+#   define USE_KERNEL_VIRTUAL_BUFFERS           1
+#else
+#   define USE_KERNEL_VIRTUAL_BUFFERS           1
+#endif
+#endif
+
 /*
     USE_NEW_LINUX_SIGNAL
 
@@ -47,6 +95,15 @@
 #   define USE_NEW_LINUX_SIGNAL                 0
 #endif
 
+/*
+    USE_LINUX_PCIE
+
+        This define enables galcore as a Linux PCIE driver.
+*/
+#ifndef USE_LINUX_PCIE
+#   define USE_LINUX_PCIE                       0
+#endif
+
 /*
     VIVANTE_PROFILER
 
@@ -56,26 +113,6 @@
 #   define VIVANTE_PROFILER                     1
 #endif
 
-/*
-    VIVANTE_PROFILER_CONTEXT
-
-        This define enables the profiler according each context.
-*/
-#ifndef VIVANTE_PROFILER_CONTEXT
-#   define VIVANTE_PROFILER_CONTEXT             1
-#endif
-
-#ifndef VIVANTE_PROFILER_PERDRAW
-#   define VIVANTE_PROFILER_PERDRAW             0
-#endif
-
-#ifndef VIVANTE_PROFILER_NEW
-#   define VIVANTE_PROFILER_NEW                 0
-#endif
-
-#ifndef VIVANTE_PROFILER_PM
-#   define VIVANTE_PROFILER_PM                  1
-#endif
 /*
     gcdUSE_VG
 
@@ -86,12 +123,12 @@
 #endif
 
 /*
-    USE_SW_FB
+    gcdUSE_VX
 
-        Set to 1 if the frame buffer memory cannot be accessed by the GPU.
+        Enable VX HAL layer.
 */
-#ifndef USE_SW_FB
-#   define USE_SW_FB                            0
+#ifndef gcdUSE_VX
+#   define gcdUSE_VX                            1
 #endif
 
 /*
@@ -159,6 +196,9 @@
         When set to 1, a dump of all states and memory uploads, as well as other
         hardware related execution will be printed to the debug console.  This
         data can be used for playing back applications.
+
+        When set to 2, for vxc, all output memory will be dump.
+
 */
 #ifndef gcdDUMP
 #   define gcdDUMP                              0
@@ -174,7 +214,19 @@
 #   define gcdDUMP_API                          0
 #endif
 
+#ifndef gcdDUMP_2DVG
+#   define gcdDUMP_2DVG                         0
+#endif
 
+/*
+    gcdDUMP_AHB_ACCESS
+
+        When set to 1, a dump of all AHB register access will be printed to kernel
+        message.
+*/
+#ifndef gcdDUMP_AHB_ACCESS
+#   define gcdDUMP_AHB_ACCESS                   0
+#endif
 
 /*
     gcdDEBUG_OPTION
@@ -237,19 +289,39 @@
 #           define gcdDEBUG_OPTION_NONE_DEPTH                   0
 #       endif
 
+/*
+    gcdDEBUG_FORCE_CONTEXT_UPDATE
+        When set to 1, context will be updated before every commit.
+*/
+#ifndef gcdDEBUG_FORCE_CONTEXT_UPDATE
+#           define gcdDEBUG_FORCE_CONTEXT_UPDATE                0
+#       endif
+
+
+/*
+    gcdDEBUG_FORCE_CONTEXT_UPDATE
+        When set to 1, pool of each type surface can be specified by
+        changing poolPerType[] in gcsSURF_NODE_Construct.
+*/
+#ifndef gcdDEBUG_OPTION_SPECIFY_POOL
+#           define gcdDEBUG_OPTION_SPECIFY_POOL                 0
+#       endif
+
 #   endif
 #endif
 
 /*
-    gcdDUMP_SWAP_PER_DRAW
+    gcdDUMP_VERIFY_PER_DRAW
 
-        When set to 1, dump swap command for every single draw to make simulation comparison happy.
+        When set to 1, verify RT and images(if used) for every single draw to ease simulation debug.
         Only valid for ES3 driver for now.
 */
-#ifndef gcdDUMP_SWAP_PER_DRAW
-#   define gcdDUMP_SWAP_PER_DRAW                0
+#ifndef gcdDUMP_VERIFY_PER_DRAW
+#   define gcdDUMP_VERIFY_PER_DRAW                0
 #endif
 
+
+
 /*
     gcdDUMP_FRAMERATE
         When set to a value other than zero, averaqe frame rate will be dumped.
@@ -314,7 +386,6 @@
 
     Set to 1 for infinite speed hardware.
     Set to 2 for bypassing the HAL.
-    Set to 3 for bypassing the drivers.
 */
 #ifndef gcdNULL_DRIVER
 #   define gcdNULL_DRIVER  0
@@ -338,6 +409,15 @@
 #   define gcdCMD_BUFFER_SIZE                   (128 << 10)
 #endif
 
+/*
+    gcdCMD_BLT_BUFFER_SIZE
+
+        Number of bytes in a command buffer.
+*/
+#ifndef gcdCMD_BLT_BUFFER_SIZE
+#   define gcdCMD_BLT_BUFFER_SIZE                (1 << 10)
+#endif
+
 /*
     gcdCMD_BUFFERS
 
@@ -376,16 +456,6 @@
 #   define gcdPOWER_CONTROL_DELAY               0
 #endif
 
-/*
-    gcdMIRROR_PAGETABLE
-
-        Enable it when GPUs with old MMU and new MMU exist at same SoC. It makes
-        each GPU use same virtual address to access same physical memory.
-*/
-#ifndef gcdMIRROR_PAGETABLE
-#   define gcdMIRROR_PAGETABLE                  0
-#endif
-
 /*
     gcdMMU_SIZE
 
@@ -393,13 +463,12 @@
         virtual data.
 */
 #ifndef gcdMMU_SIZE
-#if gcdMIRROR_PAGETABLE
-#   define gcdMMU_SIZE                          0x200000
-#else
 #   define gcdMMU_SIZE                          (2048 << 10)
 #endif
-#endif
 
+#ifndef gcdGC355_VGMMU_MEMORY_SIZE_KB
+#   define gcdGC355_VGMMU_MEMORY_SIZE_KB   32
+#endif
 /*
     gcdSECURE_USER
 
@@ -495,15 +564,23 @@
         If the value is 0, no timeout will be checked for.
 */
 #ifndef gcdGPU_TIMEOUT
-#if gcdFPGA_BUILD
-#       define gcdGPU_TIMEOUT                   0
-#       define gcdGPU_2D_TIMEOUT                0
-#   else
-#       define gcdGPU_TIMEOUT                   20000
-#       define gcdGPU_2D_TIMEOUT                4000
-#   endif
+#   define gcdGPU_TIMEOUT                   20000
 #endif
 
+/*
+    gcdGPU_2D_TIMEOUT
+
+        This define specified the number of milliseconds the system will wait
+        before it broadcasts the 2D GPU is stuck.  In other words, it will define
+        the timeout of any operation that needs to wait for the GPU.
+
+        If the value is 0, no timeout will be checked for.
+*/
+#ifndef gcdGPU_2D_TIMEOUT
+#   define gcdGPU_2D_TIMEOUT                4000
+#endif
+
+
 /*
     gcdGPU_ADVANCETIMER
 
@@ -647,14 +724,6 @@
 #   define gcdFRAME_DB_NAME                     "/var/log/frameDB.log"
 #endif
 
-/*
-    gcdDISABLE_CORES_2D3D
-            disable the 2D3D cores for 2D openVG
-*/
-#ifndef gcdDISABLE_CORES_2D3D
-#   define gcdDISABLE_CORES_2D3D                0
-#endif
-
 /*
    gcdPAGED_MEMORY_CACHEABLE
 
@@ -699,27 +768,6 @@
 #   define gcdENABLE_INFINITE_SPEED_HW          0
 #endif
 
-/*
-    gcdMULTI_GPU
-
-    Enable/disable multi-GPU support.
-    0      : Disable multi-GPU support
-    1      : Enable one of the 3D cores
-    [2..X] : Number of 3D GPU Cores
-*/
-#ifndef gcdMULTI_GPU
-#   define gcdMULTI_GPU                         0
-#endif
-
-/*
-    gcdMULTI_GPU_AFFINITY
-
-    Enable/disable the binding of a context to one GPU
-*/
-#ifndef gcdMULTI_GPU_AFFINITY
-#   define gcdMULTI_GPU_AFFINITY                0
-#endif
-
 /*
     gcdPOWEROFF_TIMEOUT
 
@@ -738,34 +786,6 @@
 #   define QNX_SINGLE_THREADED_DEBUGGING        0
 #endif
 
-/*
-    gcdRENDER_THREADS
-
-        Number of render threads. Make it zero, and there will be no render
-        threads.
-*/
-#ifndef gcdRENDER_THREADS
-#   define gcdRENDER_THREADS                    0
-#endif
-
-/*
-    gcdSMP
-
-        This define enables SMP support.
-
-        Currently, it only works on Linux/Android,
-        Kbuild will config it according to whether
-        CONFIG_SMP is set.
-
-*/
-#ifndef gcdSMP
-#ifdef __APPLE__
-#   define gcdSMP                               1
-#else
-#   define gcdSMP                               0
-#endif
-#endif
-
 /*
     gcdSHARED_RESOLVE_BUFFER_ENABLED
 
@@ -782,15 +802,6 @@
 #   define gcdUSE_TRIANGLE_STRIP_PATCH          1
 #endif
 
-/*
-    gcdENABLE_OUTER_CACHE_PATCH
-
-        Enable the outer cache patch.
-*/
-#ifndef gcdENABLE_OUTER_CACHE_PATCH
-#   define gcdENABLE_OUTER_CACHE_PATCH          0
-#endif
-
 /*
     gcdPROCESS_ADDRESS_SPACE
 
@@ -810,7 +821,7 @@
         they can use same GPU virtual address.
 */
 #ifndef gcdSHARED_PAGETABLE
-#   define gcdSHARED_PAGETABLE                  !gcdPROCESS_ADDRESS_SPACE
+#   define gcdSHARED_PAGETABLE                  0
 #endif
 
 #ifndef gcdUSE_PVR
@@ -849,7 +860,7 @@
         is be used to debug.
 */
 #ifndef gcdLINK_QUEUE_SIZE
-#   define gcdLINK_QUEUE_SIZE                   5
+#   define gcdLINK_QUEUE_SIZE                   64
 #endif
 
 /*  gcdALPHA_KILL_IN_SHADER
@@ -862,6 +873,9 @@
 #endif
 
 
+#ifndef gcdPRINT_SWAP_TIME
+#   define gcdPRINT_SWAP_TIME                   0
+#endif
 
 /*
     gcdDVFS
@@ -887,25 +901,6 @@
 #    define gcdGLB27_SHADER_REPLACE_OPTIMIZATION 1
 #endif
 
-/*
-    gcdSTREAM_OUT_BUFFER
-
-        Enable suppport for the secondary stream out buffer.
-*/
-#ifndef gcdSTREAM_OUT_BUFFER
-#   define gcdSTREAM_OUT_BUFFER                 0
-#   define gcdSTREAM_OUT_NAIVE_SYNC             0
-#endif
-
-/*
-    gcdUSE_HARDWARE_CONFIGURATION_TABLES
-
-        Enable the use of hardware configuration tables,
-        instead of query hardware and determine the features.
-*/
-#ifndef gcdUSE_HARDWARE_CONFIGURATION_TABLES
-#   define gcdUSE_HARDWARE_CONFIGURATION_TABLES 0
-#endif
 
 /*
     gcdSUPPORT_SWAP_RECTANGLE
@@ -1021,16 +1016,6 @@
 #   define gcdANDROID_IMPLICIT_NATIVE_BUFFER_SYNC   0
 #endif
 
-/*
-    gcdANDROID_UNALIGNED_LINEAR_COMPOSITION_ADJUST
-
-        Enable source surface address adjust when composition on android.
-        Android only.
-*/
-#ifndef gcdANDROID_UNALIGNED_LINEAR_COMPOSITION_ADJUST
-#   define  gcdANDROID_UNALIGNED_LINEAR_COMPOSITION_ADJUST      1
-#endif
-
 /*
     gcdUSE_WCLIP_PATCH
 
@@ -1044,15 +1029,6 @@
 #   define gcdUSE_NPOT_PATCH                    1
 #endif
 
-/*
-    gcd3DBLIT
-
-        TODO: Should be replaced by feature bit if available.
-*/
-#ifndef gcd3DBLIT
-#   define gcd3DBLIT                            0
-#endif
-
 /*
     gcdINTERNAL_COMMENT
 
@@ -1106,14 +1082,10 @@
 
 #ifndef gcdMOVG
 #   define gcdMOVG                              0
-#if gcdMOVG
-#       define GC355_PROFILER                   1
-#   endif
-#       define gcdENABLE_TS_DOUBLE_BUFFER       1
+#   define gcdENABLE_TS_DOUBLE_BUFFER           1
 #else
 #if gcdMOVG
-#       define GC355_PROFILER                   1
-#       define gcdENABLE_TS_DOUBLE_BUFFER       0
+#   define gcdENABLE_TS_DOUBLE_BUFFER           0
 #else
 #       define gcdENABLE_TS_DOUBLE_BUFFER       1
 #endif
@@ -1125,29 +1097,13 @@
  */
 
 #ifndef gcdINTERRUPT_STATISTIC
-#if defined(LINUX)
+#if defined(LINUX) || defined(__QNXNTO__) || defined(UNDER_CE)
 #   define gcdINTERRUPT_STATISTIC               1
 #else
 #   define gcdINTERRUPT_STATISTIC               0
 #endif
 #endif
 
-/*
-    gcdYINVERTED_RENDERING
-        When it's not zero, we will rendering display buffer
-        with top-bottom direction. All other offscreen rendering
-        will be bottom-top, which follow OpenGL ES spec.
-*/
-#ifndef gcdYINVERTED_RENDERING
-#   define gcdYINVERTED_RENDERING 1
-#endif
-
-#if gcdYINVERTED_RENDERING
-/* disable unaligned linear composition adjust in Y-inverted rendering mode. */
-#   undef  gcdANDROID_UNALIGNED_LINEAR_COMPOSITION_ADJUST
-#   define gcdANDROID_UNALIGNED_LINEAR_COMPOSITION_ADJUST 0
-#endif
-
 /*
     gcdFENCE_WAIT_LOOP_COUNT
         Wait fence, loop count.
@@ -1156,15 +1112,6 @@
 #   define gcdFENCE_WAIT_LOOP_COUNT 100
 #endif
 
-/*
-    gcdHAL_3D_DRAWBLIT
-        When it's not zero, we will enable HAL 3D drawblit
-        to replace client 3dblit.
-*/
-#ifndef gcdHAL_3D_DRAWBLIT
-#   define gcdHAL_3D_DRAWBLIT 1
-#endif
-
 /*
     gcdPARTIAL_FAST_CLEAR
         When it's not zero, partial fast clear is enabled.
@@ -1182,7 +1129,17 @@
         It wil become to a parameter of resolve function.
 */
 #ifndef gcdREMOVE_SURF_ORIENTATION
-#   define gcdREMOVE_SURF_ORIENTATION 0
+#   define gcdREMOVE_SURF_ORIENTATION 1
+#endif
+
+
+
+/*
+    gcdTEST_DEC200
+        Test part for DEC200. Remove when release.
+*/
+#ifndef gcdTEST_DEC200
+#   define gcdTEST_DEC200                       0
 #endif
 
 /*
@@ -1198,9 +1155,18 @@
          disable input devices usage under fb mode to support fb+vdk multi-process
 */
 #ifndef gcdUSE_INPUT_DEVICE
-#   define gcdUSE_INPUT_DEVICE        1
+#   define gcdUSE_INPUT_DEVICE        0
 #endif
 
+/*
+    gcdPERFORMANCE_ANALYSIS
+
+        When set to 1, driver will pass information through loadstate
+        to HW. This loadstate does not impact HW execution.
+*/
+#ifndef gcdPERFORMANCE_ANALYSIS
+#   define gcdPERFORMANCE_ANALYSIS              0
+#endif
 
 /*
     gcdFRAMEINFO_STATISTIC
@@ -1208,7 +1174,11 @@
 */
 #ifndef gcdFRAMEINFO_STATISTIC
 
-#if (defined(DBG) && DBG) || defined(DEBUG) || defined(_DEBUG) || gcdDUMP
+#if (defined(DBG) && DBG) || defined(DEBUG)                || \
+     defined(_DEBUG) || gcdDUMP || gcdPERFORMANCE_ANALYSIS || \
+     (defined(WIN32) && !defined(UNDER_CE))                || \
+     gcdFPGA_BUILD
+
 #   define gcdFRAMEINFO_STATISTIC      1
 #else
 #   define gcdFRAMEINFO_STATISTIC      0
@@ -1217,22 +1187,21 @@
 #endif
 
 /*
-    gcdPACKED_OUTPUT_ADDRESS
-        When it's not zero, ps output is already packed after linked
+    gcdDEC_ENABLE_AHB
+        Enable DEC300 compression AHB mode or not.
 */
-#ifndef gcdPACKED_OUTPUT_ADDRESS
-#   define gcdPACKED_OUTPUT_ADDRESS             1
+#ifndef gcdDEC_ENABLE_AHB
+#   define gcdDEC_ENABLE_AHB                    0
 #endif
 
 /*
-    gcdENABLE_THIRD_PARTY_OPERATION
-        Enable third party operation like tpc or not.
+    gcdENABLE_UNIFIED_CONSTANT
+        Enable unified constant or not.
 */
-#ifndef gcdENABLE_THIRD_PARTY_OPERATION
-#   define gcdENABLE_THIRD_PARTY_OPERATION      1
+#ifndef gcdENABLE_UNIFIED_CONSTANT
+#   define gcdENABLE_UNIFIED_CONSTANT           1
 #endif
 
-
 /*
     Core configurations. By default enable all cores.
 */
@@ -1248,6 +1217,44 @@
 #   define gcdENABLE_VG                         0
 #endif
 
+#ifndef gcdVG_ONLY
+#   define  gcdVG_ONLY  (!gcdENABLE_3D && !gcdENABLE_2D && gcdENABLE_VG)
+#endif
+
+#if defined(WIN32) && !defined(UNDER_CE) && (gcdENABLE_VG == 1)
+
+#ifdef gcdUSE_VX
+#undef gcdUSE_VX
+#endif
+
+#ifdef COMMAND_PROCESSOR_VERSION
+#undef  COMMAND_PROCESSOR_VERSION
+#endif
+
+#ifdef gcdENABLE_TRUST_APPLICATION
+#undef  gcdENABLE_TRUST_APPLICATION
+#endif
+
+#ifdef gcdENABLE_3D
+#undef  gcdENABLE_3D
+#endif
+
+#ifdef gcdENABLE_2D
+#undef  gcdENABLE_2D
+#endif
+
+#define gcdENABLE_3D 0
+#define gcdENABLE_2D 0
+#define gcdUSE_VX 0
+#define COMMAND_PROCESSOR_VERSION 2
+#define gcdENABLE_TRUST_APPLICATION 0
+
+#endif  /* Only for GC355 Cmodel build. */
+
+#ifndef gcdGC355_PROFILER
+#   define gcdGC355_PROFILER                    0
+#endif
+
 #ifndef gcdGC355_MEM_PRINT
 #   define gcdGC355_MEM_PRINT                      0
 #else
@@ -1257,9 +1264,6 @@
 #   endif
 #endif
 
-#ifndef gcdENABLE_UNIFIED_CONSTANT
-#   define gcdENABLE_UNIFIED_CONSTANT           1
-#endif
 
 /*
     gcdRECORD_COMMAND
@@ -1268,4 +1272,133 @@
 #   define gcdRECORD_COMMAND                    0
 #endif
 
+/*
+    gcdALLOC_CMD_FROM_RESERVE
+
+    Provide a way by which location of command buffer can be
+    specified. This is a DEBUG option to limit command buffer
+    to some memory range.
+*/
+#ifndef gcdALLOC_CMD_FROM_RESERVE
+#   define gcdALLOC_CMD_FROM_RESERVE            0
+#endif
+
+/*
+    gcdBOUNDARY_CHECK
+
+    When enabled, add bounary before and after a range of
+    GPU address. So overflow can be trapped by MMU exception.
+    This is a debug option for new MMU and gcdUSE_MMU_EXCEPTION
+    is enabled.
+*/
+#ifndef gcdBOUNDARY_CHECK
+#   define gcdBOUNDARY_CHECK                    0
+#endif
+
+/*
+    gcdRENDER_QUALITY_CHECK
+
+    When enabled, we disable performance opt patch
+    to get know rendering quality comparing with other vendor.
+*/
+#ifndef gcdRENDER_QUALITY_CHECK
+#   define gcdRENDER_QUALITY_CHECK              0
+#endif
+
+/*
+    gcdSYSTRACE
+
+    When enabled, we embed systrace in function header/footer
+    to gather time information on linux platforms include android.
+    '1' to trace API (EGL, ES11, ES2x, ES3x, etc)
+    '2' to trace HAL (except compiler)
+    '4' to trace HAL compiler
+    See gc_hal_user_debug.c for more detailed trace zones.
+*/
+#ifndef gcdSYSTRACE
+#   define gcdSYSTRACE                          0
+#endif
+
+#ifndef gcdENABLE_APPCTXT_BLITDRAW
+#   define gcdENABLE_APPCTXT_BLITDRAW                     0
+#endif
+
+/*
+    gcdENABLE_TRUST_APPLICATION
+
+    When enabled, trust application is used to handle 'security' registers.
+
+    1) If HW doesn't have robust and security feature, this option is meaningless.
+    2) If HW have robust and security and this option is not enable,
+       security registers are handled by non secure driver. It is for
+       platform doesn't want/need to use trust zone.
+*/
+#ifndef gcdENABLE_TRUST_APPLICATION
+#if (defined(_WIN32) && !defined(UNDER_CE)) || (defined (LINUX) && !defined(EMULATOR))
+#   define gcdENABLE_TRUST_APPLICATION          1
+#else
+#   define gcdENABLE_TRUST_APPLICATION          0
+#endif
+#endif
+
+/* Disable gcdENABLE_TRUST_APPLICATION when oboslete gcdSECURITY enabled. */
+#if gcdSECURITY
+#undef gcdENABLE_TRUST_APPLICATION
+#define gcdENABLE_TRUST_APPLICATION             0
+#endif
+
+#ifndef gcdMMU_SECURE_AREA_SIZE
+#   define gcdMMU_SECURE_AREA_SIZE              128
+#endif
+
+/*
+VIV:gcdUSE_MMU_EXCEPTION
+
+    When enabled, enable and check exception interrupt raised by MMU.
+*/
+#ifndef gcdUSE_MMU_EXCEPTION
+#   define gcdUSE_MMU_EXCEPTION                 1
+#endif
+
+#ifndef gcdVX_OPTIMIZER
+#   define gcdVX_OPTIMIZER                      0
+#endif
+
+#ifndef gcdOCL_READ_IMAGE_OPTIMIZATION
+#   define gcdOCL_READ_IMAGE_OPTIMIZATION       0
+#endif
+
+#ifndef gcdALLOC_ON_FAULT
+#   define gcdALLOC_ON_FAULT                    0
+#endif
+
+/*
+    gcdDISABLE_GPU_VIRTUAL_ADDRESS
+
+        When enabled, disable MMU and all virtual allocated from MMU.
+*/
+#ifndef gcdDISABLE_GPU_VIRTUAL_ADDRESS
+#   define gcdDISABLE_GPU_VIRTUAL_ADDRESS       0
+#endif
+
+/*
+    gcd2D_COMPRESSION_DEC400_ALIGN_MODE
+
+        Only for DEC400 compression.
+        Set 0 as 16bytes aligned. 1 as 32bytes aligned. 2 as 64bytes aligned.
+        Default is 0 which means 32bytes aligned.
+*/
+#ifndef gcd2D_COMPRESSION_DEC400_ALIGN_MODE
+#   define gcd2D_COMPRESSION_DEC400_ALIGN_MODE  1
+#endif
+
+/*
+    gcdENABLE_KERNEL_FENCE
+        When enabled, use kernel fence to do resource tracking.
+*/
+#ifndef gcdENABLE_KENREL_FENCE
+#   define gcdENABLE_KERNEL_FENCE               0
+#endif
+
+
 #endif /* __gc_hal_options_h_ */
