@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, Sonos, Inc.
+ * Copyright (c) 2011-2019, Sonos, Inc.
  *
  * SPDX-License-Identifier:	GPL-2.0
  *
@@ -19,8 +19,6 @@
 #include <linux/module.h>
 #include <linux/gpio.h>
 #include <linux/of_gpio.h>
-#include <net/netlink.h>
-#include <linux/rtnetlink.h>
 
 #define MV88E6020_REG_PHY_CNTL        0x00
 #define MV88E6020_REG_PHY_SPEC_CNTL   0x10
@@ -119,7 +117,7 @@ static unsigned long mv88e6020_errata_count=0;
 
 static struct proc_dir_entry *procdir;
 
-static int switch_to_cpu_link_status = 0;
+static int swicth_to_cpu_link_status = 0;
 
 typedef struct _mv88e6020_env {
 	char	*name;
@@ -136,52 +134,13 @@ static mv88e6020_env_t mv88e6020_named_entries[] = {
 	{ NULL,		0},
 };
 
-void mv88e6020_announce_linkup(struct net_device *dev)
-{
-	struct net *rtnl;
-	struct sk_buff *skb;
-	struct nlmsghdr *nlh;
-	unsigned char *b;
-	int size = NLMSG_SPACE(1024);
-
-	if (dev == NULL) {
-		printk("%s: dev NULL\n", __func__);
-		return;
-	}
-	rtnl = dev_net(dev);
-
-	printk("%s: announcing link status change\n", __func__);
-
-	skb = nlmsg_new(size, GFP_ATOMIC);
-	if (!skb) {
-		netlink_set_err(rtnl->rtnl, 0, RTMGRP_Rincon, ENOBUFS);
-		return;
-	}
-	b = skb->tail;
-	nlh = nlmsg_put(skb, 0, 0, RWM_MII, 0, 0);
-	if ( !nlh )
-		goto nlmsg_failure;
-	nla_put(skb, RWA_DEV_NAME, IFNAMSIZ, dev->name);
-	nlh->nlmsg_len = skb->tail - b;
-
-	NETLINK_CB(skb).dst_group = RTMGRP_Rincon;
-	nlmsg_end(skb, nlh);
-	rtnl_notify(skb, rtnl, 0, RTMGRP_Rincon, NULL, GFP_ATOMIC);
-	return;
-
-nlmsg_failure:
-	kfree_skb(skb);
-	netlink_set_err(rtnl->rtnl, 0, RTMGRP_Rincon, EINVAL);
-	printk("%s: phy announce failed.\n", __func__);
-}
-
 static int mv88e6020_busychk(struct phy_device *phydev)
 {
 	u16 reg = 0;
 	u32 timeout = MV88E6020_PHY_TIMEOUT;
 
 	do {
-		reg = mdiobus_read(phydev->bus, MV88E6020_GLOBAL2_REGS_ADDR,
+		reg = mdiobus_read(phydev->mdio.bus, MV88E6020_GLOBAL2_REGS_ADDR,
 			MV88E6020_SMI_PHY_ADDR_REG);
 		if (timeout-- == 0) {
 			printk("SMI busy timeout\n");
@@ -208,7 +167,7 @@ static int mv88e6020_phy_read(int phynum, u32 regnum)
 		printk("%s: switch is busy\n", __func__);
 		return 0;
 	}
-	err = mdiobus_write(mv88e6020_phydev->bus, MV88E6020_GLOBAL2_REGS_ADDR,
+	err = mdiobus_write(mv88e6020_phydev->mdio.bus, MV88E6020_GLOBAL2_REGS_ADDR,
 			MV88E6020_SMI_PHY_ADDR_REG, cmd);
 	if (err < 0) {
 		printk("%s: write error %d\n", __func__, err);
@@ -220,7 +179,7 @@ static int mv88e6020_phy_read(int phynum, u32 regnum)
 		return -EIO;
 	}
 
-	err = mdiobus_read(mv88e6020_phydev->bus, MV88E6020_GLOBAL2_REGS_ADDR,
+	err = mdiobus_read(mv88e6020_phydev->mdio.bus, MV88E6020_GLOBAL2_REGS_ADDR,
 			MV88E6020_SMI_PHY_DATA_REG);
 	if (err < 0) {
 		printk("%s: read error %d\n", __func__, err);
@@ -238,7 +197,7 @@ static int mv88e6020_phy_write(int phynum, u32 regnum, u16 val)
 		return 0;
 	}
 
-	err = mdiobus_write(mv88e6020_phydev->bus, MV88E6020_GLOBAL2_REGS_ADDR,
+	err = mdiobus_write(mv88e6020_phydev->mdio.bus, MV88E6020_GLOBAL2_REGS_ADDR,
 			MV88E6020_SMI_PHY_DATA_REG, val);
 	if (err < 0) {
 		printk("%s: write error %d\n", __func__, err);
@@ -252,7 +211,7 @@ static int mv88e6020_phy_write(int phynum, u32 regnum, u16 val)
 
 	cmd = MV88E6020_SMI_PHY_ADDR_WRITE |
 		((MV88E6020_SMI_PHY_ADDR_BASE | phynum) << 5) | regnum;
-	err = mdiobus_write(mv88e6020_phydev->bus, MV88E6020_GLOBAL2_REGS_ADDR,
+	err = mdiobus_write(mv88e6020_phydev->mdio.bus, MV88E6020_GLOBAL2_REGS_ADDR,
 			MV88E6020_SMI_PHY_ADDR_REG, cmd);
 	if (err < 0) {
 		printk("%s: write addr error %d\n", __func__, err);
@@ -284,23 +243,23 @@ static unsigned long get_stat_value(char port, int opcode) {
 	unsigned long ret = 0;
 	int count = 0;
 
-	while((mdiobus_read(mv88e6020_phydev->bus, MV88E6020_GLOBAL1_REGS_ADDR, MV88E6020_STATS_OP_REG) & MV88E6020_STATS_BUSY) && count < 100) {
+	while((mdiobus_read(mv88e6020_phydev->mdio.bus, MV88E6020_GLOBAL1_REGS_ADDR, MV88E6020_STATS_OP_REG) & MV88E6020_STATS_BUSY) && count < 100) {
 		udelay(10);
 		count++;
 	}
-	if((mdiobus_read(mv88e6020_phydev->bus, MV88E6020_GLOBAL1_REGS_ADDR, MV88E6020_STATS_OP_REG) & MV88E6020_STATS_BUSY)) {
+	if((mdiobus_read(mv88e6020_phydev->mdio.bus, MV88E6020_GLOBAL1_REGS_ADDR, MV88E6020_STATS_OP_REG) & MV88E6020_STATS_BUSY)) {
 		printk(KERN_WARNING "mv88e6020: Status register busy!\n");
 		return -EBUSY;
 	}
 
-	mdiobus_write(mv88e6020_phydev->bus, MV88E6020_GLOBAL1_REGS_ADDR, MV88E6020_STATS_OP_REG,
+	mdiobus_write(mv88e6020_phydev->mdio.bus, MV88E6020_GLOBAL1_REGS_ADDR, MV88E6020_STATS_OP_REG,
 				MV88E6020_STATS_BUSY |
 				MV88E6020_STATS_READ |
 				MV88E6020_STATS_PORT(port) |
 				opcode);
-	ret = (unsigned long)((mdiobus_read(mv88e6020_phydev->bus,
+	ret = (unsigned long)((mdiobus_read(mv88e6020_phydev->mdio.bus,
 				MV88E6020_GLOBAL1_REGS_ADDR, MV88E6020_STATS_READ_REG_HI) << 16) |
-				mdiobus_read(mv88e6020_phydev->bus,
+				mdiobus_read(mv88e6020_phydev->mdio.bus,
 				MV88E6020_GLOBAL1_REGS_ADDR, MV88E6020_STATS_READ_REG_LO));
 
 	return ret;
@@ -310,16 +269,16 @@ static int flush_stat_regs(void) {
 	int ret = 0;
 	int count = 0;
 
-	while((mdiobus_read(mv88e6020_phydev->bus, MV88E6020_GLOBAL1_REGS_ADDR, MV88E6020_STATS_OP_REG) & MV88E6020_STATS_BUSY) && count < 100) {
+	while((mdiobus_read(mv88e6020_phydev->mdio.bus, MV88E6020_GLOBAL1_REGS_ADDR, MV88E6020_STATS_OP_REG) & MV88E6020_STATS_BUSY) && count < 100) {
 		udelay(10);
 		count++;
 	}
-	if((mdiobus_read(mv88e6020_phydev->bus, MV88E6020_GLOBAL1_REGS_ADDR, MV88E6020_STATS_OP_REG) & MV88E6020_STATS_BUSY)) {
+	if((mdiobus_read(mv88e6020_phydev->mdio.bus, MV88E6020_GLOBAL1_REGS_ADDR, MV88E6020_STATS_OP_REG) & MV88E6020_STATS_BUSY)) {
 		printk(KERN_WARNING "mv88e6020: Status register busy!\n");
 		return -EBUSY;
 	}
 
-	mdiobus_write(mv88e6020_phydev->bus, MV88E6020_GLOBAL1_REGS_ADDR, MV88E6020_STATS_OP_REG,
+	mdiobus_write(mv88e6020_phydev->mdio.bus, MV88E6020_GLOBAL1_REGS_ADDR, MV88E6020_STATS_OP_REG,
 				MV88E6020_STATS_BUSY |
 				MV88E6020_STATS_FLUSH);
 
@@ -368,7 +327,7 @@ int mv88e6020_get_stats(char port, struct net_device_stats *phy_stats) {
 	return 0;
 }
 
-static void mv88e6020_check_link(void)
+static void mv88e6020_check_link(struct phy_device *phydev)
 {
 	int link, phy;
 
@@ -408,13 +367,9 @@ static void mv88e6020_check_link(void)
 		}
 	}
 
-	switch_to_cpu_link_status = link;
+	swicth_to_cpu_link_status = link;
 
-	if (mv88e6020_phydev == NULL) {
-		printk("mv88e6020_phydev NULL\n");
-		return;
-	}
-	mv88e6020_announce_linkup(mv88e6020_phydev->attached_dev);
+	sonos_announce_linkup(phydev->attached_dev);
 }
 
 void mv88e6020_sonos_get_port_status(unsigned int port, struct ethtool_cmd *cmd)
@@ -434,16 +389,15 @@ static void mv88e6020_link_change(struct work_struct *work)
 		mv88e6020_phy_read(0, MV88E6020_REG_INT_STATUS);
 		mv88e6020_phy_read(1, MV88E6020_REG_INT_STATUS);
 
-		mv88e6020_check_link();
+		mv88e6020_check_link(mv88e6020_phydev);
 
-		gb2int = mdiobus_read(mv88e6020_phydev->bus,
-				MV88E6020_GLOBAL2_REGS_ADDR,
-				MV88E6020_PHY_INTR_STATUS_REG);
+		gb2int = mdiobus_read(mv88e6020_phydev->mdio.bus, MV88E6020_GLOBAL2_REGS_ADDR,
+							  MV88E6020_PHY_INTR_STATUS_REG);
 		gb2int &= mask;
 
 		if (pass++ > 1000) {
 			printk("%s: exceeded max attempts to clear interrupt\n", __func__);
-			mv88e6020_max_pass = pass;
+		    mv88e6020_max_pass = pass;
 			return;
 		}
 	}
@@ -455,21 +409,21 @@ static void mv88e6020_link_change(struct work_struct *work)
 	}
 }
 
-static void mv88e6020_init_irq(struct phy_device *phydev)
+void mv88e6020_init_irq(void)
 {
 	int ret;
         struct device_node *node = NULL;
 
         node = of_find_compatible_node(NULL, NULL, "Sonos,mv88e6020_phy");
         if (node) {
-                mv88e6020_irq = gpio_to_irq(of_get_named_gpio(node, "interrupts", 0));
+                mv88e6020_irq = gpio_to_irq(of_get_named_gpio(node,
+			"interrupts", 0));
         }
 
 	if ( mv88e6020_irq < 0 ) {
 		printk("%s: no irq\n", __func__);
 		return;
 	}
-
 	INIT_WORK(&mv88e6020_link_isr_work, mv88e6020_link_change);
 
 	ret = request_irq(mv88e6020_irq, mv88e6020_isr, IRQF_TRIGGER_LOW,
@@ -504,9 +458,9 @@ static void mv88e6020_errata34(struct work_struct *work)
 		}
 	}
 
+	schedule_delayed_work(&mv88e6020_errata34_work, HZ);
 }
 
-#ifdef CONFIG_PROC_FS
 static int mv88e6020_proc_read(struct seq_file *m, void *v)
 {
 	seq_printf(m, "Interupts %ld\n"
@@ -528,7 +482,7 @@ static int mv88e6020_proc_dump(struct seq_file *m, void *v)
 	int reg, val;
 
 	for (reg = 0; reg <= 0x1f; reg++) {
-		val = mdiobus_read(mv88e6020_phydev->bus, devAddr, reg);
+		val = mdiobus_read(mv88e6020_phydev->mdio.bus, devAddr, reg);
 		seq_printf(m, "%04x ", val);
 		if ((reg & 0x0f) == 0x0f)
 			seq_printf(m, "\n");
@@ -612,7 +566,7 @@ static ssize_t mv88e6020_port_proc_write(struct file *file,
 				if ( kstrtol(buf+3, 16, &longtemp) != 0 )
 					return -EIO;
 				regnum = longtemp;
-				mdiobus_write(mv88e6020_phydev->bus, devaddr,
+				mdiobus_write(mv88e6020_phydev->mdio.bus, devaddr,
 					regnum, val);
 			}
 		}
@@ -695,19 +649,100 @@ static const struct file_operations mv88e6020_phy_proc_fops = {
 	.write		= mv88e6020_phy_proc_write,
 };
 
-void m88e6020_proc_fs(void)
+static int m88e6020_config_init(struct phy_device *phydev)
 {
+	int phy, err, ident;
 	mv88e6020_env_t *entry;
 	int var_num;
 
-	if ( procdir )
-		return;
+	mv88e6020_phydev = phydev;
+
+	for (phy = 0; phy <= 1; phy++) {
+		int scr;
+		scr = mv88e6020_phy_read(phy, MV88E6020_REG_PHY_SPEC_CNTL);
+		if (scr & 0x4000) {
+			printk("%s: phy%d: disabling energy detect only\n",
+				__func__, phy);
+                        mv88e6020_phy_write(phy, MV88E6020_REG_PHY_SPEC_CNTL,
+				0x0138);
+		} else {
+			printk("%s: phy%d: energy detect already disabled\n",
+				__func__, phy);
+		}
+	}
+
+	err = mdiobus_write(mv88e6020_phydev->mdio.bus, MV88E6020_PORT6_SWITCH_ADDR,
+			MV88E6020_SWITCH_REG_PORT_CONTROL, 0);
+	if (err < 0) {
+		 printk("%s: disable port6 write error %d\n", __func__, err);
+	}
+
+	for (phy = 0; phy <= 1; phy++) {
+		int id2, id3;
+		id2 = mv88e6020_phy_read(phy, MII_PHYSID1);
+		id3 = mv88e6020_phy_read(phy, MII_PHYSID2);
+		printk("%s: PHY%d: ID %04x %04x\n", __func__, phy, id2, id3);
+	}
+
+	ident = mdiobus_read(mv88e6020_phydev->mdio.bus, MV88E6020_PORT0_SWITCH_ADDR,
+			MV88E6020_SWITCH_REG_IDENT);
+	printk("switch port0: product number 0x%03x, revision 0x%1x\n",
+			(ident >> 4), (ident & 0xf));
+	ident = mdiobus_read(mv88e6020_phydev->mdio.bus, MV88E6020_PORT1_SWITCH_ADDR,
+			MV88E6020_SWITCH_REG_IDENT);
+	printk("switch port1: product number 0x%03x, revision 0x%1x\n",
+			(ident >> 4), (ident & 0xf));
+	ident = mdiobus_read(mv88e6020_phydev->mdio.bus, MV88E6020_PORT5_SWITCH_ADDR,
+			MV88E6020_SWITCH_REG_IDENT);
+	printk("switch port5: product number 0x%03x, revision 0x%1x\n",
+			(ident >> 4), (ident & 0xf));
+
+	mdiobus_read(mv88e6020_phydev->mdio.bus, MV88E6020_GLOBAL1_REGS_ADDR,
+			MV88E6020_REG_SWITCH_INTR_STATUS);
+	mv88e6020_phy_read(0, MV88E6020_REG_INT_STATUS);
+	mv88e6020_phy_read(1, MV88E6020_REG_INT_STATUS);
+
+	for (phy = 0; phy <= 1; phy++) {
+		mv88e6020_phy_write(phy, MV88E6020_REG_INT_MASK,
+			MV88E6020_INT_LINK);
+	}
+
+	err = mdiobus_write(mv88e6020_phydev->mdio.bus, MV88E6020_GLOBAL2_REGS_ADDR,
+			MV88E6020_PHY_INTR_MASK_REG,
+			MV88E6020_GLOBAL2_PHY_INT(0) |
+			MV88E6020_GLOBAL2_PHY_INT(1));
+	if (err < 0) {
+		printk("%s: write error %d\n", __func__, err);
+	}
+
+	err = mdiobus_write(mv88e6020_phydev->mdio.bus, MV88E6020_GLOBAL1_REGS_ADDR,
+			MV88E6020_REG_SWITCH_INTR_MASK, MV88E6020_DEVICE_INT);
+	if (err < 0) {
+		printk("%s: write error %d\n", __func__, err);
+	}
+
+	err = mdiobus_write(mv88e6020_phydev->mdio.bus,
+			MV88E6020_PORT5_SWITCH_ADDR,
+			0x01, 0xc03d);
+	if (err < 0) {
+		printk("%s: error force internal link up %d\n", __func__, err);
+	}
+
+	mv88e6020_init_irq();
+
+	mv88e6020_check_link(mv88e6020_phydev);
+
+	if (flush_stat_regs())
+		printk(KERN_WARNING "%s: Couldn't flush statistics registers.\n", __func__);
+
+	INIT_DELAYED_WORK(&mv88e6020_errata34_work, mv88e6020_errata34);
+	schedule_delayed_work(&mv88e6020_errata34_work, HZ);
 
 	procdir = proc_mkdir(MV88E6020_PROC_DIR, NULL);
 	if (procdir == NULL) {
 		printk("Couldn't create base dir /proc/%s\n",
 			MV88E6020_PROC_DIR);
-		return;
+		return -ENOMEM;
 	}
 
 	if (! proc_create_data("data", 0, procdir,
@@ -737,110 +772,11 @@ void m88e6020_proc_fs(void)
 			&mv88e6020_phy_proc_fops, (void *)var_num))
 			 goto cleanup;
 	}
-	return;
+	return 0;
+
 cleanup:
 	remove_proc_subtree(MV88E6020_PROC_DIR, NULL);
 	procdir = NULL;
-	return;
-}
-#endif
-
-#ifdef CONFIG_PM
-static int m88e6020_suspend(struct phy_device *phydev)
-{
-        return 0;
-}
-
-static int m88e6020_resume(struct phy_device *phydev)
-{
-        return 0;
-}
-#endif
-
-static int m88e6020_config_init(struct phy_device *phydev)
-{
-	int phy, err, ident;
-
-	mv88e6020_phydev = phydev;
-
-	for (phy = 0; phy <= 1; phy++) {
-		int scr;
-		scr = mv88e6020_phy_read(phy, MV88E6020_REG_PHY_SPEC_CNTL);
-		if (scr & 0x4000) {
-			printk("%s: phy%d: disabling energy detect only\n",
-				__func__, phy);
-                        mv88e6020_phy_write(phy, MV88E6020_REG_PHY_SPEC_CNTL,
-				0x0138);
-		} else {
-			printk("%s: phy%d: energy detect already disabled\n",
-				__func__, phy);
-		}
-	}
-
-	err = mdiobus_write(mv88e6020_phydev->bus, MV88E6020_PORT6_SWITCH_ADDR,
-			MV88E6020_SWITCH_REG_PORT_CONTROL, 0);
-	if (err < 0) {
-		 printk("%s: disable port6 write error %d\n", __func__, err);
-	}
-
-	for (phy = 0; phy <= 1; phy++) {
-		int id2, id3;
-		id2 = mv88e6020_phy_read(phy, MII_PHYSID1);
-		id3 = mv88e6020_phy_read(phy, MII_PHYSID2);
-		printk("%s: PHY%d: ID %04x %04x\n", __func__, phy, id2, id3);
-	}
-
-	ident = mdiobus_read(mv88e6020_phydev->bus, MV88E6020_PORT0_SWITCH_ADDR,
-			MV88E6020_SWITCH_REG_IDENT);
-	printk("switch port0: product number 0x%03x, revision 0x%1x\n",
-			(ident >> 4), (ident & 0xf));
-	ident = mdiobus_read(mv88e6020_phydev->bus, MV88E6020_PORT1_SWITCH_ADDR,
-			MV88E6020_SWITCH_REG_IDENT);
-	printk("switch port1: product number 0x%03x, revision 0x%1x\n",
-			(ident >> 4), (ident & 0xf));
-	ident = mdiobus_read(mv88e6020_phydev->bus, MV88E6020_PORT5_SWITCH_ADDR,
-			MV88E6020_SWITCH_REG_IDENT);
-	printk("switch port5: product number 0x%03x, revision 0x%1x\n",
-			(ident >> 4), (ident & 0xf));
-
-	mdiobus_read(mv88e6020_phydev->bus, MV88E6020_GLOBAL1_REGS_ADDR,
-			MV88E6020_REG_SWITCH_INTR_STATUS);
-	mv88e6020_phy_read(0, MV88E6020_REG_INT_STATUS);
-	mv88e6020_phy_read(1, MV88E6020_REG_INT_STATUS);
-
-	for (phy = 0; phy <= 1; phy++) {
-		mv88e6020_phy_write(phy, MV88E6020_REG_INT_MASK,
-			MV88E6020_INT_LINK);
-	}
-
-	err = mdiobus_write(mv88e6020_phydev->bus, MV88E6020_GLOBAL2_REGS_ADDR,
-			MV88E6020_PHY_INTR_MASK_REG,
-			MV88E6020_GLOBAL2_PHY_INT(0) |
-			MV88E6020_GLOBAL2_PHY_INT(1));
-	if (err < 0) {
-		printk("%s: write error %d\n", __func__, err);
-	}
-
-	err = mdiobus_write(mv88e6020_phydev->bus, MV88E6020_GLOBAL1_REGS_ADDR,
-			MV88E6020_REG_SWITCH_INTR_MASK, MV88E6020_DEVICE_INT);
-	if (err < 0) {
-		printk("%s: write error %d\n", __func__, err);
-	}
-
-	mv88e6020_init_irq(phydev);
-
-	mv88e6020_check_link();
-
-	if (flush_stat_regs())
-		printk(KERN_WARNING "%s: Couldn't flush statistics registers.\n", __func__);
-
-	INIT_DELAYED_WORK(&mv88e6020_errata34_work, mv88e6020_errata34);
-	schedule_delayed_work(&mv88e6020_errata34_work, HZ);
-	mdiobus_write(mv88e6020_phydev->bus, MV88E6020_PORT5_SWITCH_ADDR, 1, 0xC03D);
-
-#ifdef CONFIG_PROC_FS
-	m88e6020_proc_fs();
-#endif
 	return 0;
 }
 
@@ -851,7 +787,7 @@ static int m88e6020_config_aneg(struct phy_device *phydev)
 
 static int m88e6020_genphy_read_status(struct phy_device *phydev)
 {
-	if ( switch_to_cpu_link_status ) {
+	if ( swicth_to_cpu_link_status ) {
 		phydev->speed = SPEED_100;
 		phydev->duplex = DUPLEX_FULL;
 		phydev->link = 1;
@@ -873,56 +809,52 @@ static int m88e6020_config_intr(struct phy_device *phydev)
 	return 0;
 }
 
-static struct phy_driver marvell_drivers[] = {
-	{
-		.phy_id        = 0x00000200,
-		.phy_id_mask   = 0x000fff00,
-		.name          = "Marvell 88E6020",
-		.features      = PHY_BASIC_FEATURES,
-		.flags         = PHY_HAS_INTERRUPT,
-		.config_init   = &m88e6020_config_init,
-		.config_aneg   = &m88e6020_config_aneg,
-		.read_status   = &m88e6020_genphy_read_status,
-		.ack_interrupt = &m88e6020_ack_interrupt,
-		.config_intr   = &m88e6020_config_intr,
-#ifdef CONFIG_PM
-		.suspend       = &m88e6020_suspend,
-		.resume        = &m88e6020_resume,
-#endif
-		.driver = { .owner = THIS_MODULE },
-	},
+static struct phy_driver marvell_drivers = {
+	.phy_id = 0x00000200,
+	.phy_id_mask = 0x000fff00,
+	.name = "Marvell 88E6020",
+	.features = PHY_GBIT_FEATURES,
+	.flags = PHY_HAS_INTERRUPT,
+	.config_init = &m88e6020_config_init,
+	.config_aneg = &m88e6020_config_aneg,
+	.read_status = &m88e6020_genphy_read_status,
+	.ack_interrupt = &m88e6020_ack_interrupt,
+	.config_intr = &m88e6020_config_intr,
 };
 
 static int __init mv88e6020_init(void)
 {
 	int ret = 0;
 
-	switch_to_cpu_link_status = 0;
 	mv88e6020_phydev = NULL;
-	procdir = NULL;
+	ret = phy_driver_register(&marvell_drivers, THIS_MODULE);
 
-	ret = phy_drivers_register(marvell_drivers,
-		ARRAY_SIZE(marvell_drivers));
+	if ( ret )
+		goto err;
 
+	swicth_to_cpu_link_status = 0;
+	return 0;
+err:
 	return ret;
 }
 
 static void __exit mv88e6020_exit(void)
 {
-#ifdef CONFIG_PROC_FS
-	remove_proc_subtree(MV88E6020_PROC_DIR, NULL);
-#endif
-	phy_drivers_unregister(marvell_drivers,
-			ARRAY_SIZE(marvell_drivers));
+	printk("%s: done\n", __func__);
+	if ( procdir )
+		remove_proc_subtree(MV88E6020_PROC_DIR, NULL);
+	phy_driver_unregister(&marvell_drivers);
 	mv88e6020_phydev = NULL;
 }
 
-int mv88e6020_phy_power_down(struct phy_device *phydev, int on)
+#if defined(CONFIG_SONOS_PARAMOUNT) || defined(CONFIG_SONOS_NEPTUNE)
+int phy_power_down(struct phy_device *phydev, int on)
 {
     return 0;
 }
+#endif
 
-subsys_initcall(mv88e6020_init);
+module_init(mv88e6020_init);
 module_exit(mv88e6020_exit);
 
 static struct mdio_device_id __maybe_unused marvell_tbl[] = {
