@@ -1,17 +1,10 @@
 /*
- * drivers/net/phy/ter21x3.c
+ * Copyright (c) 2020, Sonos, Inc.
  *
- * Driver for Teridian 21x3 PHY
- *
- * Author: SONOS
- *	 modification of at803x by Matus Ujhelyi <ujhelyi.m@gmail.com>
- *
- * This program is free software; you can redistribute  it and/or modify it
- * under  the terms of  the GNU General  Public License as published by the
- * Free Software Foundation;  either version 2 of the  License, or (at your
- * option) any later version.
+ * SPDX-License-Identifier:     GPL-2.0
  */
 
+
 #include <linux/phy.h>
 #include <linux/module.h>
 #include <linux/string.h>
@@ -20,11 +13,12 @@
 #include <linux/gpio.h>
 #include <linux/of_gpio.h>
 #include <linux/irq.h>
+#include <linux/kernel.h>
+#include <linux/proc_fs.h>
 
 MODULE_DESCRIPTION("Teridian 21x3 PHY driver");
 MODULE_LICENSE("GPL");
 
-#ifdef CONFIG_SONOS
 static struct proc_dir_entry *procdir = NULL;
 #define TER21X3_PROC_DIR			"driver/phy"
 
@@ -36,9 +30,6 @@ static struct proc_dir_entry *procdir = NULL;
 #define TER21X3_DIAG_SPEED			1 << 10
 
 struct ethtool_cmd ter21x3_status[2] = {{0}};
-static struct phy_device *ter21x3_phydev = NULL;
-static struct net_device *ter21x3_mac_device = NULL;
-void (*ter21x3_link_up)(struct net_device *ndev) = NULL;
 
 int phy_power_down(struct phy_device *phydev, int on)
 {
@@ -99,11 +90,11 @@ static ssize_t ter21x3_phy_proc_write(struct file *file,
 		peq = strchr(buf, '=');
 		if (peq != NULL) {
 			*peq = 0;
-			if ( strict_strtol(peq+1, 16, &longtemp) != 0 )
+			if ( kstrtol(peq+1, 16, &longtemp) != 0 )
 				return -EIO;
 			val = longtemp;
 			if (strncmp(buf, "reg", 3) == 0) {
-				if (strict_strtol(buf+3, 16, &longtemp) != 0 )
+				if (kstrtol(buf+3, 16, &longtemp) != 0 )
 					return -EIO;
 				regnum = longtemp;
 				phy_write(phydev, regnum, val);
@@ -122,18 +113,12 @@ static const struct file_operations ter21x3_phy_proc_fops = {
 	.write		= ter21x3_phy_proc_write,
 };
 
-static void ter21x3_check_link(void)
+static void ter21x3_check_link(struct phy_device *phydev)
 {
 	int link;
-	struct phy_device *phydev;
 	int val;
 	unsigned int prev_speed = ter21x3_status[0].speed;
-	if (ter21x3_phydev == NULL) {
-		printk("ter21x3_phydev NULL\n");
-		return;
-	}
 
-	phydev = ter21x3_phydev;
 	link = 0;
 	val = phy_read(phydev, MII_BMSR);
 	if (val & BMSR_LSTATUS) {
@@ -156,20 +141,19 @@ static void ter21x3_check_link(void)
 
 	if (ter21x3_status[0].speed != prev_speed) {
 		if (ter21x3_status[0].speed == 0) {
-			printk("ath802: port down\n");
+			printk("ter21x3: port down\n");
+			netif_carrier_off(phydev->attached_dev);
 		} else {
-			printk("ath802: port up, speed %d,"
+			printk("ter21x3: port up, speed %d,"
 				" %s duplex\n",
 				ter21x3_status[0].speed,
 				(ter21x3_status[0].duplex ==
 					DUPLEX_FULL) ? "full" : "half");
+			netif_carrier_on(phydev->attached_dev);
 		}
 	}
 
-
-	if (ter21x3_link_up != NULL) {
-		(*ter21x3_link_up)(ter21x3_mac_device);
-	}
+	sonos_announce_linkup(phydev->attached_dev);
 }
 
 static int ter21x3_ack_interrupt(struct phy_device *phydev)
@@ -177,7 +161,7 @@ static int ter21x3_ack_interrupt(struct phy_device *phydev)
 	int err;
 
 	err = phy_read(phydev, TER21X3_INTR_STATUS);
-	ter21x3_check_link();
+	ter21x3_check_link(phydev);
 
 	return (err < 0) ? err : 0;
 }
@@ -200,12 +184,12 @@ static int ter21x3_config_init(struct phy_device *phydev)
 	int val;
 	u32 features;
 	struct device_node *node = NULL;
-	int ethernet_phy_interrupt;
+	int ethernet_phy_interrupt = -1;
 
-	ter21x3_phydev = phydev;
 	node = of_find_compatible_node(NULL, NULL, "Sonos,ter21x3_phy");
 	if (node) {
-		ethernet_phy_interrupt = gpio_to_irq(of_get_named_gpio(node, "interrupts", NULL));
+		ethernet_phy_interrupt = gpio_to_irq(of_get_named_gpio(node,
+			"interrupts", 0));
 	} else {
 		printk(KERN_ERR "could not find device node for interrupts\n");
 	}
@@ -250,17 +234,18 @@ static int ter21x3_config_init(struct phy_device *phydev)
 	phydev->supported = features;
 	phydev->advertising = features;
 
-	if ( procdir == NULL )
+	if ( procdir == NULL ) {
 		procdir = proc_mkdir(TER21X3_PROC_DIR, NULL);
-	if (procdir == NULL) {
-		printk("Couldn't create base dir /proc/%s\n",
-			TER21X3_PROC_DIR);
-		return -ENOMEM;
-	}
-	if (! proc_create_data("reg", 0, procdir,
+		if (procdir == NULL) {
+			printk("Couldn't create base dir /proc/%s\n",
+				TER21X3_PROC_DIR);
+			return -ENOMEM;
+		}
+		if (! proc_create_data("reg", 0, procdir,
 			&ter21x3_phy_proc_fops, phydev)) {
-		printk("%s/phy not created\n", TER21X3_PROC_DIR);
-		goto cleanup;
+			printk("%s/phy not created\n", TER21X3_PROC_DIR);
+			goto cleanup;
+		}
 	}
 	return 0;
 cleanup:
@@ -278,37 +263,31 @@ static int ter21x3_config_aneg(struct phy_device *phydev)
 
 static int ter21x3_read_status(struct phy_device *phydev)
 {
-	genphy_read_status(phydev);
+	int val;
+
+	val = phy_read(phydev, MII_BMSR);
+	if (val & BMSR_LSTATUS) {
+		phydev->link = 1;
+		val = phy_read(phydev, TER21X3_DIAG_REG);
+		if (val & TER21X3_DIAG_SPEED) {
+			phydev->speed = SPEED_100;
+		} else {
+			phydev->speed = SPEED_10;
+		}
+		if (val & TER21X3_DIAG_DUPLEX) {
+			phydev->duplex = DUPLEX_FULL;
+		} else {
+			phydev->duplex = DUPLEX_HALF;
+		}
+	} else {
+		phydev->link = 0;
+		phydev->duplex = 0;
+		phydev->speed = 0;
+	}
+
 	return 0;
 }
 
-void sonos_get_port_status(unsigned int port, struct ethtool_cmd *cmd)
-{
-	*cmd = ter21x3_status[port];
-}
-
-void sonos_phy_connect(void (*link_up)(struct net_device *), struct net_device *ndev)
-{
-	ter21x3_link_up = link_up;
-	ter21x3_mac_device = ndev;
-	ter21x3_check_link();
-}
-
-void sonos_phy_disconnect(void)
-{
-	ter21x3_status[0].duplex = 0;
-	ter21x3_status[0].speed = 0;
-	if (ter21x3_link_up != NULL)
-		(*ter21x3_link_up)(ter21x3_mac_device);
-	ter21x3_link_up = NULL;
-	ter21x3_mac_device = NULL;
-	if ( procdir )
-		remove_proc_subtree(TER21X3_PROC_DIR, NULL);
-	procdir = NULL;
-	ter21x3_phydev = NULL;
-}
-#endif
-
 static struct phy_driver ter21x3_driver = {
 	.phy_id		= 0x000e7237,
 	.name		= "Teridian 78Q21x3 ethernet",
@@ -320,22 +299,13 @@ static struct phy_driver ter21x3_driver = {
 	.read_status	= &ter21x3_read_status,
 	.ack_interrupt	= &ter21x3_ack_interrupt,
 	.config_intr	= &ter21x3_config_intr,
-	.driver		= {
-		.owner = THIS_MODULE,
-	},
 };
 
 static int __init ter21x3_init(void)
 {
 	int ret;
 
-	ret = phy_driver_register(&ter21x3_driver);
-	if (ret)
-		goto err;
-	return 0;
-
-err:
-	phy_driver_unregister(&ter21x3_driver);
+	ret = phy_driver_register(&ter21x3_driver, THIS_MODULE);
 	return ret;
 }
 
