@@ -16,14 +16,14 @@
 #include <linux/module.h>
 #include <linux/string.h>
 #include <linux/netdevice.h>
-#include <linux/etherdevice.h>
 #include <linux/gpio.h>
 #include <linux/of_gpio.h>
 #include <linux/irq.h>
 #include <linux/kernel.h>
+#include <linux/version.h>
 #include <linux/proc_fs.h>
-#include <net/netlink.h>
-#include <linux/rtnetlink.h>
+
+#include <linux/sonos_kernel.h>
 
 MODULE_DESCRIPTION("TI 83822 PHY driver");
 MODULE_LICENSE("GPL");
@@ -31,10 +31,16 @@ MODULE_LICENSE("GPL");
 static struct proc_dir_entry *procdir = NULL;
 #define TI83822_PROC_DIR	"driver/phy"
 
+#define TI83822_REGCR				0x0D
+#define TI83822_ADDAR				0x0E
 #define TI83822_PHYSTS				0x10
 #define TI83822_PHY_SPECIFIC_CONTROL		0x11
 #define TI83822_INTR_STAT1			0x12
 #define TI83822_INTR_STAT2			0x13
+#define TI83822_PHYRCR				0x1f
+#define TI83822_SOR1				0x467
+#define TI83822_EEECFG3				0x4d1
+
 #define TI83822_INTERRUPT_ENABLE		0x02
 #define TI83822_INTERRUPT_OUTPUT		0x01
 #define TI83822_LINK_INIT			(TI83822_INTERRUPT_ENABLE | TI83822_INTERRUPT_OUTPUT)
@@ -44,48 +50,7 @@ static struct proc_dir_entry *procdir = NULL;
 #define TI83822_PHYSTS_DUPLEX			(1 << 2)
 
 struct ethtool_cmd ti83822_status[2] = {{0}};
-static struct phy_device *ti83822_phydev = NULL;
 static struct net_device *ti83822_mac_device = NULL;
-void (*ti83822_link_up)(struct net_device *ndev) = NULL;
-
-void ti83822_announce_linkup(struct net_device *dev)
-{
-	struct net *rtnl;
-	struct sk_buff *skb;
-	struct nlmsghdr *nlh;
-	unsigned char *b;
-	int size = NLMSG_SPACE(1024);
-
-	if (dev == NULL) {
-		printk("%s: dev NULL\n", __func__);
-		return;
-	}
-	rtnl = dev_net(dev);
-
-	printk("%s: announcing link status change\n", __func__);
-
-	skb = nlmsg_new(size, GFP_ATOMIC);
-	if (!skb) {
-		netlink_set_err(rtnl->rtnl, 0, RTMGRP_Rincon, ENOBUFS);
-		return;
-	}
-	b = skb->tail;
-	nlh = nlmsg_put(skb, 0, 0, RWM_MII, 0, 0);
-	if ( !nlh )
-		goto nlmsg_failure;
-	nla_put(skb, RWA_DEV_NAME, IFNAMSIZ, dev->name);
-	nlh->nlmsg_len = skb->tail - b;
-
-	NETLINK_CB(skb).dst_group = RTMGRP_Rincon;
-	nlmsg_end(skb, nlh);
-	rtnl_notify(skb, rtnl, 0, RTMGRP_Rincon, NULL, GFP_ATOMIC);
-	return;
-
-nlmsg_failure:
-	kfree_skb(skb);
-	netlink_set_err(rtnl->rtnl, 0, RTMGRP_Rincon, EINVAL);
-	printk("%s: phy announce failed.\n", __func__);
-}
 
 int ti83822_phy_power_down(struct phy_device *phydev, int on)
 {
@@ -114,10 +79,57 @@ static int ti83822_resume(struct phy_device *phydev)
 	return 0;
 }
 
-static int ti83822_proc_phydump(struct seq_file *m, void *v)
+static int ti83822_soft_reset(struct phy_device *phydev)
+{
+	return 0;
+}
+
+static int ti83822_read_special(struct phy_device *phydev, int offset)
+{
+	int	err;
+
+	err = phy_write(phydev, TI83822_REGCR, 0x1f);
+	err = phy_write(phydev, TI83822_ADDAR, offset);
+	err = phy_write(phydev, TI83822_REGCR, 0x401f);
+
+	err = phy_read(phydev, TI83822_ADDAR);
+
+	return err;
+}
+
+static int ti83822_write_special(struct phy_device *phydev, int offset, int val)
+{
+	int	err;
+
+	err = phy_write(phydev, TI83822_REGCR, 0x1f);
+	err = phy_write(phydev, TI83822_ADDAR, offset);
+	err = phy_write(phydev, TI83822_REGCR, 0x401f);
+	err = phy_write(phydev, TI83822_ADDAR, val);
+
+	return err;
+}
+
+static int ti83822_disable_eee(struct phy_device *phydev)
+{
+	int	err;
+
+	err = phy_write(phydev, TI83822_REGCR, 0x1f);
+	err = phy_write(phydev, TI83822_ADDAR, TI83822_EEECFG3);
+	err = phy_write(phydev, TI83822_REGCR, 0x401f);
+	err = phy_write(phydev, TI83822_ADDAR, 0x180);
+
+	err = phy_write(phydev, TI83822_REGCR, 0x07);
+	err = phy_write(phydev, TI83822_ADDAR, 0x3c);
+	err = phy_write(phydev, TI83822_REGCR, 0x4007);
+	err = phy_write(phydev, TI83822_ADDAR, 0x0);
+
+	return err;
+}
+
+static int ti83822_proc_phy_special_dump(struct seq_file *m, void *v)
 {
 	struct phy_device *phydev = (struct phy_device *)m->private;
-	int i, reg, val;
+	int i;
 	u32 offset;
 
 	struct ti83822_ranges {
@@ -135,13 +147,13 @@ static int ti83822_proc_phydump(struct seq_file *m, void *v)
                 {0x180, 0x18a},
                 {0x215, 0x215},
                 {0x21d, 0x21d},
-                {0x403, 0x403},
+                {0x403, 0x404},
                 {0x428, 0x428},
                 {0x456, 0x456},
                 {0x460, 0x463},
                 {0x465, 0x465},
-                {0x467, 0x469},
-                {0x4a0, 0x4d1},
+                {TI83822_SOR1, 0x469},
+                {0x4a0, TI83822_EEECFG3},
                 {0x3000, 0x3001},
                 {0x3014, 0x3014},
                 {0x3016, 0x3016},
@@ -151,9 +163,23 @@ static int ti83822_proc_phydump(struct seq_file *m, void *v)
 	for (i = 0 ; i < ARRAY_SIZE(ranges) ; i++) {
 		for (offset = ranges[i].start; offset <= ranges[i].end; offset++ ) {
 			seq_printf(m, "reg %04x = %04x\n", offset,
-				phy_read(phydev, offset));
+				ti83822_read_special(phydev, offset));
 		}
 	}
+	return 0;
+}
+
+static int ti83822_phy_special_proc_open(struct inode *inode, struct file *file)
+{
+	return single_open(file, ti83822_proc_phy_special_dump, PDE_DATA(inode));
+}
+
+
+static int ti83822_proc_phydump(struct seq_file *m, void *v)
+{
+	struct phy_device *phydev = (struct phy_device *)m->private;
+	int reg, val;
+
 	for (reg = 0; reg <= 0x1f; reg++) {
 		val = phy_read(phydev, reg);
 		seq_printf(m, "%04x ", val);
@@ -189,6 +215,15 @@ static ssize_t ti83822_phy_proc_write(struct file *file,
 
 	if ( result == 0 ) {
 		buf[count] = '\0';
+		if (strncmp(buf, "eee", 3) == 0) {
+			val = ti83822_read_special(phydev, TI83822_EEECFG3);
+			printk("toggling EEE capabilities in register 0x4d1, from 0x%04x to 0x%04x\n", \
+				val, ((val & 0xfffe) | (val ^ 0x01)));
+			val &= 0xffff;
+			val ^= 0x01;
+			result = ti83822_write_special(phydev, TI83822_EEECFG3, val);
+			return count;
+		}
 		peq = strchr(buf, '=');
 		if (peq != NULL) {
 			*peq = 0;
@@ -206,6 +241,15 @@ static ssize_t ti83822_phy_proc_write(struct file *file,
 	}
 	return result;
 }
+
+static const struct file_operations ti83822_phy_special_proc_fops = {
+	.owner		= THIS_MODULE,
+	.open		= ti83822_phy_special_proc_open,
+	.read		= seq_read,
+	.llseek		= seq_lseek,
+	.release	= single_release,
+};
+
 static const struct file_operations ti83822_phy_proc_fops = {
 	.owner		= THIS_MODULE,
 	.open		= ti83822_phy_proc_open,
@@ -215,18 +259,12 @@ static const struct file_operations ti83822_phy_proc_fops = {
 	.write		= ti83822_phy_proc_write,
 };
 
-static void ti83822_check_link(void)
+static void ti83822_check_link(struct phy_device *phydev)
 {
 	int link;
-	struct phy_device *phydev;
 	int val;
 	unsigned int prev_speed = ti83822_status[0].speed;
-	if (ti83822_phydev == NULL) {
-		printk("ti83822_phydev NULL\n");
-		return;
-	}
 
-	phydev = ti83822_phydev;
 	link = 0;
 	val = phy_read(phydev, MII_BMSR);
 	if (val & BMSR_LSTATUS) {
@@ -250,19 +288,18 @@ static void ti83822_check_link(void)
 	if (ti83822_status[0].speed != prev_speed) {
 		if (ti83822_status[0].speed == 0) {
 			printk("ti83822: port down\n");
+			netif_carrier_off(ti83822_mac_device);
 		} else {
 			printk("ti83822: port up, speed %d,"
 				" %s duplex\n",
 				ti83822_status[0].speed,
 				(ti83822_status[0].duplex ==
 					DUPLEX_FULL) ? "full" : "half");
+			netif_carrier_on(ti83822_mac_device);
 		}
 	}
 
-
-	if (ti83822_link_up != NULL) {
-		(*ti83822_link_up)(ti83822_mac_device);
-	}
+	sonos_announce_linkup(phydev->attached_dev);
 }
 
 
@@ -278,7 +315,7 @@ static int ti83822_ack_interrupt(struct phy_device *phydev)
 	val = phy_read(phydev, TI83822_PHY_SPECIFIC_CONTROL);
 	err = phy_write(phydev, TI83822_PHY_SPECIFIC_CONTROL, (val | TI83822_INTERRUPT_ENABLE));
 
-	ti83822_check_link();
+	ti83822_check_link(phydev);
 
 	return (err < 0) ? err : 0;
 }
@@ -308,20 +345,30 @@ static int ti83822_config_init(struct phy_device *phydev)
 	struct device_node *node = NULL;
 	int ethernet_phy_interrupt;
 
-	ti83822_phydev = phydev;
-	ti83822_link_up = ti83822_announce_linkup;
 	ti83822_mac_device = phydev->attached_dev;
 
 	node = of_find_compatible_node(NULL, NULL, "Sonos,ti83822_phy");
 	if (node) {
 		ethernet_phy_interrupt = gpio_to_irq(of_get_named_gpio(node, "interrupts", 0));
 	} else {
-		printk(KERN_ERR "could not find device node to get interrupts\n");
+		printk(KERN_ERR "no Sonos,ti83822 device node in the device tree\n");
+		return 0;
 	}
 
 	irq_set_irq_type(ethernet_phy_interrupt, IRQ_TYPE_LEVEL_LOW);
 	phydev->irq = ethernet_phy_interrupt;
 
+	val = ti83822_read_special(phydev, TI83822_EEECFG3);
+	printk(KERN_INFO "%s: intentionally disabling EEE [0x%04x -> 0x180] ***\n", __func__, val);
+	ti83822_disable_eee(phydev);
+	val = phy_read(phydev, TI83822_PHYRCR);
+	printk(KERN_INFO "%s: digital restart - writing 0x%04x to register 0x%x\n", __func__, (val | 0x4000), TI83822_PHYRCR);
+	val = phy_write(phydev, TI83822_PHYRCR, (val | 0x4000));
+
+	if (product_is_apollo()) {
+		ti83822_write_special(phydev, 0x404, 0x160);
+	}
+
 	val = ti83822_config_intr(phydev);
 	if ( val )
 		return val;
@@ -359,6 +406,11 @@ static int ti83822_config_init(struct phy_device *phydev)
 			printk("%s/phy not created\n", TI83822_PROC_DIR);
 			goto cleanup;
 		}
+		if (! proc_create_data("reg_special", 0, procdir,
+			&ti83822_phy_special_proc_fops, phydev)) {
+			printk("%s/phy_special not created\n", TI83822_PROC_DIR);
+			goto cleanup;
+		}
 	}
 
 	return 0;
@@ -407,27 +459,6 @@ void sonos_get_port_status(unsigned int port, struct ethtool_cmd *cmd)
 	*cmd = ti83822_status[port];
 }
 
-void sonos_phy_connect(void (*link_up)(struct net_device *), struct net_device *ndev)
-{
-	ti83822_link_up = link_up;
-	ti83822_mac_device = ndev;
-	ti83822_check_link();
-}
-
-void sonos_phy_disconnect(void)
-{
-	ti83822_status[0].duplex = 0;
-	ti83822_status[0].speed = 0;
-	if (ti83822_link_up != NULL)
-		(*ti83822_link_up)(ti83822_mac_device);
-	ti83822_link_up = NULL;
-	ti83822_mac_device = NULL;
-	if ( procdir )
-		remove_proc_subtree(TI83822_PROC_DIR, NULL);
-	procdir = NULL;
-	ti83822_phydev = NULL;
-}
-
 static struct phy_driver ti83822_driver = {
 	.phy_id		= 0x2000a240,
 	.name		= "TI DP83822 ethernet",
@@ -441,9 +472,12 @@ static struct phy_driver ti83822_driver = {
 	.config_intr	= &ti83822_config_intr,
 	.suspend	= &ti83822_suspend,
 	.resume		= &ti83822_resume,
+	.soft_reset	= &ti83822_soft_reset,
+#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 9, 0)
 	.driver		= {
 		.owner = THIS_MODULE,
 	},
+#endif
 };
 
 
@@ -451,15 +485,11 @@ static int __init ti83822_init(void)
 {
 	int ret;
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 9, 0)
 	ret = phy_driver_register(&ti83822_driver);
-	if (ret)
-		goto err;
-
-	return 0;
-
-err:
-	phy_driver_unregister(&ti83822_driver);
-
+#else
+	ret = phy_driver_register(&ti83822_driver, THIS_MODULE);
+#endif
 	return ret;
 }
 
