@@ -20,8 +20,12 @@
 #include <linux/regulator/consumer.h>
 #include <linux/soc/mediatek/infracfg.h>
 
+#ifdef CONFIG_COMMON_CLK_MT2701
 #include <dt-bindings/power/mt2701-power.h>
+#endif
+#include <dt-bindings/power/mt2712-power.h>
 #include <dt-bindings/power/mt8173-power.h>
+#include <dt-bindings/power/mt8518-power.h>
 
 #define SPM_VDE_PWR_CON			0x0210
 #define SPM_MFG_PWR_CON			0x0214
@@ -30,7 +34,7 @@
 #define SPM_DIS_PWR_CON			0x023c
 #define SPM_CONN_PWR_CON		0x0280
 #define SPM_VEN2_PWR_CON		0x0298
-#define SPM_AUDIO_PWR_CON		0x029c	/* MT8173 */
+#define SPM_AUDIO_PWR_CON		0x029c	/* MT8173, MT2712 */
 #define SPM_BDP_PWR_CON			0x029c	/* MT2701 */
 #define SPM_ETH_PWR_CON			0x02a0
 #define SPM_HIF_PWR_CON			0x02a4
@@ -38,6 +42,9 @@
 #define SPM_MFG_2D_PWR_CON		0x02c0
 #define SPM_MFG_ASYNC_PWR_CON		0x02c4
 #define SPM_USB_PWR_CON			0x02cc
+#define SPM_USB2_PWR_CON		0x02d4	/* MT2712 */
+#define SPM_AUDAFE_PWR_CON		0x0d10	/* MT8518 */
+#define SPM_AUDSRC_PWR_CON		0x0d14	/* MT8518 */
 
 #define SPM_PWR_STATUS			0x060c
 #define SPM_PWR_STATUS_2ND		0x0610
@@ -57,12 +64,13 @@
 #define PWR_STATUS_ETH			BIT(15)
 #define PWR_STATUS_HIF			BIT(16)
 #define PWR_STATUS_IFR_MSC		BIT(17)
+#define PWR_STATUS_USB2			BIT(19)	/* MT2712 */
 #define PWR_STATUS_VENC_LT		BIT(20)
 #define PWR_STATUS_VENC			BIT(21)
-#define PWR_STATUS_MFG_2D		BIT(22)
-#define PWR_STATUS_MFG_ASYNC		BIT(23)
-#define PWR_STATUS_AUDIO		BIT(24)
-#define PWR_STATUS_USB			BIT(25)
+#define PWR_STATUS_MFG_2D		BIT(22)	/* MT8173 */
+#define PWR_STATUS_MFG_ASYNC		BIT(23)	/* MT8173 */
+#define PWR_STATUS_AUDIO		BIT(24)	/* MT8173, MT2712 */
+#define PWR_STATUS_USB			BIT(25)	/* MT8173, MT2712 */
 
 enum clk_id {
 	CLK_NONE,
@@ -71,10 +79,31 @@ enum clk_id {
 	CLK_VENC,
 	CLK_VENC_LT,
 	CLK_ETHIF,
+	CLK_JPGDEC,
+	CLK_AUDIO,
 	CLK_MAX,
 };
 
-#define MAX_CLKS	2
+static const char * const clk_names[] = {
+	NULL,
+	"mm",
+	"mfg",
+	"venc",
+	"venc_lt",
+	"ethif",
+	"jpgdec",
+	"audio",
+	NULL,
+};
+
+#define MAX_CLKS	3
+
+struct bus_prot_ext {
+	u32 set_ofs;
+	u32 clr_ofs;
+	u32 en_ofs;
+	u32 sta_ofs;
+};
 
 struct scp_domain_data {
 	const char *name;
@@ -83,9 +112,9 @@ struct scp_domain_data {
 	u32 sram_pdn_bits;
 	u32 sram_pdn_ack_bits;
 	u32 bus_prot_mask;
+	struct bus_prot_ext bp_ext;
 	enum clk_id clk_id[MAX_CLKS];
 	bool active_wakeup;
-	bool disabled;
 };
 
 struct scp;
@@ -94,13 +123,8 @@ struct scp_domain {
 	struct generic_pm_domain genpd;
 	struct scp *scp;
 	struct clk *clk[MAX_CLKS];
-	u32 sta_mask;
-	void __iomem *ctl_addr;
-	u32 sram_pdn_bits;
-	u32 sram_pdn_ack_bits;
-	u32 bus_prot_mask;
-	bool active_wakeup;
-	bool disabled;
+	const struct scp_domain_data *data;
+	struct regulator *supply;
 };
 
 struct scp {
@@ -115,8 +139,9 @@ static int scpsys_domain_is_on(struct scp_domain *scpd)
 {
 	struct scp *scp = scpd->scp;
 
-	u32 status = readl(scp->base + SPM_PWR_STATUS) & scpd->sta_mask;
-	u32 status2 = readl(scp->base + SPM_PWR_STATUS_2ND) & scpd->sta_mask;
+	u32 status = readl(scp->base + SPM_PWR_STATUS) & scpd->data->sta_mask;
+	u32 status2 = readl(scp->base + SPM_PWR_STATUS_2ND) &
+				scpd->data->sta_mask;
 
 	/*
 	 * A domain is on when both status bits are set. If only one is set
@@ -137,15 +162,16 @@ static int scpsys_power_on(struct generic_pm_domain *genpd)
 	struct scp *scp = scpd->scp;
 	unsigned long timeout;
 	bool expired;
-	void __iomem *ctl_addr = scpd->ctl_addr;
-	u32 sram_pdn_ack = scpd->sram_pdn_ack_bits;
+	void __iomem *ctl_addr = scp->base + scpd->data->ctl_offs;
+	u32 sram_pdn_ack = scpd->data->sram_pdn_ack_bits;
 	u32 val;
 	int ret;
 	int i;
 
-	if (scpd->disabled) {
-		dev_info(scp->dev, "Skipping power on for disabled domain %s\n", genpd->name);
-		return 0;
+	if (scpd->supply) {
+		ret = regulator_enable(scpd->supply);
+		if (ret)
+			return ret;
 	}
 
 	for (i = 0; i < MAX_CLKS && scpd->clk[i]; i++) {
@@ -192,7 +218,7 @@ static int scpsys_power_on(struct generic_pm_domain *genpd)
 	val |= PWR_RST_B_BIT;
 	writel(val, ctl_addr);
 
-	val &= ~scpd->sram_pdn_bits;
+	val &= ~scpd->data->sram_pdn_bits;
 	writel(val, ctl_addr);
 
 	/* wait until SRAM_PDN_ACK all 0 */
@@ -211,13 +237,24 @@ static int scpsys_power_on(struct generic_pm_domain *genpd)
 			expired = true;
 	}
 
-	if (scpd->bus_prot_mask) {
-		ret = mtk_infracfg_clear_bus_protection(scp->infracfg,
-				scpd->bus_prot_mask);
+	if (scpd->data->bus_prot_mask) {
+		if (((scpd->data->bp_ext.set_ofs && scpd->data->bp_ext.clr_ofs) || scpd->data->bp_ext.en_ofs) &&
+				scpd->data->bp_ext.sta_ofs)
+			ret = mtk_infracfg_clear_bus_protection_ext(scp->infracfg,
+					scpd->data->bus_prot_mask, scpd->data->bp_ext.clr_ofs,
+					scpd->data->bp_ext.sta_ofs, scpd->data->bp_ext.en_ofs);
+		else
+			ret = mtk_infracfg_clear_bus_protection(scp->infracfg,
+					scpd->data->bus_prot_mask);
 		if (ret)
 			goto err_pwr_ack;
 	}
 
+#if defined(CONFIG_MACH_MT8518)
+	if (!strcmp(genpd->name, "audafe"))
+		clk_disable_unprepare(scpd->clk[0]);
+#endif
+
 	return 0;
 
 err_pwr_ack:
@@ -226,6 +263,9 @@ err_pwr_ack:
 			clk_disable_unprepare(scpd->clk[i]);
 	}
 err_clk:
+	if (scpd->supply)
+		regulator_disable(scpd->supply);
+
 	dev_err(scp->dev, "Failed to power on domain %s\n", genpd->name);
 
 	return ret;
@@ -237,26 +277,35 @@ static int scpsys_power_off(struct generic_pm_domain *genpd)
 	struct scp *scp = scpd->scp;
 	unsigned long timeout;
 	bool expired;
-	void __iomem *ctl_addr = scpd->ctl_addr;
-	u32 pdn_ack = scpd->sram_pdn_ack_bits;
+	void __iomem *ctl_addr = scp->base + scpd->data->ctl_offs;
+	u32 pdn_ack = scpd->data->sram_pdn_ack_bits;
 	u32 val;
 	int ret;
 	int i;
 
-	if (scpd->disabled) {
-		dev_info(scp->dev, "Skipping power off for disabled domain %s\n", genpd->name);
-		return 0;
+#if defined(CONFIG_MACH_MT8518)
+	if (!strcmp(genpd->name, "audafe")) {
+		ret = clk_prepare_enable(scpd->clk[0]);
+		if (ret)
+			goto out;
 	}
+#endif
 
-	if (scpd->bus_prot_mask) {
-		ret = mtk_infracfg_set_bus_protection(scp->infracfg,
-				scpd->bus_prot_mask);
+	if (scpd->data->bus_prot_mask) {
+		if (((scpd->data->bp_ext.set_ofs && scpd->data->bp_ext.clr_ofs) || scpd->data->bp_ext.en_ofs) &&
+				scpd->data->bp_ext.sta_ofs)
+			ret = mtk_infracfg_set_bus_protection_ext(scp->infracfg,
+					scpd->data->bus_prot_mask, scpd->data->bp_ext.set_ofs,
+					scpd->data->bp_ext.sta_ofs, scpd->data->bp_ext.en_ofs);
+		else
+			ret = mtk_infracfg_set_bus_protection(scp->infracfg,
+					scpd->data->bus_prot_mask);
 		if (ret)
 			goto out;
 	}
 
 	val = readl(ctl_addr);
-	val |= scpd->sram_pdn_bits;
+	val |= scpd->data->sram_pdn_bits;
 	writel(val, ctl_addr);
 
 	/* wait until SRAM_PDN_ACK all 1 */
@@ -311,6 +360,9 @@ static int scpsys_power_off(struct generic_pm_domain *genpd)
 	for (i = 0; i < MAX_CLKS && scpd->clk[i]; i++)
 		clk_disable_unprepare(scpd->clk[i]);
 
+	if (scpd->supply)
+		regulator_disable(scpd->supply);
+
 	return 0;
 
 out:
@@ -327,31 +379,15 @@ static bool scpsys_active_wakeup(struct device *dev)
 	genpd = pd_to_genpd(dev->pm_domain);
 	scpd = container_of(genpd, struct scp_domain, genpd);
 
-	return scpd->active_wakeup;
+	return scpd->data->active_wakeup;
 }
 
-static void init_clks(struct platform_device *pdev, struct clk *clk[CLK_MAX])
+static void init_clks(struct platform_device *pdev, struct clk **clk)
 {
-	enum clk_id clk_ids[] = {
-		CLK_MM,
-		CLK_MFG,
-		CLK_VENC,
-		CLK_VENC_LT,
-		CLK_ETHIF
-	};
-
-	static const char * const clk_names[] = {
-		"mm",
-		"mfg",
-		"venc",
-		"venc_lt",
-		"ethif",
-	};
-
 	int i;
 
-	for (i = 0; i < ARRAY_SIZE(clk_ids); i++)
-		clk[clk_ids[i]] = devm_clk_get(&pdev->dev, clk_names[i]);
+	for (i = CLK_NONE + 1; i < CLK_MAX; i++)
+		clk[i] = devm_clk_get(&pdev->dev, clk_names[i]);
 }
 
 static struct scp *init_scp(struct platform_device *pdev,
@@ -374,14 +410,6 @@ static struct scp *init_scp(struct platform_device *pdev,
 	if (IS_ERR(scp->base))
 		return ERR_CAST(scp->base);
 
-	scp->infracfg = syscon_regmap_lookup_by_phandle(pdev->dev.of_node,
-			"infracfg");
-	if (IS_ERR(scp->infracfg)) {
-		dev_err(&pdev->dev, "Cannot find infracfg controller: %ld\n",
-				PTR_ERR(scp->infracfg));
-		return ERR_CAST(scp->infracfg);
-	}
-
 	scp->domains = devm_kzalloc(&pdev->dev,
 				sizeof(*scp->domains) * num, GFP_KERNEL);
 	if (!scp->domains)
@@ -394,6 +422,27 @@ static struct scp *init_scp(struct platform_device *pdev,
 	if (!pd_data->domains)
 		return ERR_PTR(-ENOMEM);
 
+	scp->infracfg = syscon_regmap_lookup_by_phandle(pdev->dev.of_node,
+			"infracfg");
+	if (IS_ERR(scp->infracfg)) {
+		dev_err(&pdev->dev, "Cannot find infracfg controller: %ld\n",
+				PTR_ERR(scp->infracfg));
+		return ERR_CAST(scp->infracfg);
+	}
+
+	for (i = 0; i < num; i++) {
+		struct scp_domain *scpd = &scp->domains[i];
+		const struct scp_domain_data *data = &scp_domain_data[i];
+
+		scpd->supply = devm_regulator_get_optional(&pdev->dev, data->name);
+		if (IS_ERR(scpd->supply)) {
+			if (PTR_ERR(scpd->supply) == -ENODEV)
+				scpd->supply = NULL;
+			else
+				return ERR_CAST(scpd->supply);
+		}
+	}
+
 	pd_data->num_domains = num;
 
 	init_clks(pdev, clk);
@@ -403,6 +452,11 @@ static struct scp *init_scp(struct platform_device *pdev,
 		struct generic_pm_domain *genpd = &scpd->genpd;
 		const struct scp_domain_data *data = &scp_domain_data[i];
 
+		pd_data->domains[i] = genpd;
+		scpd->scp = scp;
+
+		scpd->data = data;
+
 		for (j = 0; j < MAX_CLKS && data->clk_id[j]; j++) {
 			struct clk *c = clk[data->clk_id[j]];
 
@@ -415,19 +469,6 @@ static struct scp *init_scp(struct platform_device *pdev,
 			scpd->clk[j] = c;
 		}
 
-		pd_data->domains[i] = genpd;
-		scpd->scp = scp;
-
-		scpd->sta_mask = data->sta_mask;
-		scpd->ctl_addr = scp->base + data->ctl_offs;
-		scpd->sram_pdn_bits = data->sram_pdn_bits;
-		scpd->sram_pdn_ack_bits = data->sram_pdn_ack_bits;
-		scpd->bus_prot_mask = data->bus_prot_mask;
-		scpd->active_wakeup = data->active_wakeup;
-		scpd->disabled = data->disabled;
-		for (j = 0; j < MAX_CLKS && data->clk_id[j]; j++)
-			scpd->clk[j] = clk[data->clk_id[j]];
-
 		genpd->name = data->name;
 		genpd->power_off = scpsys_power_off;
 		genpd->power_on = scpsys_power_on;
@@ -474,20 +515,15 @@ static void mtk_register_power_domains(struct platform_device *pdev,
 /*
  * MT2701 power domain support
  */
-
+#ifdef CONFIG_COMMON_CLK_MT2701
 static const struct scp_domain_data scp_domain_data_mt2701[] = {
-
 	[MT2701_POWER_DOMAIN_CONN] = {
 		.name = "conn",
 		.sta_mask = PWR_STATUS_CONN,
 		.ctl_offs = SPM_CONN_PWR_CON,
 		.bus_prot_mask = 0x0104,
+		.clk_id = {CLK_NONE},
 		.active_wakeup = true,
-#ifdef CONFIG_SONOS
-		/* Sonos does not use the MT6625 PMIC so the
-		 * "conn" interconnect is not needed. */
-		.disabled = true,
-#endif /* CONFIG_SONOS */
 	},
 	[MT2701_POWER_DOMAIN_DISP] = {
 		.name = "disp",
@@ -530,6 +566,7 @@ static const struct scp_domain_data scp_domain_data_mt2701[] = {
 		.sta_mask = PWR_STATUS_BDP,
 		.ctl_offs = SPM_BDP_PWR_CON,
 		.sram_pdn_bits = GENMASK(11, 8),
+		.clk_id = {CLK_NONE},
 		.active_wakeup = true,
 	},
 	[MT2701_POWER_DOMAIN_ETH] = {
@@ -548,11 +585,13 @@ static const struct scp_domain_data scp_domain_data_mt2701[] = {
 		.sram_pdn_bits = GENMASK(11, 8),
 		.sram_pdn_ack_bits = GENMASK(15, 12),
 		.clk_id = {CLK_ETHIF},
+		.active_wakeup = true,
 	},
 	[MT2701_POWER_DOMAIN_IFR_MSC] = {
 		.name = "ifr_msc",
 		.sta_mask = PWR_STATUS_IFR_MSC,
 		.ctl_offs = SPM_IFR_MSC_PWR_CON,
+		.clk_id = {CLK_NONE},
 		.active_wakeup = true,
 	},
 };
@@ -572,6 +611,107 @@ static int __init scpsys_probe_mt2701(struct platform_device *pdev)
 	return 0;
 }
 
+/*
+ * MT2712 power domain support
+ */
+#endif
+
+static const struct scp_domain_data scp_domain_data_mt2712[] = {
+	[MT2712_POWER_DOMAIN_MM] = {
+		.name = "mm",
+		.sta_mask = PWR_STATUS_DISP,
+		.ctl_offs = SPM_DIS_PWR_CON,
+		.sram_pdn_bits = GENMASK(8, 8),
+		.sram_pdn_ack_bits = GENMASK(12, 12),
+		.clk_id = {CLK_MM},
+		.active_wakeup = true,
+	},
+	[MT2712_POWER_DOMAIN_VDEC] = {
+		.name = "vdec",
+		.sta_mask = PWR_STATUS_VDEC,
+		.ctl_offs = SPM_VDE_PWR_CON,
+		.sram_pdn_bits = GENMASK(8, 8),
+		.sram_pdn_ack_bits = GENMASK(12, 12),
+		.clk_id = {CLK_MM},
+		.active_wakeup = true,
+	},
+	[MT2712_POWER_DOMAIN_VENC] = {
+		.name = "venc",
+		.sta_mask = PWR_STATUS_VENC,
+		.ctl_offs = SPM_VEN_PWR_CON,
+		.sram_pdn_bits = GENMASK(11, 8),
+		.sram_pdn_ack_bits = GENMASK(15, 12),
+		.clk_id = {CLK_MM, CLK_VENC, CLK_JPGDEC},
+		.active_wakeup = true,
+	},
+	[MT2712_POWER_DOMAIN_ISP] = {
+		.name = "isp",
+		.sta_mask = PWR_STATUS_ISP,
+		.ctl_offs = SPM_ISP_PWR_CON,
+		.sram_pdn_bits = GENMASK(11, 8),
+		.sram_pdn_ack_bits = GENMASK(13, 12),
+		.clk_id = {CLK_MM},
+		.active_wakeup = true,
+	},
+	[MT2712_POWER_DOMAIN_AUDIO] = {
+		.name = "audio",
+		.sta_mask = PWR_STATUS_AUDIO,
+		.ctl_offs = SPM_AUDIO_PWR_CON,
+		.sram_pdn_bits = GENMASK(11, 8),
+		.sram_pdn_ack_bits = GENMASK(15, 12),
+		.clk_id = {CLK_AUDIO},
+		.active_wakeup = true,
+	},
+	[MT2712_POWER_DOMAIN_USB] = {
+		.name = "usb",
+		.sta_mask = PWR_STATUS_USB,
+		.ctl_offs = SPM_USB_PWR_CON,
+		.sram_pdn_bits = GENMASK(10, 8),
+		.sram_pdn_ack_bits = GENMASK(14, 12),
+		.clk_id = {CLK_NONE},
+		.active_wakeup = true,
+	},
+	[MT2712_POWER_DOMAIN_USB2] = {
+		.name = "usb2",
+		.sta_mask = PWR_STATUS_USB2,
+		.ctl_offs = SPM_USB2_PWR_CON,
+		.sram_pdn_bits = GENMASK(10, 8),
+		.sram_pdn_ack_bits = GENMASK(14, 12),
+		.clk_id = {CLK_NONE},
+		.active_wakeup = true,
+	},
+	[MT2712_POWER_DOMAIN_MFG] = {
+		.name = "mfg",
+		.sta_mask = PWR_STATUS_MFG,
+		.ctl_offs = SPM_MFG_PWR_CON,
+		.sram_pdn_bits = GENMASK(11, 8),
+		.sram_pdn_ack_bits = GENMASK(19, 16),
+		.clk_id = {CLK_MFG},
+		.bus_prot_mask = BIT(14) | BIT(21) | BIT(23),
+		.bp_ext = {
+			.set_ofs = INFRA_TOPAXI_PROTECTEN_SET,
+			.clr_ofs = INFRA_TOPAXI_PROTECTEN_CLR,
+			.sta_ofs = INFRA_TOPAXI_PROTECTSTA1,
+		},
+		.active_wakeup = true,
+	},
+};
+
+#define NUM_DOMAINS_MT2712	ARRAY_SIZE(scp_domain_data_mt2712)
+
+static int __init scpsys_probe_mt2712(struct platform_device *pdev)
+{
+	struct scp *scp;
+
+	scp = init_scp(pdev, scp_domain_data_mt2712, NUM_DOMAINS_MT2712);
+	if (IS_ERR(scp))
+		return PTR_ERR(scp);
+
+	mtk_register_power_domains(pdev, scp, NUM_DOMAINS_MT2712);
+
+	return 0;
+}
+
 /*
  * MT8173 power domain support
  */
@@ -695,17 +835,78 @@ static int __init scpsys_probe_mt8173(struct platform_device *pdev)
 	return 0;
 }
 
+/*
+ * MT8518 power domain support
+ */
+
+static const struct scp_domain_data scp_domain_data_mt8518[] = {
+	[MT8518_POWER_DOMAIN_DISP] = {
+		.name = "disp",
+		.sta_mask = PWR_STATUS_DISP,
+		.ctl_offs = SPM_DIS_PWR_CON,
+		.sram_pdn_bits = GENMASK(11, 8),
+		.sram_pdn_ack_bits = GENMASK(12, 12),
+		.bus_prot_mask = BIT(1),
+		.clk_id = {CLK_MM},
+		.active_wakeup = true,
+	},
+	[MT8518_POWER_DOMAIN_AUDAFE] = {
+		.name = "audafe",
+		.sta_mask = BIT(21),
+		.ctl_offs = SPM_AUDAFE_PWR_CON,
+		.sram_pdn_bits = GENMASK(11, 8),
+		.sram_pdn_ack_bits = GENMASK(15, 12),
+		.bus_prot_mask = BIT(25) | BIT(26) | BIT(28),
+		.clk_id = {CLK_AUDIO},
+		.active_wakeup = true,
+	},
+	[MT8518_POWER_DOMAIN_AUDSRC] = {
+		.name = "audsrc",
+		.sta_mask = BIT(20),
+		.ctl_offs = SPM_AUDSRC_PWR_CON,
+		.sram_pdn_bits = GENMASK(8, 8),
+		.sram_pdn_ack_bits = GENMASK(9, 9),
+		.bus_prot_mask = BIT(27),
+		.clk_id = {CLK_NONE},
+		.active_wakeup = true,
+	},
+};
+
+#define NUM_DOMAINS_MT8518	ARRAY_SIZE(scp_domain_data_mt8518)
+
+static int __init scpsys_probe_mt8518(struct platform_device *pdev)
+{
+	struct scp *scp;
+
+	scp = init_scp(pdev, scp_domain_data_mt8518, NUM_DOMAINS_MT8518);
+	if (IS_ERR(scp))
+		return PTR_ERR(scp);
+
+	mtk_register_power_domains(pdev, scp, NUM_DOMAINS_MT8518);
+
+	return 0;
+}
+
 /*
  * scpsys driver init
  */
 
 static const struct of_device_id of_scpsys_match_tbl[] = {
+#ifdef CONFIG_COMMON_CLK_MT2701
 	{
 		.compatible = "mediatek,mt2701-scpsys",
 		.data = scpsys_probe_mt2701,
+	},
+#endif
+	{
+		.compatible = "mediatek,mt2712-scpsys",
+		.data = scpsys_probe_mt2712,
 	}, {
 		.compatible = "mediatek,mt8173-scpsys",
 		.data = scpsys_probe_mt8173,
+	}, {
+		.compatible = "mediatek,mt8518-scpsys",
+		.data = scpsys_probe_mt8518,
 	}, {
 		/* sentinel */
 	}
@@ -729,25 +930,9 @@ static struct platform_driver scpsys_drv = {
 	.probe = scpsys_probe,
 	.driver = {
 		.name = "mtk-scpsys",
+		.suppress_bind_attrs = true,
 		.owner = THIS_MODULE,
 		.of_match_table = of_match_ptr(of_scpsys_match_tbl),
 	},
 };
-
-static int __init scpsys_drv_init(void)
-{
-	return platform_driver_register(&scpsys_drv);
-}
-
-/*
- * There are some Mediatek drivers which depend on the power domain driver need
- * to probe in earlier initcall levels. So scpsys driver also need to probe
- * earlier.
- *
- * IOMMU(M4U) and SMI drivers for example. SMI is a bridge between IOMMU and
- * multimedia HW. IOMMU depends on SMI, and SMI is a power domain consumer,
- * so the proper probe sequence should be scpsys -> SMI -> IOMMU driver.
- * IOMMU drivers are initialized during subsys_init by default, so we need to
- * move SMI and scpsys drivers to subsys_init or earlier init levels.
- */
-subsys_initcall(scpsys_drv_init);
+builtin_platform_driver(scpsys_drv);
