@@ -82,6 +82,11 @@
 #define UWK_CTL1_0P_LS_P	BIT(7)
 #define UWK_CTL1_IS_P		BIT(6)  /* polarity for ip sleep */
 
+#define PERI_SSUSB_SPM_CTRL		0x514
+/*#define PERI_SSUSB_SPM_CTRL		0x510 */
+#define RG_SSUSB_SPM_INT_EN		BIT(1)
+#define RG_SSUSB_IP_SLEEP_EN	BIT(4)
+
 enum ssusb_wakeup_src {
 	SSUSB_WK_IP_SLEEP = 1,
 	SSUSB_WK_LINE_STATE = 2,
@@ -212,18 +217,18 @@ static int xhci_mtk_clks_enable(struct xhci_hcd_mtk *mtk)
 {
 	int ret;
 
+	ret = clk_prepare_enable(mtk->ref_clk);
+	if (ret) {
+		dev_err(mtk->dev, "failed to enable ref_clk\n");
+		goto ref_clk_err;
+	}
+
 	ret = clk_prepare_enable(mtk->sys_clk);
 	if (ret) {
 		dev_err(mtk->dev, "failed to enable sys_clk\n");
 		goto sys_clk_err;
 	}
 
-	ret = clk_prepare_enable(mtk->free_clk);
-	if (ret) {
-		dev_err(mtk->dev, "failed to enable free_clk\n");
-		goto free_clk_err;
-	}
-
 	if (mtk->wakeup_src) {
 		ret = clk_prepare_enable(mtk->wk_deb_p0);
 		if (ret) {
@@ -242,10 +247,10 @@ static int xhci_mtk_clks_enable(struct xhci_hcd_mtk *mtk)
 usb_p1_err:
 	clk_disable_unprepare(mtk->wk_deb_p0);
 usb_p0_err:
-	clk_disable_unprepare(mtk->free_clk);
-free_clk_err:
 	clk_disable_unprepare(mtk->sys_clk);
 sys_clk_err:
+	clk_disable_unprepare(mtk->ref_clk);
+ref_clk_err:
 	return -EINVAL;
 }
 
@@ -255,8 +260,8 @@ static void xhci_mtk_clks_disable(struct xhci_hcd_mtk *mtk)
 		clk_disable_unprepare(mtk->wk_deb_p1);
 		clk_disable_unprepare(mtk->wk_deb_p0);
 	}
-	clk_disable_unprepare(mtk->free_clk);
 	clk_disable_unprepare(mtk->sys_clk);
+	clk_disable_unprepare(mtk->ref_clk);
 }
 
 /* only clocks can be turn off for ip-sleep wakeup mode */
@@ -265,25 +270,39 @@ static void usb_wakeup_ip_sleep_en(struct xhci_hcd_mtk *mtk)
 	u32 tmp;
 	struct regmap *pericfg = mtk->pericfg;
 
-	regmap_read(pericfg, PERI_WK_CTRL1, &tmp);
-	tmp &= ~UWK_CTL1_IS_P;
-	tmp &= ~(UWK_CTL1_IS_C(0xf));
-	tmp |= UWK_CTL1_IS_C(0x8);
-	regmap_write(pericfg, PERI_WK_CTRL1, tmp);
-	regmap_write(pericfg, PERI_WK_CTRL1, tmp | UWK_CTL1_IS_E);
+	if (mtk->uses_new_wakeup) {
+		regmap_read(pericfg, PERI_SSUSB_SPM_CTRL, &tmp);
+		tmp |= RG_SSUSB_SPM_INT_EN | RG_SSUSB_IP_SLEEP_EN;
+		regmap_write(pericfg, PERI_SSUSB_SPM_CTRL, tmp);
+		regmap_read(pericfg, PERI_SSUSB_SPM_CTRL, &tmp);
+		dev_dbg(mtk->dev, "%s(): WK_CTRL=%#x\n", __func__, tmp);
+	} else {
+		regmap_read(pericfg, PERI_WK_CTRL1, &tmp);
+		tmp &= ~UWK_CTL1_IS_P;
+		tmp &= ~(UWK_CTL1_IS_C(0xf));
+		tmp |= UWK_CTL1_IS_C(0x8);
+		regmap_write(pericfg, PERI_WK_CTRL1, tmp);
+		regmap_write(pericfg, PERI_WK_CTRL1, tmp | UWK_CTL1_IS_E);
 
-	regmap_read(pericfg, PERI_WK_CTRL1, &tmp);
-	dev_dbg(mtk->dev, "%s(): WK_CTRL1[P6,E25,C26:29]=%#x\n",
-		__func__, tmp);
+		regmap_read(pericfg, PERI_WK_CTRL1, &tmp);
+		dev_dbg(mtk->dev, "%s(): WK_CTRL1[P6,E25,C26:29]=%#x\n",
+			__func__, tmp);
+	}
 }
 
 static void usb_wakeup_ip_sleep_dis(struct xhci_hcd_mtk *mtk)
 {
 	u32 tmp;
 
-	regmap_read(mtk->pericfg, PERI_WK_CTRL1, &tmp);
-	tmp &= ~UWK_CTL1_IS_E;
-	regmap_write(mtk->pericfg, PERI_WK_CTRL1, tmp);
+	if (mtk->uses_new_wakeup) {
+		regmap_read(mtk->pericfg, PERI_SSUSB_SPM_CTRL, &tmp);
+		tmp &= ~(RG_SSUSB_SPM_INT_EN | RG_SSUSB_IP_SLEEP_EN);
+		regmap_write(mtk->pericfg, PERI_SSUSB_SPM_CTRL, tmp);
+	} else {
+		regmap_read(mtk->pericfg, PERI_WK_CTRL1, &tmp);
+		tmp &= ~UWK_CTL1_IS_E;
+		regmap_write(mtk->pericfg, PERI_WK_CTRL1, tmp);
+	}
 }
 
 /*
@@ -295,6 +314,9 @@ static void usb_wakeup_line_state_en(struct xhci_hcd_mtk *mtk)
 	u32 tmp;
 	struct regmap *pericfg = mtk->pericfg;
 
+	if (mtk->uses_new_wakeup)
+		return;
+
 	/* line-state of u2-port0 */
 	regmap_read(pericfg, PERI_WK_CTRL1, &tmp);
 	tmp &= ~UWK_CTL1_0P_LS_P;
@@ -317,6 +339,9 @@ static void usb_wakeup_line_state_dis(struct xhci_hcd_mtk *mtk)
 	u32 tmp;
 	struct regmap *pericfg = mtk->pericfg;
 
+	if (mtk->uses_new_wakeup)
+		return;
+
 	/* line-state of u2-port0 */
 	regmap_read(pericfg, PERI_WK_CTRL1, &tmp);
 	tmp &= ~UWK_CTL1_0P_LS_E;
@@ -381,7 +406,8 @@ static int usb_wakeup_of_property_parse(struct xhci_hcd_mtk *mtk,
 }
 
 static int xhci_mtk_setup(struct usb_hcd *hcd);
-static const struct xhci_driver_overrides xhci_mtk_overrides __initconst = {
+
+static const struct xhci_driver_overrides xhci_mtk_overrides = {
 	.extra_priv_size = sizeof(struct xhci_hcd),
 	.reset = xhci_mtk_setup,
 };
@@ -559,13 +585,21 @@ static int xhci_mtk_probe(struct platform_device *pdev)
 		return PTR_ERR(mtk->sys_clk);
 	}
 
-	mtk->free_clk = devm_clk_get(dev, "free_ck");
-	if (IS_ERR(mtk->free_clk)) {
-		dev_err(dev, "fail to get free_clk\n");
-		return PTR_ERR(mtk->free_clk);
+	/*
+	 * reference clock is usually a "fixed-clock", make it optional
+	 * for backward compatibility and ignore the error if it does
+	 * not exist.
+	 */
+	mtk->ref_clk = devm_clk_get(dev, "ref_ck");
+	if (IS_ERR(mtk->ref_clk)) {
+		if (PTR_ERR(mtk->ref_clk) == -EPROBE_DEFER)
+			return -EPROBE_DEFER;
+
+		mtk->ref_clk = NULL;
 	}
 
 	mtk->lpm_support = of_property_read_bool(node, "usb3-lpm-capable");
+	mtk->uses_new_wakeup = of_property_read_bool(node, "mediatek,new-wakeup");
 
 	ret = usb_wakeup_of_property_parse(mtk, node);
 	if (ret)
@@ -671,13 +705,13 @@ static int xhci_mtk_probe(struct platform_device *pdev)
 		goto power_off_phys;
 	}
 
-	if (HCC_MAX_PSA(xhci->hcc_params) >= 4)
-		xhci->shared_hcd->can_do_streams = 1;
-
 	ret = usb_add_hcd(hcd, irq, IRQF_SHARED);
 	if (ret)
 		goto put_usb3_hcd;
 
+	if (HCC_MAX_PSA(xhci->hcc_params) >= 4)
+		xhci->shared_hcd->can_do_streams = 1;
+
 	ret = usb_add_hcd(xhci->shared_hcd, irq, IRQF_SHARED);
 	if (ret)
 		goto dealloc_usb2_hcd;
@@ -736,7 +770,6 @@ static int xhci_mtk_remove(struct platform_device *dev)
 	return 0;
 }
 
-#ifdef CONFIG_PM_SLEEP
 /*
  * if ip sleep fails, and all clocks are disabled, access register will hang
  * AHB bus, so stop polling roothubs to avoid regs access on bus suspend.
@@ -744,7 +777,7 @@ static int xhci_mtk_remove(struct platform_device *dev)
  * to wake up system immediately after system suspend complete if ip sleep
  * fails, it is what we wanted.
  */
-static int xhci_mtk_suspend(struct device *dev)
+static int __maybe_unused xhci_mtk_suspend(struct device *dev)
 {
 	struct xhci_hcd_mtk *mtk = dev_get_drvdata(dev);
 	struct usb_hcd *hcd = mtk->hcd;
@@ -763,7 +796,7 @@ static int xhci_mtk_suspend(struct device *dev)
 	return 0;
 }
 
-static int xhci_mtk_resume(struct device *dev)
+static int __maybe_unused xhci_mtk_resume(struct device *dev)
 {
 	struct xhci_hcd_mtk *mtk = dev_get_drvdata(dev);
 	struct usb_hcd *hcd = mtk->hcd;
@@ -785,18 +818,27 @@ static int xhci_mtk_resume(struct device *dev)
 static const struct dev_pm_ops xhci_mtk_pm_ops = {
 	SET_SYSTEM_SLEEP_PM_OPS(xhci_mtk_suspend, xhci_mtk_resume)
 };
-#define DEV_PM_OPS	(&xhci_mtk_pm_ops)
-#else
-#define DEV_PM_OPS	NULL
-#endif /* CONFIG_PM */
+#define DEV_PM_OPS IS_ENABLED(CONFIG_PM) ? &xhci_mtk_pm_ops : NULL
+
+static const struct dev_pm_ops xhci_mtk_phone_pm_ops = {
+	SET_NOIRQ_SYSTEM_SLEEP_PM_OPS(xhci_mtk_suspend, xhci_mtk_resume)
+};
+#define DEV_PHONE_PM_OPS (IS_ENABLED(CONFIG_PM) ? &xhci_mtk_phone_pm_ops : NULL)
+
 
 #ifdef CONFIG_OF
 static const struct of_device_id mtk_xhci_of_match[] = {
-	{ .compatible = "mediatek,mt2701-xhci"},
 	{ .compatible = "mediatek,mt8173-xhci"},
 	{ },
 };
 MODULE_DEVICE_TABLE(of, mtk_xhci_of_match);
+
+static const struct of_device_id mtk_xhci_phone_of_match[] = {
+	{ .compatible = "mediatek,mt67xx-xhci"},
+	{ },
+};
+MODULE_DEVICE_TABLE(of, mtk_xhci_phone_of_match);
+
 #endif
 
 static struct platform_driver mtk_xhci_driver = {
@@ -808,8 +850,29 @@ static struct platform_driver mtk_xhci_driver = {
 		.of_match_table = of_match_ptr(mtk_xhci_of_match),
 	},
 };
+
+static struct platform_driver mtk_xhci_phone_driver = {
+	.probe	= xhci_mtk_probe,
+	.remove	= xhci_mtk_remove,
+	.driver	= {
+		.name = "xhci-mtk-phone",
+		.pm = DEV_PHONE_PM_OPS,
+		.of_match_table = of_match_ptr(mtk_xhci_phone_of_match),
+	},
+};
+
 MODULE_ALIAS("platform:xhci-mtk");
 
+int xhci_mtk_register_plat(void)
+{
+	return platform_driver_register(&mtk_xhci_phone_driver);
+}
+
+void xhci_mtk_unregister_plat(void)
+{
+	platform_driver_unregister(&mtk_xhci_phone_driver);
+}
+
 static int __init xhci_mtk_init(void)
 {
 	xhci_init_driver(&xhci_mtk_hc_driver, &xhci_mtk_overrides);
