@@ -30,6 +30,7 @@
 #include <linux/crypto.h>
 #ifdef CONFIG_SONOS_SECBOOT
 #include <linux/scatterlist.h>
+#include <linux/sonos_sec_fs_keys.h>
 #endif
 #include "ubifs.h"
 
@@ -149,7 +150,7 @@ static int aes_crypt(const struct ubifs_info *c, const void *str, int len, const
  * Note, if the input buffer was not compressed, it is copied to the output
  * buffer and %UBIFS_COMPR_NONE is returned in @compr_type.
  */
-#ifdef CONFIG_SONOS_SECBOOT
+#if defined CONFIG_SONOS_SECBOOT
 void ubifs_compress(const struct ubifs_info *c, const void *in_buf,
 		    int in_len, void *out_buf, int *out_len, int *compr_type, const void *crypto_key)
 #else
@@ -186,7 +187,7 @@ void ubifs_compress(const struct ubifs_info *c, const void *in_buf,
 	if (in_len - *out_len < UBIFS_MIN_COMPRESS_DIFF)
 		goto no_compr;
 
-#ifdef CONFIG_SONOS_SECBOOT
+#if defined CONFIG_SONOS_SECBOOT
 	if (crypto_key) {
 		aes_crypt(c, out_buf, *out_len, crypto_key);
 	}
@@ -197,7 +198,7 @@ no_compr:
 	memcpy(out_buf, in_buf, in_len);
 	*out_len = in_len;
 	*compr_type = UBIFS_COMPR_NONE;
-#ifdef CONFIG_SONOS_SECBOOT
+#if defined CONFIG_SONOS_SECBOOT
 	if (crypto_key) {
 		aes_crypt(c, out_buf, *out_len, crypto_key);
 	}
@@ -217,7 +218,7 @@ no_compr:
  * The length of the uncompressed data is returned in @out_len. This functions
  * returns %0 on success or a negative error code on failure.
  */
-#ifdef CONFIG_SONOS_SECBOOT
+#if defined CONFIG_SONOS_SECBOOT
 int ubifs_decompress(const struct ubifs_info *c, const void *in_buf,
 		     int in_len, void *out_buf, int *out_len, int compr_type, const void *crypto_key)
 #else
@@ -239,7 +240,7 @@ int ubifs_decompress(const struct ubifs_info *c, const void *in_buf,
 		ubifs_err(c, "%s compression is not compiled in", compr->name);
 		return -EINVAL;
 	}
-#ifdef CONFIG_SONOS_SECBOOT
+#if defined CONFIG_SONOS_SECBOOT
 	if (crypto_key) {
 		aes_crypt(c, in_buf, in_len, crypto_key);
 	}
@@ -331,7 +332,7 @@ void ubifs_compressors_exit(void)
 	compr_exit(&zlib_compr);
 }
 
-#ifdef CONFIG_SONOS_SECBOOT
+#if defined CONFIG_SONOS_SECBOOT && defined CONFIG_UBIFS_FS
 /**
  * ubifs_set_ubifs_key - initialize the jffs cryto key
  */
