@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, Sonos, Inc.
+ * Copyright (c) 2015-2018, Sonos, Inc.
  *
  * SPDX-License-Identifier:	GPL-2.0
  *
@@ -9,17 +9,12 @@
 #ifndef SONOS_SIGNATURE_KEYS_H
 #define SONOS_SIGNATURE_KEYS_H
 
-#include <crypto/sha.h>
-#include "sonos_signature.h"
+#include <linux/types.h>
 
 typedef struct {
-	struct key *key;
 	const uint8_t *der;
 	size_t derLen;
-        const char *name;
-        SonosKeyIdentifierScheme_t keyIdScheme;
-        uint8_t keyId[SHA1_DIGEST_SIZE];
-        size_t keyIdLen;
+	const char *name;
 } SonosRsaKeyEntry;
 
 extern const SonosRsaKeyEntry * g_SonosSigningKeys[];
