@@ -67,7 +67,7 @@
 
 #if defined(CONFIG_SONOS) || defined (__SONOS_LINUX__)
 /* Sonos socket options */
-#define SO_SONOS_OPTIONS			51
+#define SO_SONOS_OPTIONS			54
 #define SO_SONOS_OPTION_SOCKET_TAG		1
 #define SO_SONOS_OPTION_MULTICAST		2
 #define SO_SONOS_OPTION_LOW_LATENCY_AUDIO	4
@@ -110,4 +110,9 @@ struct sonos_socket_data {
 #define SO_ATTACH_BPF		50
 #define SO_DETACH_BPF		SO_DETACH_FILTER
 
+#define SO_ATTACH_REUSEPORT_CBPF	51
+#define SO_ATTACH_REUSEPORT_EBPF	52
+
+#define SO_CNX_ADVICE		53
+
 #endif /* __ASM_GENERIC_SOCKET_H */
