@@ -70,7 +70,6 @@
 #include <linux/shmem_fs.h>
 #include <linux/slab.h>
 #include <linux/perf_event.h>
-#include <linux/file.h>
 #include <linux/ptrace.h>
 #include <linux/blkdev.h>
 #include <linux/elevator.h>
@@ -81,6 +80,11 @@
 #include <linux/integrity.h>
 #include <linux/proc_ns.h>
 #include <linux/io.h>
+#include <linux/kaiser.h>
+#include <linux/cache.h>
+#ifdef CONFIG_AMLOGIC_PAGE_TRACE
+#include <linux/amlogic/page_trace.h>
+#endif
 
 #include <asm/io.h>
 #include <asm/bugs.h>
@@ -88,18 +92,25 @@
 #include <asm/sections.h>
 #include <asm/cacheflush.h>
 
+#ifdef CONFIG_SONOS_SECBOOT
+#include <sonos/firmware_allowlist.h>
+#include <crypto/sonos_signature_common_linux.h>
+#include <crypto/sonos_signature_keys.h>
+#include <crypto/sonos_signature_verify_linux.h>
+#include "mdp.h"
+#include "sonos_lock.h"
+#include "sonos_unlock.h"
+#include <linux/sonos_sec_general.h>
+#include <linux/sonos_mdp_global.h>
+
+extern void check_sysrq_authorization(void);
+#endif
+
 static int kernel_init(void *);
 
 extern void init_IRQ(void);
 extern void fork_init(void);
 extern void radix_tree_init(void);
-#ifndef CONFIG_DEBUG_RODATA
-static inline void mark_rodata_ro(void) { }
-#endif
-
-#ifdef CONFIG_SONOS_SECBOOT
-extern void check_sysrq_authorization(void);
-#endif
 
 /*
  * Debug helper: via this flag we know that we are in 'early bootup code'
@@ -168,10 +179,10 @@ static const char *panic_later, *panic_param;
 
 extern const struct obs_kernel_param __setup_start[], __setup_end[];
 
-static int __init obsolete_checksetup(char *line)
+static bool __init obsolete_checksetup(char *line)
 {
 	const struct obs_kernel_param *p;
-	int had_early_param = 0;
+	bool had_early_param = false;
 
 	p = __setup_start;
 	do {
@@ -183,13 +194,13 @@ static int __init obsolete_checksetup(char *line)
 				 * Keep iterating, as we can have early
 				 * params and __setups of same names 8( */
 				if (line[n] == '\0' || line[n] == '=')
-					had_early_param = 1;
+					had_early_param = true;
 			} else if (!p->setup_func) {
 				pr_warn("Parameter %s is obsolete, ignored\n",
 					p->str);
-				return 1;
+				return true;
 			} else if (p->setup_func(line + n))
-				return 1;
+				return true;
 		}
 		p++;
 	} while (p < __setup_end);
@@ -387,12 +398,11 @@ static void __init setup_command_line(char *command_line)
 
 static __initdata DECLARE_COMPLETION(kthreadd_done);
 
-static noinline void __init_refok rest_init(void)
+static noinline void __ref rest_init(void)
 {
 	int pid;
 
 	rcu_scheduler_starting();
-	smpboot_thread_init();
 	/*
 	 * We need to spawn init first so that it obtains pid 1, however
 	 * the init task will end up wanting to create kthreads, which, if
@@ -456,26 +466,12 @@ void __init parse_early_param(void)
 	done = 1;
 }
 
-/*
- *	Activate the first processor.
- */
-
-static void __init boot_cpu_init(void)
-{
-	int cpu = smp_processor_id();
-	/* Mark the boot cpu "present", "online" etc for SMP and UP case */
-	set_cpu_online(cpu, true);
-	set_cpu_active(cpu, true);
-	set_cpu_present(cpu, true);
-	set_cpu_possible(cpu, true);
-}
-
 void __init __weak smp_setup_processor_id(void)
 {
 }
 
 # if THREAD_SIZE >= PAGE_SIZE
-void __init __weak thread_info_cache_init(void)
+void __init __weak thread_stack_cache_init(void)
 {
 }
 #endif
@@ -491,11 +487,16 @@ static void __init mm_init(void)
 	 */
 	page_ext_init_flatmem();
 	mem_init();
+#ifdef CONFIG_AMLOGIC_PAGE_TRACE
+	/* allocate memory before first page allocated */
+	page_trace_mem_init();
+#endif
 	kmem_cache_init();
 	percpu_init_late();
 	pgtable_init();
 	vmalloc_init();
 	ioremap_huge_init();
+	kaiser_init();
 }
 
 asmlinkage __visible void __init start_kernel(void)
@@ -503,11 +504,6 @@ asmlinkage __visible void __init start_kernel(void)
 	char *command_line;
 	char *after_dashes;
 
-	/*
-	 * Need to run as early as possible, to initialize the
-	 * lockdep hash:
-	 */
-	lockdep_init();
 	set_task_stack_end_magic(&init_task);
 	smp_setup_processor_id();
 	debug_objects_early_init();
@@ -534,6 +530,7 @@ asmlinkage __visible void __init start_kernel(void)
 	setup_command_line(command_line);
 	setup_nr_cpu_ids();
 	setup_per_cpu_areas();
+	boot_cpu_state_init();
 	smp_prepare_boot_cpu();	/* arch-specific boot-cpu hooks */
 
 	build_all_zonelists(NULL, NULL);
@@ -595,6 +592,7 @@ asmlinkage __visible void __init start_kernel(void)
 	timekeeping_init();
 	time_init();
 	sched_clock_postinit();
+	printk_nmi_init();
 	perf_event_init();
 	profile_init();
 	call_function_init();
@@ -652,7 +650,7 @@ asmlinkage __visible void __init start_kernel(void)
 	/* Should be run before the first non-init thread is created */
 	init_espfix_bsp();
 #endif
-	thread_info_cache_init();
+	thread_stack_cache_init();
 	cred_init();
 	fork_init();
 	proc_caches_init();
@@ -731,24 +729,29 @@ static int __init initcall_blacklist(char *str)
 
 static bool __init_or_module initcall_blacklisted(initcall_t fn)
 {
-	struct list_head *tmp;
 	struct blacklist_entry *entry;
-	char *fn_name;
+	char fn_name[KSYM_SYMBOL_LEN];
+	unsigned long addr;
 
-	fn_name = kasprintf(GFP_KERNEL, "%pf", fn);
-	if (!fn_name)
+	if (list_empty(&blacklisted_initcalls))
 		return false;
 
-	list_for_each(tmp, &blacklisted_initcalls) {
-		entry = list_entry(tmp, struct blacklist_entry, next);
+	addr = (unsigned long) dereference_function_descriptor(fn);
+	sprint_symbol_no_offset(fn_name, addr);
+
+	/*
+	 * fn will be "function_name [module_name]" where [module_name] is not
+	 * displayed for built-in init functions.  Strip off the [module_name].
+	 */
+	strreplace(fn_name, ' ', '\0');
+
+	list_for_each_entry(entry, &blacklisted_initcalls, next) {
 		if (!strcmp(fn_name, entry->buf)) {
 			pr_debug("initcall %s blacklisted\n", fn_name);
-			kfree(fn_name);
 			return true;
 		}
 	}
 
-	kfree(fn_name);
 	return false;
 }
 #else
@@ -809,6 +812,7 @@ int __init_or_module do_one_initcall(initcall_t fn)
 	}
 	WARN(msgbuf[0], "initcall %pF returned with %s\n", fn, msgbuf);
 
+	add_latent_entropy();
 	return ret;
 }
 
@@ -887,7 +891,6 @@ static void __init do_basic_setup(void)
 	do_ctors();
 	usermodehelper_enable();
 	do_initcalls();
-	random_int_secret_init();
 }
 
 static void __init do_pre_smp_initcalls(void)
@@ -933,6 +936,95 @@ static int try_to_run_init_process(const char *init_filename)
 
 static noinline void __init kernel_init_freeable(void);
 
+#if defined(CONFIG_DEBUG_RODATA) || defined(CONFIG_SET_MODULE_RONX)
+bool rodata_enabled __ro_after_init = true;
+static int __init set_debug_rodata(char *str)
+{
+	return strtobool(str, &rodata_enabled);
+}
+__setup("rodata=", set_debug_rodata);
+#endif
+
+#ifdef CONFIG_DEBUG_RODATA
+static void mark_readonly(void)
+{
+	if (rodata_enabled)
+		mark_rodata_ro();
+	else
+		pr_info("Kernel memory protection disabled.\n");
+}
+#else
+static inline void mark_readonly(void)
+{
+	pr_warn("This architecture does not have kernel memory protection.\n");
+}
+#endif
+
+#ifdef CONFIG_SONOS_SECBOOT
+static int get_verified_serial(uint8_t *serial, size_t serialLen)
+{
+	int result = 0;
+	SonosSignature *sig = NULL;
+
+	/*
+	 * Some of these checks may be overkill within Linux (where
+	 * sys_mdp/sys_mdp3 already got some sanity checking/scrubbing)
+	 * but this code may also end up in u-boot.
+	 */
+	if (serialLen != 6 ||
+	    sys_mdp.mdp_magic != MDP_MAGIC ||
+			sys_mdp.mdp_version < MDP_VERSION_AUTH_FLAGS ||
+			!(sys_mdp.mdp_pages_present & MDP_PAGE3_PRESENT) ||
+			sys_mdp3.mdp3_magic != MDP_MAGIC3 ||
+			sys_mdp3.mdp3_version < MDP3_VERSION_SECURE_BOOT) {
+		printk(KERN_ERR "bad MDP in get_verified_serial: %x %x %x %x %x\n",
+		       sys_mdp.mdp_magic,
+		       sys_mdp.mdp_version,
+		       sys_mdp.mdp_pages_present,
+		       sys_mdp3.mdp3_magic,
+		       sys_mdp3.mdp3_version);
+		return result;
+	}
+
+	sig = kmalloc(sizeof(*sig), GFP_KERNEL);
+	if (sig && sonosUnlockVerifyCpuSerialSig(sys_mdp.mdp_serial, 6,
+					   sys_mdp3.mdp3_cpuid_sig,
+					   sizeof sys_mdp3.mdp3_cpuid_sig,
+					   sig, sonosHash, sonosRawVerify,
+					   sonosKeyLookup, "unit", NULL)) {
+		printk(KERN_DEBUG "good cpuid/serial binding statement\n");
+		memcpy(serial, sys_mdp.mdp_serial, serialLen);
+		result = 1;
+	}
+	else {
+		printk(KERN_ERR "bad cpuid/serial binding statement\n");
+	}
+
+	if (sig) {
+		kfree(sig);
+	}
+	return result;
+}
+
+/* implement some macros needed by the portable allowlist code */
+#define SFW_GETCPUID sonos_get_cpuid
+#define SFW_GETSERIAL get_verified_serial
+#define SFW_PRINT printk
+#define SFW_PLVL_INFO KERN_INFO
+#define SFW_PLVL_EMERG KERN_EMERG
+#define SFW_BE32_TO_CPU __be32_to_cpu
+
+/* get the portable implementation of check_sonos_firmware_allowlist */
+#include "sonos_fw_allowlist.c.inc"
+
+#undef SFW_GETCPUID
+#undef SFW_GETSERIAL
+#undef SFW_PRINT
+#undef SFW_PLVL_INFO
+#undef SFW_PLVL_EMERG
+#undef SFW_BE32_TO_CPU
+#endif
+
 static int __ref kernel_init(void *unused)
 {
 	int ret;
@@ -941,11 +1033,36 @@ static int __ref kernel_init(void *unused)
 	/* need to finish all async __init code before freeing the memory */
 	async_synchronize_full();
 	free_initmem();
-	mark_rodata_ro();
+	mark_readonly();
 	system_state = SYSTEM_RUNNING;
 	numa_default_policy();
 
-	flush_delayed_fput();
+#ifdef CONFIG_SONOS_SECBOOT
+	/* Initialize keytable, check firmware allowlist, proceed with booting. */
+	sonosInitKeyTable();
+
+/*
+ * Refuse to run at all if this kernel image is restricted to some
+ * specific allowlist and we are not on it.
+ *
+ * No matter what other kernel customizations you might be tempted
+ * to do, NEVER remove this code.
+ *
+ * This check is how we implement firmware that is specific to one unit.
+ */
+	if (!check_sonos_firmware_allowlist(&SONOS_FIRMWARE_ALLOWLIST.header,
+					    sizeof(SONOS_FIRMWARE_ALLOWLIST.x))) {
+		BUG();
+	}
+
+	/*
+	 * Check the authorization for sysrq, since when we really want it, we'll
+	 * be in an interrupt and unable to...
+	 */
+	check_sysrq_authorization();
+#endif
+
+	rcu_end_inkernel_boot();
 
 	if (ramdisk_execute_command) {
 		ret = run_init_process(ramdisk_execute_command);
@@ -1030,14 +1147,6 @@ static noinline void __init kernel_init_freeable(void)
 		prepare_namespace();
 	}
 
-#ifdef CONFIG_SONOS_SECBOOT
-	/*
-	 * Check the authorization for sysrq, since when we really want it, we'll
-	 * be in an interrupt and unable to...
-	 */
-	check_sysrq_authorization();
-#endif
-
 	/*
 	 * Ok, we have completed the initial bootup, and
 	 * we're essentially up and running. Get rid of the
