@@ -79,6 +79,7 @@ void hci_conn_del_sysfs(struct hci_conn *conn)
 	hci_dev_put(hdev);
 }
 
+#ifdef CONFIG_SONOS
 static ssize_t show_adv_min(struct device *dev,
 			    struct device_attribute *attr, char *buf)
 {
@@ -103,11 +104,11 @@ static ssize_t show_adv_max(struct device *dev,
 	return sprintf(buf, "%llu\n", val);
 }
 
-static int show_conn_min_interval(struct device *dev,
+static ssize_t show_conn_min_interval(struct device *dev,
                 struct device_attribute *attr, char *buf)
 {
 	struct hci_dev *hdev = to_hci_dev(dev);
-    u64 val;
+	u64 val;
 
 	hci_dev_lock(hdev);
 	val = hdev->le_conn_min_interval;
@@ -116,11 +117,11 @@ static int show_conn_min_interval(struct device *dev,
 	return sprintf(buf, "%llu\n", val);
 }
 
-static int show_conn_max_interval(struct device *dev,
+static ssize_t show_conn_max_interval(struct device *dev,
                 struct device_attribute *attr, char *buf)
 {
 	struct hci_dev *hdev = to_hci_dev(dev);
-    u64 val;
+	u64 val;
 
 	hci_dev_lock(hdev);
 	val = hdev->le_conn_max_interval;
@@ -165,11 +166,11 @@ static ssize_t store_adv_max(struct device *dev,
 	return count;
 }
 
-static int store_conn_min_interval(struct device *dev,
+static ssize_t store_conn_min_interval(struct device *dev,
                 struct device_attribute *attr, const char *buf, size_t count)
 {
 	struct hci_dev *hdev = to_hci_dev(dev);
-    u64 val;
+	u64 val;
 
 	sscanf(buf, "%llu\n", &val);
 
@@ -184,11 +185,11 @@ static int store_conn_min_interval(struct device *dev,
 }
 
 
-static int store_conn_max_interval(struct device *dev,
+static ssize_t store_conn_max_interval(struct device *dev,
                 struct device_attribute *attr, const char *buf, size_t count)
 {
 	struct hci_dev *hdev = to_hci_dev(dev);
-    u64 val;
+	u64 val;
 
 	sscanf(buf, "%llu\n", &val);
 
@@ -210,12 +211,13 @@ static DEVICE_ATTR(conn_max_interval, S_IRUGO | S_IWUSR, show_conn_max_interval,
 static struct attribute *bt_host_attrs[] = {
 	&dev_attr_adv_min_interval.attr,
 	&dev_attr_adv_max_interval.attr,
-    &dev_attr_conn_min_interval.attr,
-    &dev_attr_conn_max_interval.attr,
+	&dev_attr_conn_min_interval.attr,
+	&dev_attr_conn_max_interval.attr,
 	NULL
 };
 
 ATTRIBUTE_GROUPS(bt_host);
+#endif
 
 static void bt_host_release(struct device *dev)
 {
@@ -226,7 +228,9 @@ static void bt_host_release(struct device *dev)
 
 static struct device_type bt_host = {
 	.name    = "host",
-    .groups = bt_host_groups,
+#ifdef CONFIG_SONOS
+	.groups = bt_host_groups,
+#endif
 	.release = bt_host_release,
 };
 
