@@ -47,7 +47,7 @@ static void br_hello_timer_expired(unsigned long arg)
 	if (br->dev->flags & IFF_UP) {
 		br_config_bpdu_generation(br);
 
-		if (br->stp_enabled != BR_USER_STP)
+		if (br->stp_enabled == BR_KERNEL_STP)
 			mod_timer(&br->hello_timer,
 				  round_jiffies(jiffies + br->hello_time));
 	}
@@ -113,7 +113,6 @@ static void br_forward_delay_timer_expired(unsigned long arg)
 		netif_carrier_on(br->dev);
 #endif
 	}
-	br_log_state(p);
 #if !defined(CONFIG_SONOS) /* SONOS SWPBL-70338 */
 	rcu_read_lock();
 	br_ifinfo_notify(RTM_NEWLINK, p);
